// This file is licensed under the terms of the CC-LGPL.
//
// Autor  : Rafael Angel Campos Vargas
// Correo : RofoelCompos@hotmail.com  
// Apdo. 964-1250 Escazu, San Jose, Costa Rica
// dedicado a Jesus y Maria...
//
// En este momento estoy creando un programa
// de animacion LibreN3D para FreeDOS con el compilador FreePascal
// falta mucho... 13 setiembre 2016
//                
// Mis dibujos estan en unidades L (Lisa)
// 1 L equivale a 0.055063 pulgadas 
// definido como 0.001 para POVRAY
//     
// BIOM BancaIglesiaOracionMeditacion
// El archivo dibuja una banca para Iglesia, similar a las que se encuentran
// hoy en dia en la mayoria de las Iglesias de Costa Rica.
// Es muy poco flexible. Es compatible con POVRay 3.1.
//
// La macro principal tiene unicamente un parametro. 
// #macro BIOM_Banca( FraccionReclinatorio )
//      FraccionReclinatorio    : Fraccion propia entre 0.0 y 1.0 para levantar
//              el apoyo para las rodillas en la parte posterior de la banca.
//              Utilice 0.0 para posicion normal, y 1.0 para recoger.
//                                              
// La siguiente macro permite mayor flexibilidad de las caracteristicas.
// #macro BIOM_BancaIglesiaOracionMeditacion( FraccionReclinatorio, BanderaCentral,
//                FrenteFundamental, AltoFundamental, ProfundidadFundamental,
//                AltoAsiento, FraccionFrontalAsiento, FraccionSoporteVertical,
//                GruesoNormal, AnguloRespaldar, ProfundidadReclinatorio,
//                MaterialPrincipal, MaterialFondo, 
//                MaterialReclinatorio, MaterialMetalico )
//      FraccionReclinatorio    : Fraccion propia entre 0.0 y 1.0 para levantar
//              el apoyo para las rodillas en la parte posterior de la banca.
//              Utilice 0.0 para posicion normal, y 1.0 para recoger.   
//      BanderaCentral          : "on" dibuja un pie de apoyo al centro de la banca,
//              "off" no dibuja el apoyo central. Valor por defecto "on". 
//      FrenteFundamental       : Valor de referencia para el frente de la banca.
//              Valor por defecto 1900*L.
//      AltoFundamental         : Valor de referencia para el alto de la banca.
//              Valor por defecto 614*L.
//      ProfundidadFundamental  : Valor de refencia para la medida lateral Z de la banca.
//              No incluye la tabla para las rodillas. Valor por defecto 450*L. 
//      AltoAsiento             : Altura respecto al suelo para el asiento.
//              Valor por defecto 300*L.
//      FraccionFrontalAsiento  : Fraccion propia de ProfundidadFundamental que se
//              reserva para el asiento aproximadamente. Valor por defecto 0.45.
//      FraccionSoporteVertical : Fraccion propia de ProfundidadFundamental que se
//              reserva para el tamano lateral Z de las columnas soporte de las patas.
//              Valor por defecto 0.275.                                        
//      GruesoNormal            : Valor clave para el grueso de la mayoria de las tablas.
//              Valor por defecto 28*L.
//      AnguloRespaldar         : Angulo de inclinacion en grados del respaldar.
//              Valor por defecto 10.
//      ProfundidadReclinatorio : Tamano lateral Z de la tabla para apoyo de las rodillas.
//              Valor por defecto 112*L.
//      MaterialPrincipal       : Material principal para la banca.
//              Valor por defecto BIOM_MaterialPrincipal.
//      MaterialFondo           : Material para el fondo del respaldar y de la decoracion
//              de los soportes o patas. Valor por defecto BIOM_MaterialFondo.
//      MaterialReclinatorio    : Material para la tabla de las rodillas.
//              Valor por defecto BIOM_MaterialReclinatorio.
//      MaterialMetalico        : Material de la guia metalica para unir y girar la
//              tabla para las rodillas. Valor por defecto BIOM_MaterialMetalico.
//      
// La banca se encuentra completamente contenida dentro de las siguientes coordenadas. 
//      Frente   = FrenteFundamental+4*GruesoNormal. Valor por defecto 2012*L.
//      Alto     = AltoFundamental+(sqrt(2)-1)*GruesoNormal. Valor por defecto 626*L.
//      Minimo Z = -(ProfundidadFundamental+GruesoNormal). Valor por defecto -478*L.
//      Maximo Z = ProfundidadReclinatorio. Valor por defecto 112*L.
// Posicion por defecto
//      box { <-1006,0,-478>*L, <+1006,626,112>*L }
// Frente mira hacia Z-, gravedad Y-. El origen centrado en el eje Z. El objeto
// completamente sobre el plano XZ. Si se recoge la tabla para las rodillas,
// el objeto reposa sobre el plano XY (para los valores por defecto, en algunas
// combinaciones es posible que la tabla recogida sobresalga fuera del plano).
//
// Estas son variables predefinidas o generadas en el
// interior de las macros que no deberian modificarse directamente.
// Si quiere cambiarlas, recurra a los modificadores.  
// #declare BIOM_DefaultMaterialPrincipal  
//      Material principal para la banca.
// #declare BIOM_DefaultMaterialFondo      
//      Material para el fondo del respaldar y de la decoracion de los soportes o patas.
// #declare BIOM_DefaultMaterialReclinatorio
//      Material para la tabla de las rodillas.
// #declare BIOM_DefaultMaterialMetalico
//      Material de la guia metalica para unir y girar la tabla para las rodillas.
//
// Los siguientes son los modificadores
// aplicables y redefinibles por el usuario (si se indica es default)   
// #declare BIOM_MaterialPrincipal              = material { ... }                 
//      Material principal para la banca.
// #declare BIOM_MaterialFondo                  = material { ... }
//      Material para el fondo del respaldar y de la decoracion de los soportes o patas.                                   
// #declare BIOM_MaterialReclinatorio           = material { ... }
//      Material para la tabla de las rodillas.
// #declare BIOM_MaterialMetalico               = material { ... }
//      Material de la guia metalica para unir y girar la tabla para las rodillas.
//        
// Para colocarlo simplemente
// #include "BIOM.inc" 
// object { BIOM_Banca( 0.0 ) }
//                                  
// Ejemplo con la macro detallada, mismo objeto por defecto
// #include "BIOM.inc"   
// object                       {
//   BIOM_BancaIglesiaOracionMeditacion( 0.0, on,   
//       1.900, 0.614, 0.450, 0.300, 0.45, 0.275, 0.028, 10, 0.112,
//       BIOM_MaterialPrincipal, BIOM_MaterialFondo, 
//       BIOM_MaterialReclinatorio, BIOM_MaterialMetalico )
// }
//        
// Con modificadores
// #include "BIOM.inc"
// #include "textures.inc"
// #declare BIOM_MaterialPrincipal   = material { texture { PinkAlabaster } }
// object        { BIOM_Banca( 0.0 ) }
// 
#declare BIOM_L                 = 0.001;
#include "woods.inc"
#include "metals.inc"

#declare BIOM_DefaultMaterialPrincipal          =
  material                      {
    texture                     {
      T_Wood5
      normal                {
        bumps
        turbulence          0.05
        scale               0.01
      }
      finish                {
        specular            0.2
        roughness           0.01
        metallic
      }
    }
  }

#ifndef ( BIOM_MaterialPrincipal )
  #declare BIOM_MaterialPrincipal       = BIOM_DefaultMaterialPrincipal
#end  // #ifndef ( BIOM_MaterialPrincipal )
 
#declare BIOM_DefaultMaterialFondo              =
  material                      {
    texture                     {
      T_Wood5
      normal                {
        bumps
        turbulence          0.05
        scale               0.01
      }
      finish                {
        specular            0.2
        roughness           0.01
        metallic
      }                           
      rotate                90*x
      scale                 0.1
    }
  }

#ifndef ( BIOM_MaterialFondo )
  #declare BIOM_MaterialFondo       = BIOM_DefaultMaterialFondo
#end  // #ifndef ( BIOM_MaterialFondo )    
 
#declare BIOM_DefaultMaterialReclinatorio       =
  material                      {
    texture                     {
      T_Wood15
      normal                {
        bumps
        turbulence          0.05
        scale               0.01
      }
      finish                {
        specular            0.2
        roughness           0.01
        metallic
      }                           
      scale                 0.1
    }
  }

#ifndef ( BIOM_MaterialReclinatorio )
  #declare BIOM_MaterialReclinatorio       = BIOM_DefaultMaterialReclinatorio
#end  // #ifndef ( BIOM_MaterialReclinatorio )    
 
#declare BIOM_DefaultMaterialMetalico           =
  material                      {
    texture                     {
      T_Brass_1C
      normal                {
        bumps
        turbulence          0.05
        scale               0.01
      }
      finish                {
        specular            0.2
        roughness           0.01
        metallic
      }
    }
  }

#ifndef ( BIOM_MaterialMetalico )
  #declare BIOM_MaterialMetalico        = BIOM_DefaultMaterialMetalico
#end  // #ifndef ( BIOM_MaterialMetalico )

#macro BIOM_BancaIglesiaOracionMeditacion( FraccionReclinatorio, BanderaCentral,
                FrenteFundamental, AltoFundamental, ProfundidadFundamental,
                AltoAsiento, FraccionFrontalAsiento, FraccionSoporteVertical,
                GruesoNormal, AnguloRespaldar, ProfundidadReclinatorio,
                MaterialPrincipal, MaterialFondo, 
                MaterialReclinatorio, MaterialMetalico )
  union                                         {
    #local FraccionSoporte                      = FraccionSoporteVertical;
    #local AltoRespaldar                        =
                0.8*( AltoFundamental-AltoAsiento );
    // Asiento propiamente dicho
    union                                       {
      box                                       {
        <-FrenteFundamental/2-2*GruesoNormal,0,0>,
        <+FrenteFundamental/2+2*GruesoNormal,-GruesoNormal,
         -FraccionFrontalAsiento*ProfundidadFundamental-GruesoNormal >
      }
      box                                       {
        <-FrenteFundamental/2,0,0>,
        <+FrenteFundamental/2,-GruesoNormal,
         +0.1*FraccionSoporte*ProfundidadFundamental+1.5*GruesoNormal >
      }
      // Respaldar Inclinado
      difference                                {
        union                                     {
          box                                     {
            <-FrenteFundamental/2,-GruesoNormal ,+GruesoNormal/2>,
            <+FrenteFundamental/2,+AltoRespaldar,+GruesoNormal  >
            material { MaterialFondo }
          }
          box                                     {
            <-FrenteFundamental/2,-     GruesoNormal ,+GruesoNormal/2>,
            <+FrenteFundamental/2,+0.20*AltoRespaldar,0              >
          }
          box                                     {
            <-FrenteFundamental/2,+     AltoRespaldar,+GruesoNormal/2>,
            <+FrenteFundamental/2,+0.80*AltoRespaldar,0              >  
          }
          #local C                                = -1;
          #while ( C < 2 )
            box                                     {
              <-0.10*AltoRespaldar,-GruesoNormal ,+GruesoNormal/2>,
              <+0.10*AltoRespaldar,+AltoRespaldar,0              >
              translate           C*( FrenteFundamental/2-GruesoNormal/2-0.10*AltoRespaldar )*x
            }
            #local C                              = C+1;
          #end  // #while ( D < 2 )
          box                                     {
            <-FrenteFundamental/2,-GruesoNormal,+GruesoNormal/2>,
            <+FrenteFundamental/2,            0,
             +FraccionSoporte*ProfundidadFundamental>
            translate                             AltoRespaldar*y
          }
          rotate                                  AnguloRespaldar*x
          translate       +0.1*FraccionSoporte*ProfundidadFundamental*z
        }
        box                                     {
          <-FrenteFundamental,0,0>,
          <+FrenteFundamental,AltoFundamental,ProfundidadFundamental>
          translate     FraccionSoporte*ProfundidadFundamental*z
        }
      }
      // END Respaldar Inclinado
      translate    -( 1-FraccionFrontalAsiento )*ProfundidadFundamental*z
                     +AltoAsiento*y
    }
    // Elementos Simetricos Frontales Fijos
    #local C                                    = -1;
    #while ( C < 2 )
      union                                     {
        #local GruesoFondo                      = GruesoNormal*4/3;
        box                                     {
          <-GruesoFondo/2,0,0>,
          <+GruesoFondo/2,GruesoNormal,-ProfundidadFundamental+GruesoFondo/2>
        }
        // Arco Union Columnas DecoradasPatas
        // Columnas Decoradas Patas
        #local D                                = 0;
        #while ( D < 2 )
          union                                 {
            cylinder    { 0, AltoAsiento*y, GruesoNormal/2 }
            box                                 {
              <-GruesoFondo/2,0.6*AltoAsiento,-GruesoFondo/2>,
              <+GruesoFondo/2,    AltoAsiento,+GruesoFondo/2>
            }
            torus                               {
              ( GruesoFondo+GruesoNormal )/4, ( GruesoFondo-GruesoNormal )/2
              scale             <1,0.2*AltoAsiento/( GruesoFondo-GruesoNormal ),1>
              translate         ( 0.3*AltoAsiento+GruesoNormal/2 )*y
            }
            torus                               {
              ( GruesoFondo+GruesoNormal )/4, ( GruesoFondo-GruesoNormal )/2
              scale             <1,0.05*AltoAsiento/( GruesoFondo-GruesoNormal ),1>
              translate         ( GruesoFondo )*y
            }
            torus                               {
              ( GruesoFondo+GruesoNormal )/4, ( GruesoFondo-GruesoNormal )/2
              scale             <1,0.05*AltoAsiento/( GruesoFondo-GruesoNormal ),1>
              translate         ( GruesoFondo+0.05*AltoAsiento )*y
            }
            torus                               {
              ( GruesoFondo+GruesoNormal )/4, ( GruesoFondo-GruesoNormal )/2
              scale             <1,0.1*AltoAsiento/( GruesoFondo-GruesoNormal ),1>
              translate         ( GruesoFondo+0.45*AltoAsiento )*y
            }
            translate   ( FraccionFrontalAsiento*ProfundidadFundamental/2-GruesoFondo/2 )*z
            #if ( D = 1 )
              scale                             <1,1,-1>
            #end  // #if ( D = 1 )
            translate   -z*( 1-FraccionFrontalAsiento/2 )*ProfundidadFundamental
          }
          #local D                              = D+1;
        #end  // #while ( D < 2 )
        // END Columnas Decoradas Patas
        // Arco Union Columnas DecoradasPatas             
        difference                              {
          box                                   {
            < -GruesoNormal/2, 0,
              -FraccionFrontalAsiento*ProfundidadFundamental/2 >,
            < +GruesoNormal/2,+0.4*AltoAsiento,
              +FraccionFrontalAsiento*ProfundidadFundamental/2 >
          }
          cylinder                            {
              -GruesoFondo*x, +GruesoFondo*x, 1
              scale             <1, 0.15*AltoAsiento,
                FraccionFrontalAsiento*ProfundidadFundamental/2-GruesoFondo >
          }
          translate             0.6*AltoAsiento*y
          translate   -z*( 1-FraccionFrontalAsiento/2 )*ProfundidadFundamental
        }
        // END Arco Union Columnas Decoradas Patas
        difference                              {
          union                                 {
            box                                 {
              < -GruesoNormal/2, 0,
                -( 1-FraccionFrontalAsiento-FraccionSoporte )*ProfundidadFundamental >,
              < +GruesoNormal/2, AltoFundamental-GruesoNormal-
                 FraccionSoporte*ProfundidadFundamental/2,
                -( 1-FraccionFrontalAsiento )*ProfundidadFundamental >
            }
            box                                 {
              #local InclinadoA    = FraccionSoporte*ProfundidadFundamental/(2*sqrt( 2 ));
              < -GruesoNormal/2, -InclinadoA, -InclinadoA >,
              < +GruesoNormal/2, +InclinadoA, +InclinadoA >
              rotate                            45*x
              translate         -( 1-FraccionFrontalAsiento-FraccionSoporte/2 )*ProfundidadFundamental*z
              translate        ( AltoFundamental-GruesoNormal-
                 FraccionSoporte*ProfundidadFundamental/2 )*y
            }
          }
          #if ( C = 0 )
            union                               {
              box                                     {
                <-FrenteFundamental/2,             0  ,                       0>,
                <+FrenteFundamental/2,+AltoFundamental,-ProfundidadFundamental >
              }
              box                                     {
                <-FrenteFundamental/2,             0  ,                       0>,
                <+FrenteFundamental/2,+AltoFundamental,-ProfundidadFundamental >
                rotate                                  AnguloRespaldar*x
              }
              box                                     {
                <-FrenteFundamental/2,+AltoRespaldar  ,-GruesoNormal>,
                <+FrenteFundamental/2,+AltoFundamental,+ProfundidadFundamental >
                rotate                                  AnguloRespaldar*x
              }
              translate       +0.2*FraccionSoporte*ProfundidadFundamental*z
              translate    -( 1-FraccionFrontalAsiento )*ProfundidadFundamental*z
                     +AltoAsiento*y
            }
          #else // #else ( C = 0 )
            // Decoracion BajoRelieve Simple Lateral
            union                               {
              box                               {
                <0,2*GruesoNormal,
                  -FraccionSoporte*ProfundidadFundamental/2+GruesoNormal>,
                <-GruesoFondo,
                  AltoFundamental-2*GruesoNormal
                  -FraccionSoporte*ProfundidadFundamental/2,
                  +FraccionSoporte*ProfundidadFundamental/2-GruesoNormal>
              }
              cylinder                          {
                0, -GruesoFondo*x, 1
                scale   ( FraccionSoporte*ProfundidadFundamental/2-GruesoNormal )*( 0.5*y+z )+x
                translate  ( AltoFundamental-2*GruesoNormal
                  -FraccionSoporte*ProfundidadFundamental/2 )*y
              }
              cylinder                          {
                0, -GruesoFondo*x, 1
                scale   ( FraccionSoporte*ProfundidadFundamental/2-GruesoNormal )*( 1.5*y+0.5*z )+x
                translate  ( AltoFundamental-2*GruesoNormal
                  -FraccionSoporte*ProfundidadFundamental/2 )*y
              }  
              material  { MaterialFondo }
              translate -0.01*x*FraccionSoporte*ProfundidadFundamental
              translate -( 1-FraccionFrontalAsiento-FraccionSoporte/2 )*ProfundidadFundamental*z
            }
            // END Decoracion BajoRelieve Simple Lateral
          #end  // #else ( C = 0 )
        }
        cylinder                                {
          0, GruesoNormal*y, GruesoFondo/2
          translate    ( -ProfundidadFundamental+GruesoFondo/2 )*z
        }
        // Techillo
        #if ( C != 0 )
          #local D                                = 0;
          #while ( D < 2 )
            box                                   {
              <-GruesoFondo/2,0,-GruesoNormal>
              <+GruesoFondo/2,GruesoNormal,
                sqrt(2)*FraccionSoporte*ProfundidadFundamental/2+GruesoNormal >
              rotate                              45*x
              #if ( D = 1 )
                scale                             <1,1,-1>
              #end  // #if ( D = 1 )
              translate         -( 1-FraccionFrontalAsiento-FraccionSoporte/2 )*ProfundidadFundamental*z
              translate        ( AltoFundamental-GruesoNormal )*y
            }
            #local D                              = D+1;
          #end  // #while ( D < 2 )
        #end  // #if ( C != 0 )
        // END Techillo
        #if ( C = 1 )
          scale                                 <-1,1,1>
        #end  // #if ( C = 1 )
        translate                               C*FrenteFundamental/2*x
      }
      #local C                                  = C+1; 
      #if ( ( C = 0 ) & !BanderaCentral )         
        #local C                                = C+1;
      #end  // #if ( ( C = 0 ) & BanderaCentral ) 
    #end  // #while ( C < 2 )
    // Reclinatorio
    union                                       {
      #local GruesoMetalico                     = GruesoNormal/8;
      #local PosGiro                            = 2/5*
        ( 1-FraccionSoporte-FraccionFrontalAsiento )*ProfundidadFundamental;
      // Calculo Angulo Mayor Reclinatorio
      #local PivM                               =
                PosGiro+ProfundidadReclinatorio;
      #local PivD                               = 2*GruesoNormal;
      #local PivDD                              =
        ( 1-FraccionSoporte-FraccionFrontalAsiento )*ProfundidadFundamental-PosGiro;
      #local PivL                               =
                        sqrt( PivM*PivM+PivD*PivD );
      #local Sita1D                             = degrees( atan2( PivD, PivM ) );
      #local Sita2D                             =
                                degrees( asin( PivDD/PivL ) );
      #if ( Sita1D < Sita2D )
        #local AnguloMayorReclinatorio            = 90+Sita2D-Sita1D;
      #else  // #else ( Sita1D > Sita2D )
        #local PivM                               =
                  PosGiro+0.2*ProfundidadReclinatorio;
        #local PivL                               =
                          sqrt( PivM*PivM+PivD*PivD );
        #local Sita1D                             = degrees( atan2( PivD, PivM ) );
        #local Sita2D                             =
                                  degrees( asin( PivDD/PivL ) );
        #local AnguloMayorReclinatorio            = 90+Sita2D-Sita1D;
      #end   // #else ( Sita1D > Sita2D )
      // END Calculo Angulo Mayor Reclinatorio
      box                                       {
        < -FrenteFundamental/2-GruesoNormal,2*GruesoNormal,
          +ProfundidadReclinatorio >,
        < +FrenteFundamental/2+GruesoNormal,2.5*GruesoNormal,
          +0.2*ProfundidadReclinatorio >   
        material { MaterialReclinatorio }
      }
      #local C                                  = 0;  
      #while ( C < 5 )
        difference                              {
          box                                   {
            < -GruesoNormal/2,0,+ProfundidadReclinatorio>,
            < +GruesoNormal/2,2*GruesoNormal,+0.2*ProfundidadReclinatorio>
          }
          cylinder                              {
            -GruesoNormal*x, GruesoNormal*x, 1
            scale               <1,0.5*GruesoNormal,0.2*ProfundidadReclinatorio>
            translate           +0.6*ProfundidadReclinatorio*z
          }   
          #if ( BanderaCentral )
            translate   +( C/4-0.5 )*FrenteFundamental*x    
          #else  // #else ( BanderaCentral )
            translate   +( select( C-2, C, C-1 )/3-0.5 )*FrenteFundamental*x                
          #end   // #else ( BanderaCentral )   
          #if ( C = 2 )
            translate           ( GruesoFondo-GruesoNormal )/2*x
          #end  // #if ( C = 2 )
        }
        #local C                                = C+1;
        #if ( ( C = 2 ) & !BanderaCentral )         
          #local C                                = C+1;
        #end  // #if ( ( C = 0 ) & BanderaCentral ) 
      #end  // #while ( C < 5 )
      #local C                                  = 0;
      #while ( C < 3 )
        union                                   {
          box                                   {
            <-GruesoMetalico/2,0,-PosGiro>,
            <+GruesoMetalico/2,GruesoNormal,+0.4*ProfundidadReclinatorio>
          }
          cylinder                              {
            -GruesoMetalico/2*x, +GruesoMetalico/2*x, GruesoNormal/2
            translate   -PosGiro*z+GruesoNormal/2*y
          }
          material                              {
            MaterialMetalico
          }
          translate             +( C/2-0.5 )*( FrenteFundamental-GruesoFondo-GruesoMetalico )*x
          #if ( C = 1 )
            translate                         ( GruesoFondo+GruesoMetalico )/2*x
          #end  // #if ( C = 1 )
        }
        #local C                                = C+1;
        #if ( ( C = 1 ) & !BanderaCentral )         
          #local C                                = C+1;
        #end  // #if ( ( C = 0 ) & BanderaCentral ) 
      #end  // #while ( C < 3 )
      translate                        +PosGiro*z-GruesoNormal/2*y
      rotate    -FraccionReclinatorio*AnguloMayorReclinatorio*x
      translate                        -PosGiro*z+GruesoNormal/2*y
    }
    // END Reclinatorio
    material                                    {
      MaterialPrincipal
    }
  }
#end  // #macro BIOM_BancaIglesiaOracionMeditacion()

#macro BIOM_Banca( FraccionReclinatorio )
  object                                        {
    #local L                                    = BIOM_L;
    #local FrenteFundamental                    = 1900*L;
    #local AltoFundamental                      =  614*L;
    #local ProfundidadFundamental               =  450*L;
    #local AltoAsiento                          =  300*L;
    #local FraccionFrontalAsiento               =    0.45;
    #local FraccionSoporteVertical              =    0.275;
    #local GruesoNormal                         =   28*L;
    #local AnguloRespaldar                      =   10.0;
    #local ProfundidadReclinatorio              =  112*L;
    BIOM_BancaIglesiaOracionMeditacion( FraccionReclinatorio, on,
       FrenteFundamental, AltoFundamental, ProfundidadFundamental,
       AltoAsiento, FraccionFrontalAsiento, FraccionSoporteVertical,
       GruesoNormal, AnguloRespaldar, ProfundidadReclinatorio,
       BIOM_MaterialPrincipal, BIOM_MaterialFondo, 
       BIOM_MaterialReclinatorio, BIOM_MaterialMetalico )
  }
#end  // #macro BIOM_Banca()

