// This file is licensed under the terms of the CC-LGPL.
//
// Autor  : Rafael Angel Campos Vargas
// Correo : RofoelCompos@hotmail.com  
// Apdo. 964-1250 Escazu, San Jose, Costa Rica
// dedicado a Jesus y Maria...
//
// En este momento estoy creando un programa
// de animacion LibreN3D para FreeDOS con el compilador FreePascal
// falta mucho... 13 setiembre 2016
//                
// Mis dibujos estan en unidades L (Lisa)
// 1 L equivale a 0.055063 pulgadas 
// definido como 0.001 para POVRAY
//     
// BIOM BancaIglesiaOracionMeditacion
// El archivo dibuja una banca para Iglesia, similar a las que se encuentran
// hoy en dia en la mayoria de las Iglesias de Costa Rica.
// Es muy poco flexible. Es compatible con POVRay 3.1.
//
// La macro principal tiene unicamente un parametro. 
// #macro BIOM_Banca( FraccionReclinatorio )
//      FraccionReclinatorio    : Fraccion propia entre 0.0 y 1.0 para levantar
//              el apoyo para las rodillas en la parte posterior de la banca.
//              Utilice 0.0 para posicion normal, y 1.0 para recoger.
//                                              
// La siguiente macro permite mayor flexibilidad de las caracteristicas.
// #macro BIOM_BancaIglesiaOracionMeditacion( FraccionReclinatorio, BanderaCentral,
//                FrenteFundamental, AltoFundamental, ProfundidadFundamental,
//                AltoAsiento, FraccionFrontalAsiento, FraccionSoporteVertical,
//                GruesoNormal, AnguloRespaldar, ProfundidadReclinatorio,
//                MaterialPrincipal, MaterialFondo, 
//                MaterialReclinatorio, MaterialMetalico )
//      FraccionReclinatorio    : Fraccion propia entre 0.0 y 1.0 para levantar
//              el apoyo para las rodillas en la parte posterior de la banca.
//              Utilice 0.0 para posicion normal, y 1.0 para recoger.   
//      BanderaCentral          : "on" dibuja un pie de apoyo al centro de la banca,
//              "off" no dibuja el apoyo central. Valor por defecto "on". 
//      FrenteFundamental       : Valor de referencia para el frente de la banca.
//              Valor por defecto 1900*L.
//      AltoFundamental         : Valor de referencia para el alto de la banca.
//              Valor por defecto 614*L.
//      ProfundidadFundamental  : Valor de refencia para la medida lateral Z de la banca.
//              No incluye la tabla para las rodillas. Valor por defecto 450*L. 
//      AltoAsiento             : Altura respecto al suelo para el asiento.
//              Valor por defecto 300*L.
//      FraccionFrontalAsiento  : Fraccion propia de ProfundidadFundamental que se
//              reserva para el asiento aproximadamente. Valor por defecto 0.45.
//      FraccionSoporteVertical : Fraccion propia de ProfundidadFundamental que se
//              reserva para el tamano lateral Z de las columnas soporte de las patas.
//              Valor por defecto 0.275.                                        
//      GruesoNormal            : Valor clave para el grueso de la mayoria de las tablas.
//              Valor por defecto 28*L.
//      AnguloRespaldar         : Angulo de inclinacion en grados del respaldar.
//              Valor por defecto 10.
//      ProfundidadReclinatorio : Tamano lateral Z de la tabla para apoyo de las rodillas.
//              Valor por defecto 112*L.
//      MaterialPrincipal       : Material principal para la banca.
//              Valor por defecto BIOM_MaterialPrincipal.
//      MaterialFondo           : Material para el fondo del respaldar y de la decoracion
//              de los soportes o patas. Valor por defecto BIOM_MaterialFondo.
//      MaterialReclinatorio    : Material para la tabla de las rodillas.
//              Valor por defecto BIOM_MaterialReclinatorio.
//      MaterialMetalico        : Material de la guia metalica para unir y girar la
//              tabla para las rodillas. Valor por defecto BIOM_MaterialMetalico.
//      
// La banca se encuentra completamente contenida dentro de las siguientes coordenadas. 
//      Frente   = FrenteFundamental+4*GruesoNormal. Valor por defecto 2012*L.
//      Alto     = AltoFundamental+(sqrt(2)-1)*GruesoNormal. Valor por defecto 626*L.
//      Minimo Z = -(ProfundidadFundamental+GruesoNormal). Valor por defecto -478*L.
//      Maximo Z = ProfundidadReclinatorio. Valor por defecto 112*L.
// Posicion por defecto
//      box { <-1006,0,-478>*L, <+1006,626,112>*L }
// Frente mira hacia Z-, gravedad Y-. El origen centrado en el eje Z. El objeto
// completamente sobre el plano XZ. Si se recoge la tabla para las rodillas,
// el objeto reposa sobre el plano XY (para los valores por defecto, en algunas
// combinaciones es posible que la tabla recogida sobresalga fuera del plano).
//
// Estas son variables predefinidas o generadas en el
// interior de las macros que no deberian modificarse directamente.
// Si quiere cambiarlas, recurra a los modificadores.  
// #declare BIOM_DefaultMaterialPrincipal  
//      Material principal para la banca.
// #declare BIOM_DefaultMaterialFondo      
//      Material para el fondo del respaldar y de la decoracion de los soportes o patas.
// #declare BIOM_DefaultMaterialReclinatorio
//      Material para la tabla de las rodillas.
// #declare BIOM_DefaultMaterialMetalico
//      Material de la guia metalica para unir y girar la tabla para las rodillas.
//
// Los siguientes son los modificadores
// aplicables y redefinibles por el usuario (si se indica es default)   
// #declare BIOM_MaterialPrincipal              = material { ... }                 
//      Material principal para la banca.
// #declare BIOM_MaterialFondo                  = material { ... }
//      Material para el fondo del respaldar y de la decoracion de los soportes o patas.                                   
// #declare BIOM_MaterialReclinatorio           = material { ... }
//      Material para la tabla de las rodillas.
// #declare BIOM_MaterialMetalico               = material { ... }
//      Material de la guia metalica para unir y girar la tabla para las rodillas.
//        
// Para colocarlo simplemente
// #include "BIOM.inc" 
// object { BIOM_Banca( 0.0 ) }
//                                  
// Ejemplo con la macro detallada, mismo objeto por defecto
// #include "BIOM.inc"   
// object                       {
//   BIOM_BancaIglesiaOracionMeditacion( 0.0, on,   
//       1.900, 0.614, 0.450, 0.300, 0.45, 0.275, 0.028, 10, 0.112,
//       BIOM_MaterialPrincipal, BIOM_MaterialFondo, 
//       BIOM_MaterialReclinatorio, BIOM_MaterialMetalico )
// }
//        
// Con modificadores
// #include "BIOM.inc"
// #include "textures.inc"
// #declare BIOM_MaterialPrincipal   = material { texture { PinkAlabaster } }
// object        { BIOM_Banca( 0.0 ) }
// 
#if ( version >= 3.7 )
  #version 3.7; 
#end

global_settings { assumed_gamma 1.0 }

#include "colors.inc"

#declare OjoAbsoluto            = 256;
#declare VecesCentroAbsoluto    = 10;     
#declare L                      = 0.001;
camera {
        location  <   0,  0, -( VecesCentroAbsoluto+1 )*OjoAbsoluto >*L
        direction z
        sky       y
        angle     38.74
//        look_at  <   0,  0, -VecesCentroAbsoluto*OjoAbsoluto >*L
}         
light_source    {
         < 500  , 500   , -3000 >*L
         White
}
background      {
         Gray50
}    

#include "BIOM.inc"    
  
union                                   { 
  #local C                              = -1.50; 
  #while ( C <= 1.50 )
    object                                {
      BIOM_Banca( 0.0 )                        
      translate                           C*z
    }                                      
    #local C                            = C+0.75;
  #end  // #while ( C <= 1.50 )
  rotate                                90*y                                   
  translate                             -0.31*y+0.8*z     
}
