// This file is licensed under the terms of the CC-LGPL.
//
// Autor  : Rafael Angel Campos Vargas
// Correo : RofoelCompos@hotmail.com  
// Apdo. 964-1250 Escazu, San Jose, Costa Rica
// dedicado a Jesus y Maria...
//
// En este momento estoy creando un programa
// de animacion LibreN3D para FreeDOS con el compilador FreePascal
// falta mucho... 19 abril 2016
//                
// Mis dibujos estan en unidades L (Lisa)
// 1 L equivale a 0.055063 pulgadas 
// definido como 0.001 para POVRAY
//     
// CAMD CaminadoraAereaMayoresDoce
// El archivo dibuja una caminadora aerea, similar a las que
// se encuentran actualmente en algunos parques publicos de Costa Rica.
//
// La macro principal tiene dos parametros. 
// #macro CAMD_Caminadora( AnguloPieDerecho, AnguloPieIzquierdo )   
//      AnguloPieDerecho        : Angulo de rotacion para el soporte de pie derecho de
//              usuario. Valores positivos giran hacia el frente del aparato.
//      AnguloPieIzquierdo      : Angulo de rotacion para el soporte de pie izquierdo de
//              usuario. Valores positivos giran hacia el frente del aparato.
//
// La siguiente macro permite modificar algunas caracteristicas.
// #macro CAMD_CaminadoraAereaMayoresDoce( AnguloPieDerecho, AnguloPieIzquierdo,
//                AltoFundamental, FrenteFundamental, ProfundidadFundamental,
//                AnguloAgarradera, DiametroSoporte, DiametroFundamental, 
//                SobreSuelo, CurvaturaSoporte,
//                TexturaPrincipal, TexturaDinamica,
//                TexturaSoporte, TexturaDecorativa, TexturaSujetador )
//      AnguloPieDerecho        : Angulo de rotacion para el soporte de pie derecho de
//              usuario. Valores positivos giran hacia el frente del aparato.
//      AnguloPieIzquierdo      : Angulo de rotacion para el soporte de pie izquierdo de
//              usuario. Valores positivos giran hacia el frente del aparato.  
//      AltoFundamental         : Alto de referencia para la caminadora.
//              Valor por defecto 1000*L.
//      FrenteFundamental       : Valor de referencia para el frente X de la caminadora.
//              Valor por defecto 670*L.
//      ProfundidadFundamental  : Valor de referencia para la profundidad Z de la caminadora.
//              Valor por defecto 250*L.
//      AnguloAgarradera        : Angulo de inclinacion en grados para el soporte 
//              de las manos.  Valor por defecto 25.   
//      DiametroSoporte         : Diametro de los cilindros principales de la estructura.
//              Valor por defecto 100*L.
//      DiametroDinamico        : Diametro para los cilindros movibles de la estructura.
//              Valor por defecto 34*L.
//      SobreSuelo              : Elevacion sobre el suelo del cilindro soporte.
//              Valor por defecto 100*L.
//      CurvaturaSoporte        : Valor para la curvatura como radio de los arcos
//              en las laterales.  Valor por defecto 150*L. 
//      TexturaPrincipal        : Textura principal a aplicar en la estructura.
//              Valor por defecto CAMD_TexturaPrincipal.
//      TexturaDinamica         : Textura de la caminadora aerea propiamente.
//              Valor por defecto CAMD_TexturaDinamica.
//      TexturaSoporte          : Textura de la base inferior.
//              Valor por defecto CAMD_TexturaSoporte.
//      TexturaDecorativa       : Textura para decoracion de los extremos del soporte.
//              Valor por defecto CAMD_TexturaDecorativa.
//      TexturaSujetador        : Textura de los aros metalicos para sujetar
//              la caminadora a la estructura.  Valor por defecto CAMD_TexturaSujetador.
//      
// La caminadora con parametros nulos se encuentra completamente contenida
// en las siguientes coordenadas.
//      Minimo  = < -336,    0, -267 >*L
//      Maximo  = <  336, 1153,  105 >*L  
// Frente mira hacia Z-, gravedad Y-. El cilindro central alineado con el origen del
// plano XZ.
//
// Estas son variables predefinidas o generadas en el
// interna de las macros que no deberian modificarse directamente.
// Si quiere cambiarlas, recurra a los modificadores.  
// #declare CAMD_DefaultTexturaPrincipal        
//      Textura principal a aplicar en la estructura.
// #declare cAMD_DefaultTexturaDinamica         
//      Textura de la caminadora aerea propiamente.
// #declare CAMD_DefaultTexturaSoporte          
//      Textura de la base inferior.
// #declare CAMD_DefaultTexturaDecorativa       
//      Textura para decoracion de los extremos del soporte.
// #declare CAMD_DefaultTexturaSujetador        
//      Textura de los aros metalicos para sujetar
//      la caminadora a la estructura.  
//
// Los siguientes son los modificadores
// aplicables y redefinibles por el usuario (si se indica es default)   
// #declare CAMD_TexturaPrincipal                   = texture { ... }        
//      Textura principal a aplicar en la estructura.
// #declare cAMD_TexturaDinamica                    = texture { ... }         
//      Textura de la caminadora aerea propiamente.
// #declare CAMD_TexturaSoporte                     = texture { ... }
//      Textura de la base inferior.
// #declare CAMD_TexturaDecorativa                  = texture { ... }
//      Textura para decoracion de los extremos del soporte.
// #declare CAMD_TexturaSujetador                   = texture { ... }
//      Textura de los aros metalicos para sujetar 
//      la caminadora a la estructura.  
//        
// Para colocarlo simplemente
// #include "CAMD.inc" 
// object { CAMD_Caminadora(0,0)
//                                  
// Ejemplo con la macro detallada, mismo objeto default
// #include "CAMD.inc"   
// object                       {
//   CAMD_CaminadoraAereaMayoresDoce( 0, 0, 
//              1.000, 0.670, 0.250, 25, 0.100, 0.034, 0.100, 0.150,
//              CAMD_TexturaPrincipal, CAMD_TexturaDinamica,
//              CAMD_TexturaSoporte, CAMD_TexturaDecorativa, CAMD_TexturaSujetador )
// }
//        
// Con modificadores
// #include "CAMD.inc"
// #include "textures.inc"
// #declare CAMD_TexturaPrincipal   = PinkAlabaster
// object        { CAMD_Caminadora(0,0) }
//
#include "colors.inc"
#include "metals.inc" 
#include "stones.inc"
#include "textures.inc"

#declare CAMD_L                         = 0.001;

#if ( version < 3.2 )
  #ifndef( CAMD_POV3_1 )
    #declare CAMD_POV3_1        = on;
  #end
#else
  #ifndef( CAMD_POV3_1 )
    #declare CAMD_POV3_1        = off;
  #end
#end

#declare CAMD_DefaultTexturaPrincipal   =
    texture                     {                         
      T_Chrome_2C    
      normal                              { 
        ripples               
        turbulence        1
        scale             1
      } 
    }  


#ifndef ( CAMD_TexturaPrincipal )
  #declare CAMD_TexturaPrincipal        = CAMD_DefaultTexturaPrincipal
#end  // #ifndef ( CAMD_TexturaPrincipal )

#declare CAMD_DefaultTexturaDinamica   =
    texture                     {                         
      pigment { color ( 2*Blue+Cyan )/3 }
      normal                              { 
        ripples               
        turbulence        0.2
        scale             0.025
      } 
    }  


#ifndef ( CAMD_TexturaDinamica )
  #declare CAMD_TexturaDinamica        = CAMD_DefaultTexturaDinamica
#end  // #ifndef ( CAMD_TexturaDinamica )

#declare CAMD_DefaultTexturaSoporte   =
    texture                     {                         
      T_Stone31
      scale 0.01
    }  


#ifndef ( CAMD_TexturaSoporte )
  #declare CAMD_TexturaSoporte        = CAMD_DefaultTexturaSoporte
#end  // #ifndef ( CAMD_TexturaSoporte )

#declare CAMD_DefaultTexturaDecorativa   =
    texture                     {                         
      pigment  { color ( 2*Blue+Cyan )/3 }
      normal                              { 
        ripples               
        turbulence        0.2
        scale             0.025
      } 
    }  


#ifndef ( CAMD_TexturaDecorativa )
  #declare CAMD_TexturaDecorativa        = CAMD_DefaultTexturaDecorativa
#end  // #ifndef ( CAMD_TexturaDecorativa )

#declare CAMD_DefaultTexturaSujetador   =
    texture                     {                         
      Silver3
      normal                              { 
        ripples               
        turbulence        0.2
        scale             0.025
      } 
    }  


#ifndef ( CAMD_TexturaSujetador )
  #declare CAMD_TexturaSujetador        = CAMD_DefaultTexturaSujetador
#end  // #ifndef ( CAMD_TexturaSujetador )


#macro CAMD_CaminadoraAereaMayoresDoce( AnguloPieDerecho, AnguloPieIzquierdo,
                AltoFundamental, FrenteFundamental, ProfundidadFundamental,
                AnguloAgarradera, DiametroSoporte, DiametroDinamico, 
                SobreSuelo, CurvaturaSoporte,
                TexturaPrincipal, TexturaDinamica,
                TexturaSoporte, TexturaDecorativa, TexturaSujetador )
  union                                 {
    #local AireLateral                  = DiametroSoporte/3;
    #local AireCentral                  = DiametroSoporte;
    #local SeparacionBarrasCaminadora   =
                FrenteFundamental-2*DiametroSoporte-2*AireLateral-DiametroDinamico;
    #local AlturaEjeCaminadora          = AltoFundamental
                -ProfundidadFundamental*tan( radians( AnguloAgarradera ) );
    // Estructura Base
    union                               {
      cylinder                          {
        0, ( SobreSuelo+DiametroSoporte/2 )*y, DiametroSoporte/2
      }
      difference                        {
        torus                           {
          DiametroSoporte/2, DiametroSoporte/2
        }
        plane                           {
          y, 0
        }
        texture                         {
          TexturaSoporte
        }
      }
      cylinder                          {
        -( FrenteFundamental/2-CurvaturaSoporte )*x,
        +( FrenteFundamental/2-CurvaturaSoporte )*x,
        DiametroSoporte/2
        translate               ( DiametroSoporte/2+SobreSuelo )*y
      }
      #local C                          = 0;
      #while ( C < 2 )
        // Simetrico del Soporte
        union                           {
          difference                  {
            torus                     {
              ( CurvaturaSoporte-DiametroSoporte/2 ),
              DiametroSoporte/2
              rotate          90*x
            }
            plane                     {
              -y, 0
            }
            plane                     {
              x, 0
            }
            translate    < FrenteFundamental/2-CurvaturaSoporte,
                              SobreSuelo+CurvaturaSoporte, 0 >
          }
          cylinder                      {
            ( SobreSuelo+CurvaturaSoporte )*y, 1.1*AltoFundamental*y,
            DiametroSoporte/2
            translate           ( FrenteFundamental/2-DiametroSoporte/2 )*x
          }
          cone                          {
            ( FrenteFundamental/2-DiametroSoporte/2 )*x, DiametroSoporte/2,
            ( FrenteFundamental/2-DiametroSoporte-AireLateral/2 )*x,
                        1.1*DiametroDinamico/2
            translate           AlturaEjeCaminadora*y
          }
          difference                    {
            sphere                      {
              0, 1.05*DiametroSoporte/2
            }
            plane                       {
              y, 0
            }
            translate  < FrenteFundamental/2-DiametroSoporte/2,
                         1.1*AltoFundamental, 0 >
            texture                     {
              TexturaDecorativa
            }
          }
          // Agarradera
          difference                    {
            union                       {
              cylinder                  {
                0, ( FrenteFundamental/2-DiametroSoporte/2-CurvaturaSoporte
                        +DiametroDinamico/2 )*x,
                DiametroDinamico/2
              }
              difference                {
                torus                   {
                  CurvaturaSoporte-DiametroDinamico/2,
                  DiametroDinamico/2
                }
                plane                   {
                  x, 0
                }
                plane                   {
                  -z, 0
                }
                translate        < FrenteFundamental/2
                 -DiametroSoporte/2-CurvaturaSoporte+DiametroDinamico/2,
                 0, CurvaturaSoporte-DiametroDinamico/2 >
              }
              cylinder                  {
                ( CurvaturaSoporte-DiametroDinamico/2 )*z,
                ProfundidadFundamental*2*z,
                DiametroDinamico/2
                translate       ( FrenteFundamental/2-DiametroSoporte/2 )*x
              }
              rotate            AnguloAgarradera*x
              translate       ( AltoFundamental-DiametroDinamico/2 )*y
              translate       -ProfundidadFundamental*z
            }
            plane                       {
              -z, 0
            }
          }
          #if ( C = 1 )
            scale                       <-1,1,1>
          #end  // #if ( C = 1 )
        }
        #local C                        = C+1;
      #end  // #while ( C < 2 )
    }
    // Caminadora Propiamente
    // Definamos el Pie
    #local CurvaturaInferior        = 2*DiametroDinamico;
    #local LargoBarra               =
           AlturaEjeCaminadora-SobreSuelo-DiametroSoporte-AireLateral
           -CurvaturaInferior+DiametroDinamico/2;
    #local FrentePie                    =
            SeparacionBarrasCaminadora/2-AireCentral/2
            -CurvaturaInferior+DiametroDinamico/2;
    #local AltoPie                      = DiametroDinamico/2;
    #local ProfundidadPie               = 2.5*FrentePie;
    #local Pie                          =
      union                             {
        difference                      {
          superellipsoid                  {
            <0.2,0.2>
          }
          superellipsoid                  {
            <0.2,0.2>
            scale 0.95
          }
          box                           {
            < -0.5, 2, 0.75 >,
            <  0.5,-2, 0.80 >
          }
          box                           {
            < -0.6, 2, 0.25 >,
            <  0.6,-2, 0.30 >
          }
          box                           {
            < -0.6, 2,-0.25 >,
            <  0.6,-2,-0.30 >
          }
          box                           {
            < -0.5, 2,-0.75 >,
            <  0.5,-2,-0.80 >
          }
          plane                           {
            -y, 0
          }
        }
        cylinder                        {
          -0.9*z, 0.9*z, 0.2
          translate             -y-0.8*x
        }
        cylinder                        {
          -0.9*z, 0.9*z, 0.2
          translate             -y
        }
        cylinder                        {
          -0.9*z, 0.9*z, 0.2
          translate             -y+0.8*x
        }
        scale          < FrentePie/2, AltoPie, ProfundidadPie/2 >
      }
    #local C                            = 0;
    #while ( C < 2 )
      // Un ciclo para cada pie
      union                             {
        // Union con soporte
        cylinder                        {
          -AireLateral/2*x, +AireLateral*x, DiametroDinamico/2
        }
        difference                      {
          union                         {
            torus                         {
              DiametroDinamico/2, DiametroDinamico/2
              rotate              90*z
            }
            sphere                      {
              0, DiametroDinamico/2
            }
          }
          plane                         {
            -x, 0
          }
          translate             -AireLateral/2*x
          texture                       {
            TexturaSujetador
          }
        }
        cylinder                        {
          0, -LargoBarra*y, DiametroDinamico/2
        }
        difference                      {
          torus                           {
            CurvaturaInferior-DiametroDinamico/2,
            DiametroDinamico/2
            rotate                        90*x
          }
          plane                         {
            x, 0
          }
          plane                         {
            -y, 0
          }
          translate     < -CurvaturaInferior+DiametroDinamico/2, -LargoBarra, 0 >
        }
        cylinder                        {
          ( -CurvaturaInferior+DiametroDinamico/2 )*x,
          ( -SeparacionBarrasCaminadora/2+AireCentral/2 )*x,
          DiametroDinamico/2
          translate     ( -LargoBarra-CurvaturaInferior+DiametroDinamico/2 )*y
        }
        object                          {
          Pie
          translate     ( -SeparacionBarrasCaminadora/2
                                +AireCentral/2+FrentePie/2 )*x
          translate     ( -LargoBarra-CurvaturaInferior+DiametroDinamico+
                                AltoPie/2 )*y
        }
        translate               SeparacionBarrasCaminadora/2*x
        #if ( C = 0 )
          rotate                        AnguloPieIzquierdo*x
        #else  // #else ( C = 0 )
          scale                         < -1, 1, 1 >
          rotate                        AnguloPieDerecho*x
        #end   // #else ( C = 0 )
        translate                    AlturaEjeCaminadora*y
        texture { TexturaDinamica }
      }
      #local C                          = C+1;
    #end  // #while ( C < 2 )
    texture                             {
      TexturaPrincipal
    }
  }
#end  // #macro CAMD_CaminadoraParaMayoresDoce

#macro CAMD_Caminadora( AnguloPieDerecho, AnguloPieIzquierdo )
  object                                {
    #local L                            = CAMD_L;
    #local AltoFundamental              = 1000*L;
    #local FrenteFundamental            =  670*L;
    #local ProfundidadFundamental       =  250*L;
    #local AnguloAgarradera             =   25;
    #local DiametroSoporte              =  100*L;
    #local DiametroDinamico             =   34*L;
    #local SobreSuelo                   =  100*L;
    #local CurvaturaSoporte             =  150*L;
    CAMD_CaminadoraAereaMayoresDoce( AnguloPieDerecho, AnguloPieIzquierdo,
                AltoFundamental, FrenteFundamental, ProfundidadFundamental,
                AnguloAgarradera,
                DiametroSoporte, DiametroDinamico, SobreSuelo,
                CurvaturaSoporte,
                CAMD_TexturaPrincipal, CAMD_TexturaDinamica,
                CAMD_TexturaSoporte, CAMD_TexturaDecorativa,
                CAMD_TexturaSujetador )
  }
#end  // #macro CAMD_Caminadora
