// This file is licensed under the terms of the CC-LGPL.
//
// Autor  : Rafael Angel Campos Vargas
// Correo : RofoelCompos@hotmail.com  
// Apdo. 964-1250 Escazu, San Jose, Costa Rica
// dedicado a Jesus y Maria...
//
// En este momento estoy creando un programa
// de animacion LibreN3D para FreeDOS con el compilador FreePascal
// falta mucho... 15 marzo 2016
//                
// Mis dibujos estan en unidades L (Lisa)
// 1 L equivale a 0.055063 pulgadas 
// definido como 0.001 para POVRAY
//     
// CUMP CortaUnasManoPie
// El archivo dibuja un cortauas para manos y pies corriente.
// Es muy poco flexible. Es compatible con POVRay 3.1.
//
// La macro principal tiene tres parametros. 
// #macro CUMP_CortaUnas( FraccionPrensa, AnguloLima, AnguloPrensa ) 
//      FraccionPrensa          : Fraccion propia entre -1.0 a 1.0. Valores positivos para prensar
//              el cortaunas, valores negativos para cerrarlo. -1.0 es completamente cerrado,
//              y 1.0 es presion maxima. Para cerrar complemente el cortaunas, deberia
//              asignar un valor 180 a AnguloPrensa.
//      AnguloLima              : Angulo para girar la lima de unas.
//      AnguloPrensa            : Angulo para girar la prensa cortante.
//
// La siguiente macro permite modificar las texturas desde los parametros.
// #macro CUMP_CortaUnasManoPie( FraccionPrensa, AnguloLima, AnguloPrensa,
//                  TexturaPrincipal, TexturaLima, TexturaPrensa )
//      FraccionPrensa          : Fraccion propia. Valores positivos para prensar
//              el cortaunas, valores negativos para cerrarlo. -1.0 es completamente cerrado,
//              y 1.0 es presion maxima. Para cerrar complemente el cortaunas, deberia
//              asignar un valor 180 a AnguloPrensa.
//      AnguloLima              : Angulo para girar la lima de unas.
//      AnguloPrensa            : Angulo para girar la prensa cortante. 
//      TexturaPrincipal        : Textura para aplicar al cuerpo principal.
//              Valor por defecto CUMP_TexturaPrincipal.
//      TexturaLima             : Textura para la lima.
//              Valor por defecto CUMP_TexturaLima.
//      TexturaPrensa           : Textura para la prensa de presion cortante.
//              Valor por defecto CUMP_TexturaPrensa.
//      
// El cortaunas cerrado con parametros (-1,0,180) se encuentra completamente contenido
// en las siguientes coordenadas.
//      Minimo  = <  -5,  -5,  -6 >*L
//      Maximo  = <  53,   8,   6 >*L  
// Frente mira hacia X+, gravedad Y-. El origen se ubica en una perforacion cilindrica
// que suele tener un collar metalico para colgar.
//
// Estas son variables predefinidas o generadas en el
// interna de las macros que no deberian modificarse directamente.
// Si quiere cambiarlas, recurra a los modificadores.  
// #declare CUMP_DefaultTexturaPrincipal  
//      Textura para aplicar al cuerpo principal.
// #declare CUMP_DefaultTexturaLima      
//      Textura para la lima.
// #declare CUMP_DefaultTexturaPrensa           
//      Textura para la prensa de presion cortante.
//
// Los siguientes son los modificadores
// aplicables y redefinibles por el usuario (si se indica es default)   
// #declare CUMP_TexturaPrincipal               = texture { ... }                 
//      Textura para aplicar al cuerpo principal.
// #declare CUMP_TexturaLima                    = texture { ... }
//      Textura para la lima.                                   
// #declare CUMP_TexturaPrensa                  = texture { ... }
//      Textura para la prensa de presion cortante.
//        
// Para colocarlo simplemente
// #include "CUMP.inc" 
// object { CUMP_CortaUnas(0,0,0)
//                                  
// Ejemplo con la macro de texturas, mismo objeto default
// #include "CUMP.inc"   
// object                       {
//   CUMP_CortaUnasManoPie(0,0,0,
//      CUMP_TexturaPrincipal,CUMP_TexturaLima,CUMPT_TexturaPrensa)
// }
//        
// Con modificadores
// #include "CUMP.inc"
// #include "textures.inc"
// #declare CUMP_TexturaPrincipal   = PinkAlabaster
// object        { CUMP_CortaUnas(0,0,0) }
// 
#include "colors.inc"
#include "metals.inc"

#declare CUMP_L                         = 0.001;

#if ( version < 3.2 )
  #ifndef( CUMP_POV3_1 )
    #declare CUMP_POV3_1        = on;
  #end
#else
  #ifndef( CUMP_POV3_1 )
    #declare CUMP_POV3_1        = off;
  #end
#end

#declare CUMP_DefaultTexturaPrincipal   =
  texture                               { 
    checker   
    texture                             {   
      gradient x
      scale     0.025 
      sine_wave
      texture_map                       {  
        [ 0.0 T_Chrome_1A ]
        [ 1.0 T_Chrome_2D ]
      }                     
      scale         0.1
    } 
    texture                     {                         
      T_Chrome_2C    
      normal                              { 
        ripples               
        turbulence        0.2
        scale             0.1
      } 
    }  
    translate               -0.5*z-0.5*y 
    rotate                  0*y
    scale                       0.45
    translate               0.0*x
  }


#ifndef ( CUMP_TexturaPrincipal )
  #declare CUMP_TexturaPrincipal        = CUMP_DefaultTexturaPrincipal
#end  // #ifndef ( CUMP_TexturaPrincipal )

#declare CUMP_DefaultTexturaLima        =
  texture                               { 
    checker   
    texture                             {
      #if ( CUMP_POV3_1 )
        quilted
      #else
        tiling 5
      #end
      texture_map                       {  
        [ 0.0 T_Chrome_1A ]
        [ 1.0 T_Chrome_5E ]
      }                     
      scale         0.01
    }                          
    texture                             {
      T_Chrome_1E    
      normal                              { 
        ripples               
        turbulence        0.2
        scale             0.25
      }                                    
    }
    translate               -0.5*z 
    rotate                  20*y
    scale                       0.45
    translate               0.20*x
  }

#ifndef ( CUMP_TexturaLima )
  #declare CUMP_TexturaLima             = CUMP_DefaultTexturaLima
#end  // #ifndef ( CUMP_TexturaLima )

#declare CUMP_DefaultTexturaPrensa      =  
  #if ( CUMP_POV3_1 )  
  texture                               {
    T_Chrome_3C    
    normal                              { 
      ripples               
      turbulence        0.2
      scale             0.1
    }
  }
  #else
  texture                               {
    T_Chrome_3C    
    normal                              { 
      ripples               
      turbulence        0.2
      scale             0.1
    }
  }
  texture                               {   
    pigment                             {
      object                              {  
        text { ttf "timrom.ttf", "POVRay", 100, 0 rotate  -90*x scale <0.1,1,0.35> translate +0.45*x }
        color Clear color Gray25   
      }                                    
    }  
  }
  #end

#ifndef ( CUMP_TexturaPrensa )
  #declare CUMP_TexturaPrensa           = CUMP_DefaultTexturaPrensa
#end  // #ifndef ( CUMP_TexturaPrensa )

#macro CUMP_CortaUnasManoPie( FraccionPrensa, AnguloLima, AnguloPrensa,
        TexturaPrincipal, TexturaLima, TexturaPrensa )
  union                                 {
    #local FraccionApertura             = 1-abs( FraccionPrensa );
    #local L                            = CUMP_L;
    #local EjeAgarre                    = 4.82*L;
    #local Sellado                      = 10.78*L;
    #local LargoLiso                    = 56.61*L;
    // Largo de la zona relativamente plana en la base
    #local LargoTotal                   = 57.88*L;
    #local AnchoMayor                   = 11.35*L;
    #local AnchoInicial                 = 8.80*L;
    #local AnchoFinal                   = 10.2*L;
    #local AnchoCortante                = 9.08*L;
    #local RadioUna                     = 15.89*L;
    #local CorreccionUna          =
            sqrt( RadioUna*RadioUna-0.25*AnchoCortante*AnchoCortante );
    #local GruesoMetal                  = 1.14*L;
    #local AperturaMaxima               = 0.5*7.24*L;
    #local AperturaMinima               = 0.5*5.39*L;
    #local DeltaXPrensa                 = 44.41*L;
    #local DeltaYPrensa                 = 26.67*L;
    #local DeltaZPrensa                 = 9.94*L;
    #local AnguloRecPrensa              =
        180-0.975*degrees( atan2( DeltaYPrensa-GruesoMetal, DeltaXPrensa ) );
    #local PivPrensa                    = 5.68*L;
    #local ExcPivPrensa                 = 0.1*PivPrensa;
    #if ( FraccionPrensa < 0 )
      #local AnguloP                    = -FraccionPrensa*AnguloRecPrensa;
      #if ( AnguloP < 90 )
        #local AperturaTope             =
                        AperturaMaxima-AnguloP/90*ExcPivPrensa;
      #else  // #else ( AnguloP < 90 )
        #local AperturaTope             =
                        AperturaMaxima
           -( AnguloRecPrensa-AnguloP )/( AnguloRecPrensa-90 )*ExcPivPrensa;
      #end   // #else ( AnguloP < 90 )
      #local FraccionApertura           =
                ( AperturaTope-AperturaMinima )/
                ( AperturaMaxima-AperturaMinima );
    #else  // #else ( FraccionApertura < 0 )
      #local AperturaTope                 = AperturaMinima+
                FraccionApertura*( AperturaMaxima-AperturaMinima );
    #end   // #else ( FraccionApertura < 0 )
    #local RadioAire                    = 0.5*2.48*L;
    #local RadioTorusCompleto           = 0.5*4.54*L;
    #local Upper                        = 100*L;
    // Upper es un valor en exceso para cortes e intersecciones
    // Si fallara simplemente aumente su valor
    // Cuerpo Principal
    union                               {
      // Diferencia para la boca Curva
      difference                        {
        intersection                    {
          // Este prism es la vista arriba-abajo (aproximada constante)
          prism                         {
            linear_spline
            -Upper, Upper, 8,
            < -EjeAgarre, -AnchoInicial/2 >,
            < -EjeAgarre+3*LargoLiso/4, -AnchoMayor/2 >,
            < -EjeAgarre+LargoLiso, -AnchoFinal/2 >,
            < -EjeAgarre+LargoTotal, -AnchoCortante/2 >,
            < -EjeAgarre+LargoTotal, +AnchoCortante/2 >,
            < -EjeAgarre+LargoLiso, +AnchoFinal/2 >,
            < -EjeAgarre+3*LargoLiso/4, +AnchoMayor/2 >,
            < -EjeAgarre, +AnchoInicial/2 >
          }
          // Curvatura posterior de la vista arriba-abajo
          union                         {
            cylinder                    {
              -0.999*Upper*y, 0.999*Upper*y, LargoTotal*0.3
              translate           ( LargoTotal*0.3-EjeAgarre )*x
            }
            box                         {
              <0,-1,-1>,
              <1,1,1>
              scale                     0.998*Upper
            }
          }
          union                         {
            // Pareja simetrica arriba-abajo
            #local C                    = 0;
            #while ( C < 2 )
              union                     {
                // Sellado
                superellipsoid          {
                  < 0.2, 0.2 >
                  scale      < Sellado/2, GruesoMetal/2, 1.15*AnchoInicial/2 >
                  translate         -GruesoMetal/2*y
                  translate         ( -EjeAgarre+Sellado/2 )*x
                }
                cylinder                {
                  -Upper*z, Upper*z, GruesoMetal/2
                  translate         -GruesoMetal/2*y
                  translate         ( -EjeAgarre+Sellado )*x
                }
                // Prisma Lateral principal
                prism                   {
                  -Upper, Upper, 6,
                  <-EjeAgarre+Sellado,GruesoMetal>,
                  <-EjeAgarre+LargoLiso,AperturaTope>,
                  <-EjeAgarre+LargoTotal,AperturaTope-AperturaMinima>,
                  <-EjeAgarre+LargoTotal+CorreccionUna-RadioUna,
                        AperturaTope-AperturaMinima>,
                  <-EjeAgarre+LargoLiso,AperturaTope-GruesoMetal>,
                  <-EjeAgarre+Sellado,0>
                  rotate                90*x
                }
                #if ( C = 0 )
                  scale                 <1,-1,1>
                #end  // #if ( C = 0 )
              }
              #local C                  = C+1;
            #end  // #while ( C < 2 )
          }
        }
        // Eliminando la curvatura de Filo
        cylinder                        {
          -1.001*Upper*y, +1.001*Upper*y, RadioUna
          translate       ( -EjeAgarre+CorreccionUna+LargoTotal )*x
        }
        cylinder                        {
          -1.001*Upper*y, +1.001*Upper*y, RadioAire
        }
      }
      torus                             {
        ( RadioAire+RadioTorusCompleto )/2, ( RadioTorusCompleto-RadioAire )/2
        translate                       -GruesoMetal*y
      }
    }
    // Lima
    #local GruesoLima                   = 0.71*L;
    #local AgarreLima                   = 7.80*L;
    #local LargoExtraLima               = 37.17*L;
    #local AnchoLima                    = 7.52*L;
    #local AltoExtremoLima              =
                ( AgarreLima/2+LargoExtraLima-( Sellado-EjeAgarre ) )/
                        ( LargoLiso-Sellado )*( AperturaTope-GruesoMetal );
    #local AnguloLimaElevacion          =
        degrees( asin( AltoExtremoLima/( AgarreLima/2+LargoExtraLima ) ) );
    union                               {
      difference                        {
        union                           {
          prism                         {
            0, GruesoLima, 12,
            <-AgarreLima/2,-AnchoLima/2>,
            <+AgarreLima/2+0.0*LargoExtraLima,-AnchoLima/2>,
            <+AgarreLima/2+0.1*LargoExtraLima,-0.5*AnchoLima/2>,
            <+AgarreLima/2+0.3*LargoExtraLima,-0.75*AnchoLima/2>,
            <+AgarreLima/2+0.8*LargoExtraLima,-0.5*AnchoLima/2>,
            <+AgarreLima/2+0.9*LargoExtraLima,-0.25*AnchoLima/2>,
            <+AgarreLima/2+LargoExtraLima,+0.5*AnchoLima/2>,
            <+AgarreLima/2+0.8*LargoExtraLima,+0.5*AnchoLima/2>,
            <+AgarreLima/2+0.3*LargoExtraLima,+0.75*AnchoLima/2>,
            <+AgarreLima/2+0.1*LargoExtraLima,+0.5*AnchoLima/2>,
            <+AgarreLima/2+0.0*LargoExtraLima,+AnchoLima/2>,
            <-AgarreLima/2,+AnchoLima/2>  
            
          }   
          box                           {   
            <-AgarreLima/8,0, -AnchoLima/2-1.0*GruesoLima >,
            <+AgarreLima/8, GruesoLima+2*GruesoMetal, +AnchoLima/2+1.0*GruesoLima >
          }   
        }
        box                           {   
          <-AgarreLima/2, 1.01*GruesoLima, -AnchoLima/2 >,
          <+AgarreLima/2, GruesoLima+4*GruesoMetal, +AnchoLima/2 > 
        }   
        cylinder                        {
          -Upper*y, Upper*y, 0.1*LargoExtraLima
          scale         <1.5,1,0.5>
          translate    < AgarreLima/2+0.85*LargoExtraLima,0,
                          +0.5*AnchoLima/2 >
                          
        }
      }
      torus                             {
        ( RadioAire+RadioTorusCompleto )/2, ( RadioTorusCompleto-RadioAire )/2
        translate                       GruesoLima*y
      }
      texture                           {
        TexturaLima  
        scale           AgarreLima+LargoExtraLima
      }
      #if ( abs( AnguloLima ) < 90 )
        // simulacion burda
        rotate         AnguloLimaElevacion*z*( 90-abs( AnguloLima ) )/90
      #end  // #if ( abs( AnguloLima ) < 90 )
      rotate                            AnguloLima*y
      translate                         GruesoMetal*y
    }
    // Prensa
    #local EjePrensa                    = 4.54*L;
    #local DiametroCilindro             = 3.41*L;
    #local AperturaTopePrensa           =
        AperturaTope*( LargoLiso-Sellado-EjePrensa )/( LargoLiso-Sellado );
    #local AperturaMaximaPrensa         =
        AperturaMaxima*( LargoLiso-Sellado-EjePrensa )/( LargoLiso-Sellado );
    #local AperturaMinimaPrensa         =
        AperturaMinima*( LargoLiso-Sellado-EjePrensa )/( LargoLiso-Sellado );
    #local LPrensa                      = LargoLiso-Sellado-EjePrensa;
    #local AnguloListo                  =
      degrees( atan2( AperturaTopePrensa, LPrensa ) );
    #local AnguloPrensado               =
      degrees( asin( ( AperturaMaxima-AperturaMinima )/PivPrensa ) );
    union                               {
      cylinder                          {
        -AperturaMaxima*y, +AperturaMaxima*y,
        DiametroCilindro/2
      }
      difference                        {
        sphere                          {
          0, 1.5*DiametroCilindro/2
          scale <1,0.25,1>
          translate             -AperturaMaxima*y
        }
        plane                           {
          -y, 0
        }
      }
      union                             {
        difference                      {
          intersection                      {
            union                           {
              prism                           {
                -0.5, 0.5, 10,
                #local FracMetal              = GruesoMetal/DeltaYPrensa;
                <  0, 0 >,
                < -PivPrensa/DeltaXPrensa, 0 >,
                < -0.70, 0.67 >,
                < -0.86, 0.75 >,
                < -1, 1 >,
                < -1, 1+1.3*FracMetal >,
                < -0.86, 0.75+1.2*FracMetal >,
                < -0.70, 0.67+1.2*FracMetal >,
                < -PivPrensa/DeltaXPrensa, 1.2*FracMetal >,
                <  0, FracMetal >
                rotate                        -90*x
              }
              sphere                            {
                0, sqrt( 0.01*vlength( <-PivPrensa/DeltaXPrensa,0,0>-
                                        <-0.7,0.67,0> ) )
                scale <1,1,DeltaZPrensa >
                scale           <1,0.5,1>
                rotate          -45*z
                translate
                    <-PivPrensa/DeltaXPrensa,0,0>*0.9+<-0.7,0.67,0>*0.1
              }
              scale     < DeltaXPrensa, DeltaYPrensa, 1 >
            }
            prism                           {
              -Upper, Upper, 4,
              < -DeltaXPrensa, -0.75*DeltaZPrensa/2 >,
              < 0, -DeltaZPrensa/2 >,
              < 0, +DeltaZPrensa/2 >,
              < -DeltaXPrensa, +0.75*DeltaZPrensa/2 >
            }
            union                           {
              sphere                          {
                0, sqrt ( DeltaXPrensa*DeltaXPrensa+DeltaYPrensa*DeltaYPrensa )/3
                translate   2*( -DeltaXPrensa*x+DeltaYPrensa* y )/3
              }
              box                           {
                -0.8*<1,0,1>, 0.8
                scale  < DeltaXPrensa, DeltaYPrensa, DeltaZPrensa >
              }
            }
            translate                       max( GruesoMetal, ExcPivPrensa )*x
          }
          cylinder                              {
            -Upper*y, Upper*y, DiametroCilindro/2
            scale        <2*PivPrensa/DiametroCilindro, 1, 1>
          }
        }  // difference Prensa
        sphere                                  {
          0, 1
          scale         < DiametroCilindro/2, GruesoMetal, DiametroCilindro/2 >
          translate                             GruesoMetal*y
        }
        texture                           {
          TexturaPrensa         
          translate             -1.100*x
          scale               <DeltaXPrensa, DeltaYPrensa, AnchoInicial/2 >
        }
        translate                           -GruesoMetal/2*y
        #if ( FraccionPrensa < 0 )
          rotate                            FraccionPrensa*AnguloRecPrensa*z
        #else  // #else ( FraccionPrensa < 0 )
          rotate                            AnguloListo*z
          rotate                FraccionPrensa*AnguloPrensado*z
        #end   // #else ( FraccionPrensa < 0 )
        translate                       +GruesoMetal/2*y
        rotate                                  AnguloPrensa*y
        translate                       AperturaMaxima*y
      }  // union Prensa giradora
      translate            1*( AperturaMaxima-AperturaTope )*y
      translate                       ( LargoLiso-EjeAgarre-EjePrensa )*x
    }
    texture                             {
      TexturaPrincipal  
      scale         LargoTotal
    }
  }
#end  // #macro CUMP_CortaUnasManoPie()

#macro CUMP_CortaUnas( FraccionPrensa, AnguloLima, AnguloPrensa )
  object                                {
    CUMP_CortaUnasManoPie( FraccionPrensa, AnguloLima, AnguloPrensa,
        CUMP_TexturaPrincipal, CUMP_TexturaLima, CUMP_TexturaPrensa )
  }
#end  // #macro CUMP_CortaUnas()
