// This file is licensed under the terms of the CC-LGPL.
//
// Autor  : Rafael Angel Campos Vargas
// Correo : RofoelCompos@hotmail.com  
// Apdo. 964-1250 Escazu, San Jose, Costa Rica
// dedicado a Jesus y Maria...
//
// En este momento estoy creando un programa
// de animacion LibreN3D para FreeDOS con el compilador FreePascal
// falta mucho... 17 agosto 2016
//                
// Mis dibujos estan en unidades L (Lisa)
// 1 L equivale a 0.055063 pulgadas 
// definido como 0.001 para POVRAY
//     
// GPSM GrifoPlasticoSimulacionMetal
// El archivo dibuja un grifo para servicio de agua domestico
// un modelo muy popular en Costa Rica.
//
// La macro principal tiene solo un parametro. 
// #macro GPSM_Grifo( AnguloLlave ) 
//      AnguloLlave             : Angulo para girar la llave del grifo.
//
// La siguiente macro permite modificar las texturas.
// #macro GPSM_GrifoPlasticoSimulacionMetal( AnguloLlave,
//               MaterialGrifo, MaterialUnionInterna, MaterialBoquilla )
//      AnguloLlave             : Angulo para girar la llave del grifo.   
//      MaterialGrifo           : Material principal del aparato.
//              Valor por defecto GPMS_MaterialGrifo.   
//      MaterialUnionInterna    : Material interno y poco visible 
//              en la union del grifo a la tuberia. 
//              Valor por defecto GPSM_MaterialUnionInterno.
//      MaterialBoquilla        : Material interno en la boca del grifo.
//              Valor por defecto GPSM_MaterialBoquilla.
//                                                                       
// El grifo con angulo nulo se encuentra completamente contenido 
// entre las siguientes coordenada.
//      Minimo  = <  -18,  -37,  -18 >*L
//      Maximo  = <  103,   81,   18 >*L 
// El grifo mira hacia X+, gravedad Y-. La rosca de unin con la tubera de la casa
// parte centrada desde el origen de coordenadas. 
//
// Estas son variables predefinidas o generadas en el
// interior de las macros que no deberian modificarse directamente.
// Si quiere cambiarlas, recurra a los modificadores.  
// #declare GPSM_DefaultMaterialGrifo       
//      Material principal del aparato.
// #declare GPSM_DefaultMaterialUnionInterna          
//      Material interno y poco visible en la union del grifo y la tuberia.
// #declare GPSM_DefaultMaterialBoquilla       
//      Material interno en la boca del grifo.
//
// Los siguientes son los modificadores
// aplicables y redefinibles por el usuario (si se indica es default)   
// #declare GPSM_MaterialGrifo                      = material { ... }        
//      Material principal del aparato.
// #declare GPSM_MaterialUnionInterna               = material { ... }
//      Material interno y poco visible en la union del grifo y la tuberia.
// #declare GPSM_MaterialBoquilla                   = material { ... }         
//      Material interno en la boca del grifo.
//        
// Para colocarlo simplemente
// #include "GPSM.inc" 
// object { GPSM_Grifo(0) }
//                                  
// Ejemplo con la macro detallada, mismo objeto default
// #include "GPSM.inc"   
// object                       {
//   GPSM_GrifoPlasticoSimulacionMetal( 0, 
//          GPSM_MaterialGrifo, GPSM_MaterialUnionInterna, GPSM_MaterialBoquilla )
// }
//        
// Con modificadores
// #include "GPSM.inc"
// #include "textures.inc"
// #declare GPSM_MaterialPrincipal   = material {  texture { PinkAlabaster } }
// object        { GPSM_Grifo(0) }
//
#include "colors.inc"
#include "metals.inc"

#declare GPSM_L                                 = 0.001;

#declare GPSM_DefaultMaterialGrifo              =
  material                                      {
    texture                               { 
      checker   
      texture                             {   
        bumps
        scale     0.025 
        sine_wave
        texture_map                       {  
          [ 0.0 T_Chrome_1C ]
          [ 1.0 T_Chrome_1D ]
        }                     
        scale         0.1
      } 
      texture                     {                         
        T_Chrome_1C    
        normal                              { 
          ripples
          turbulence        0.2
          scale             0.1
        } 
      }  
      translate               -0.5*z-0.5*y 
      rotate                  0*y
      scale                       0.45
      translate               0.0*x
    }
  }

#ifndef ( GPSM_MaterialGrifo )
  #declare GPSM_MaterialGrifo                   = GPSM_DefaultMaterialGrifo
#end  // #ifndef ( GPSM_MaterialGrifo )

#declare GPSM_DefaultMaterialUnionInterna       =
  material                                      {
    texture                                     {
      pigment                                   {
        Green
      }
    }
  }

#ifndef ( GPSM_MaterialUnionInterna )
  #declare GPSM_MaterialUnionInterna            =
                        GPSM_DefaultMaterialUnionInterna
#end  // #ifndef ( GPSM_MaterialUnionInterna )

#declare GPSM_DefaultMaterialBoquilla           =
  material                                      {
    texture                                     {
      pigment                                   {
        Gray40
      }
    }
  }

#ifndef ( GPSM_MaterialBoquilla )
  #declare GPSM_MaterialBoquilla                =
                        GPSM_DefaultMaterialBoquilla
#end  // #ifndef ( GPSM_MaterialBoquilla )

#macro GPSM_GrifoPlasticoSimulacionMetal(
                AnguloLlave,
                MaterialGrifo, MaterialUnionInterna, MaterialBoquilla )
  union                                         {
    #local L                                    = GPSM_L;
    #local ProfundidadUnion                     = 18*L;
    #local DiametroUnion                        = 19*L;
    #local ProfundidadBoquilla                  = 09*L;
    #local DiametroBoquilla                     = 27*L;
    #local DiametroCilindroVertical             = 29*L;
    #local AltoCilindroVertical                 = 40*L;
    #local DesplazamientoCilindroVertical       = 40*L;
    #local DiametroHexagonoCilindroVertical     = 40*L;
    #local DiametroHexagonoUnionPosterior       = 28*L;
    #local LargoHexagonoUnionPosterior          = 12*L;
    #local DiametroCilindroLlave                = 12*L;
    #local DiametroEsferaCentralLlave           = 21*L;
    #local AltoRelativoLlave                    = 16*L;
    #local SeparacionEsferaLlave                = 28*L;
    #local CurvaturaSalidaAgua                  = 35*L;
    #local DiametroSalidaAgua                   = 22*L;
    #local AnguloSalidaAgua                     = 80;
    // BEGIN posterior espiral
    union                                       {
      difference                                {
        union                                   {
          cylinder                              {
            0, -ProfundidadUnion*x, DiametroUnion/2
          }
          #local TotalRosca                     = 20;
          #local C                              = 0;
          #local DeltaRosca                     = ProfundidadUnion/TotalRosca;
          #local AnguloRoscaD                   =
                        degrees( atan2( DeltaRosca, DiametroUnion ) );
          #while ( C < TotalRosca )
            difference                          {
              cylinder                          {
                -0.4*DeltaRosca*x, +0.4*DeltaRosca*x,
                1.1*DiametroUnion/2
              }
              plane                             {
                z, 0
              }
              #if ( mod( C, 2 ) = 0 )
                rotate                          -AnguloRoscaD*z
                scale                           <1,1,-1>
              #else  // #if ( mod( C, 2 ) = 0 )
                rotate                          AnguloRoscaD*z
              #end
              translate                         ( C-TotalRosca+0.5 )
                                                *DeltaRosca*x
            }
            #local C                            = C+1;
          #end  // #while ( C < TotalRosca )
        }
        cylinder                                {
          ProfundidadUnion*x, -1.1*ProfundidadUnion*x, 0.8*DiametroUnion/2
        }
      }
      difference                                {
        cylinder                                {
          0, -0.9*ProfundidadUnion*x, 0.8*DiametroUnion/2
        }
        cylinder                                {
          -0.1*ProfundidadUnion*x, -1.1*ProfundidadUnion*x, 0.7*DiametroUnion/2
        }
        cylinder                                {
          ProfundidadUnion*x, -1.1*ProfundidadUnion*x, 0.1*DiametroUnion/2
        }
        material                                {
          MaterialUnionInterna
        }
      }
    }
    // END posterior espiral
    // BEGIN hexagono union posterior
    intersection                            {
      union                                 {
        cylinder                            {
          0, LargoHexagonoUnionPosterior*x,
          DiametroHexagonoUnionPosterior/2
        }
        sphere                              {
          0,
          DiametroHexagonoUnionPosterior/2
          scale             <0.2,1,1>
          translate             LargoHexagonoUnionPosterior*x
        }
      }
      #local C                              = 0;
      #while ( C < 3 )
        box                                 {
          <0,-DiametroHexagonoUnionPosterior,
           -0.44*DiametroHexagonoUnionPosterior>,
          <2*LargoHexagonoUnionPosterior,+DiametroHexagonoUnionPosterior,
           +0.44*DiametroHexagonoUnionPosterior>
          rotate            C*60*x
        }
        #local C                            = C+1;
      #end  // #while ( C < 3 )
    }
    // END hexagono union posterior
    // BEGIN salida agua
    union                                       {
      difference                                {
        torus                                   {
          CurvaturaSalidaAgua, DiametroSalidaAgua/2
          rotate                        90*x
        }
        plane                                   {
          x, 0
        }
        plane                                   {
          y, 0
          rotate                ( 90-AnguloSalidaAgua )*z
        }
      }
      // BEGIN boquilla
      union                                       {
        difference                                {
          union                                   {
            cylinder                              {
              0, -ProfundidadBoquilla*x, DiametroBoquilla/2
            }
            #local TotalRosca                     = 6;
            #local C                              = 0;
            #local DeltaRosca                     = ProfundidadBoquilla/TotalRosca;
            #local AnguloRoscaD                   =
                          degrees( atan2( DeltaRosca, DiametroBoquilla ) );
            #while ( C < TotalRosca )
              difference                          {
                cylinder                          {
                  -0.4*DeltaRosca*x, +0.4*DeltaRosca*x,
                  1.1*DiametroBoquilla/2
                }
                plane                             {
                  z, 0
                }
                #if ( mod( C, 2 ) = 0 )
                  rotate                          -AnguloRoscaD*z
                  scale                           <1,1,-1>
                #else  // #if ( mod( C, 2 ) = 0 )
                  rotate                          AnguloRoscaD*z
                #end
                translate                         ( C-TotalRosca+0.5 )
                                                  *DeltaRosca*x
              }
              #local C                            = C+1;
            #end  // #while ( C < TotalRosca )
          }
          cylinder                                {
            ProfundidadBoquilla*x, -1.1*ProfundidadBoquilla*x, 0.6*DiametroBoquilla/2
          }
        }
        union                                     {
          difference                                {
            cylinder                                {
              0, -0.95*ProfundidadBoquilla*x, 0.6*DiametroBoquilla/2
            }
            cylinder                                {
              -0.1*ProfundidadBoquilla*x, -1.1*ProfundidadBoquilla*x, 0.5*DiametroBoquilla/2
            }
          }
          difference                              {
            union                                 {
              #local C                            = 0;
              #while ( C < 3 )
                box                               {
                  <0,0.6*DiametroBoquilla/2,DiametroBoquilla/40>,
                  <-0.95*ProfundidadBoquilla,-0.6*DiametroBoquilla/2,
                          -DiametroBoquilla/40>
                  rotate          C*60*x
                }
                #local C                          = C+1;
              #end  // #while ( C < 3 )
            }
            cylinder                                {
              -0.1*ProfundidadBoquilla*x, -1.1*ProfundidadBoquilla*x, 0.1*DiametroBoquilla/2
            }
          }
          material                                {
            MaterialBoquilla
          }
        }
        translate         CurvaturaSalidaAgua*y
        scale                   <-1,1,1>
        rotate                  -AnguloSalidaAgua*z
      }
      // END boquilla
      translate         ( LargoHexagonoUnionPosterior+
                                DesplazamientoCilindroVertical )*x
      translate                 -CurvaturaSalidaAgua*y
      translate         +( DiametroHexagonoUnionPosterior/2-
                           DiametroSalidaAgua/2 )*y
    }
    // END salida agua
    // BEGIN conjunto llave de giro
    union                                       {
      // BEGIN cilindro vertical
      union                                     {
        cylinder                                {
          0, AltoCilindroVertical*y, DiametroCilindroVertical/2
        }
        sphere                                  {
          AltoCilindroVertical*y, DiametroCilindroVertical/2
        }
        // BEGIN hexagono ajuste cilindro vertical
        intersection                            {
          union                                 {
            cylinder                            {
              0, 0.2*AltoCilindroVertical*y,
              DiametroHexagonoCilindroVertical/2
            }
            sphere                              {
              0, DiametroHexagonoCilindroVertical/2
              scale             <1,0.2,1>
              translate         0.2*AltoCilindroVertical*y
            }
            translate           0.8*AltoCilindroVertical*y
          }
          #local C                              = 0;
          #while ( C < 3 )
            box                                 {
              <-DiametroHexagonoCilindroVertical,0,
               -0.44*DiametroHexagonoCilindroVertical>,
              <+DiametroHexagonoCilindroVertical,1.1*AltoCilindroVertical,
               +0.44*DiametroHexagonoCilindroVertical>
              rotate            C*60*y
            }
            #local C                            = C+1;
          #end  // #while ( C < 3 )
        }
        // END hexagono ajuste cilindro vertical
      }
      // END cilindro vertical
      // BEGIN llave dinamica
    #local DiametroCilindroLlave                = 12*L;
    #local DiametroEsferaCentralLlave           = 21*L;
    #local AltoRelativoLlave                    = 16*L;
      union                                     {
        cylinder                                {
          -0.1*AltoRelativoLlave*y,
          AltoRelativoLlave*y, DiametroCilindroLlave/2
        }
        sphere                                  {
          AltoRelativoLlave*y, DiametroEsferaCentralLlave/2
        }
        sphere                                  {
          AltoRelativoLlave*y+SeparacionEsferaLlave*x,
          0.6*DiametroEsferaCentralLlave/2
        }
        cone                                    {
          0, 0.6*DiametroEsferaCentralLlave/2,
          +SeparacionEsferaLlave*x,
          0.3*DiametroEsferaCentralLlave/2
          translate                     AltoRelativoLlave*y
        }
        cone                                    {
          0, 0.6*DiametroEsferaCentralLlave/2,
          -SeparacionEsferaLlave*x,
          0.3*DiametroEsferaCentralLlave/2
          translate                     AltoRelativoLlave*y
        }
        sphere                                  {
          AltoRelativoLlave*y-SeparacionEsferaLlave*x,
          0.6*DiametroEsferaCentralLlave/2
        }
        rotate                  AnguloLlave*y
        translate               ( AltoCilindroVertical+
                                  DiametroCilindroVertical/2 )*y
      }
      // END llave dinamica
      translate                 DesplazamientoCilindroVertical*x
    }
    // END conjunto llave de giro
    // BEGIN cuerpo central
    lathe                                       {
      bezier_spline
      4,
      < 0.800,-0.200 >,
      < 2.000, 0.500 >,
      < 1.500, 1.100 >,
      < 0.900, 1.200 >
      rotate            -90*z
      scale             < DesplazamientoCilindroVertical,
                          DiametroUnion/2, DiametroUnion/2 >
      translate           LargoHexagonoUnionPosterior*x
    }
    // END cuerpo central
    material                                    {
      MaterialGrifo
    }
  }
#end  // #macro GPSM_GrifoPlasticoSimulacionMetal

#macro GPSM_Grifo( AnguloLlave )
  object                                        {
    GPSM_GrifoPlasticoSimulacionMetal( AnguloLlave,
        GPSM_MaterialGrifo, GPSM_MaterialUnionInterna, GPSM_MaterialBoquilla )
  }
#end  // #macro GPSM_Grifo()
