// This file is licensed under the terms of the CC-LGPL.
//
// Autor  : Rafael Angel Campos Vargas
// Correo : RofoelCompos@hotmail.com  
// Apdo. 964-1250 Escazu, San Jose, Costa Rica
// dedicado a Jesus y Maria...
//
// En este momento estoy creando un programa
// de animacion LibreN3D para FreeDOS con el compilador FreePascal
// falta mucho... 14 diciembre 2016
//                
// Mis dibujos estan en unidades L (Lisa)
// 1 L equivale a 0.055063 pulgadas 
// definido como 0.001 para POVRAY
//     
// JHSV JuegoHomeSigloVeintiuno
// El archivo dibuja un tablero para el tradicional juego Home-Parchis, con relieve
// plastico para facilitar el tiro del dado y la colocacion de las piezas
// por ninos menores. El original se baso en un modelo para
// la serie Paw Patrol de Nickelodeon producto marca Cardinal Industries. 
// Se elimino intencionalmente las referencias publicitarias para
// facilitar la aplicacion legal de la patente CC-LGPL.
// El mecanismo de tirada del dado consiste en una semiesfera en un compartimento
// plastico y una plaquita metalica que sirve de rebote y "arroja" el dado en su interior.
// Es muy poco flexible. Es compatible con POVRay 3.1.
//
// La macro principal tiene tres parametros. 
// #macro JHSV_Home( Semilla, NumeroDado, FraccionPresionar )   
//      Semilla                 : Para semilla del generador aleatorio.
//      NumeroDado              : Cara del lado resultado de la tirada.
//      FraccionPresionar       : Fraccion propia para el mecanismo de tirada de dado.
//              Use 0.0 para liberar y 1.0 para completamente presionado.
//
// La siguiente macro permite mayor versatilidad. Debido al efecto de la semilla,
// es de esperar que la posicion del dado varie respecto a la macro anterior.
// #macro JHSV_JuegoHomeSigloVeintiuno( NumeroDado, FraccionPresionar,
//                LadoFundamental, CurvaturaFundamental,
//                GruesoPlastico, GruesoMetal,
//                DiametroPosicion, AlturaPosicion, LadoDado,
//                RotateDado, TranslateDado,
//                MaterialPrincipal, MaterialTablero, MaterialMetalico,
//                MaterialDado, MaterialSalida, MaterialMeta )
//      NumeroDado              : Cara del lado resultado de la tirada.
//      FraccionPresionar       : Fraccion propia para el mecanismo de tirada de dado.
//              Use 0.0 para liberar y 1.0 para completamente presionado.   
//      LadoFundamental         : Determina el lado del cuadrado que delimita la figura.
//              Valor por defecto 187*L.
//      CurvaturaFundamental    : Curvatura de las esquinas como diametro del circulo.
//              Deberia ser algo menor a sqrt(2)*LadoFundamental.
//              Valor por defecto 230*L.
//      GruesoPlastico          : Grueso del plastico utilizado. 
//              Valor por defecto 1.7*L.
//      GruesoMetal             : Grueso del metal de rebote en la estructura para tirar
//              el dado. Valor por defecto 0.3*L.
//      DiametroPosicion        : Diametro de los cilindros para ubicar fichas.
//              Valor por defecto 11.0*L.
//      AlturaPosicion          : Altura de los cilindros para ubicar fichas.
//              Valor por defecto 4.5*L.
//      LadoDado                : Lado del cubo para dibujar el dado.
//              Valor por defecto 7*L.
//      RotateDado              : Se aplicara este rotate al origen antes de ubicar el dado.
//              Utilice k*y si no quiere variar el resultado NumeroDado.
//      TranslateDado           : Luego de ubicar el dado al centro de la placa
//              metalica (incluye el efecto de FraccionPresionar), se aplicara
//              un translate con este TranslateDado.
//      MaterialPrincipal       : Material principal a aplicar a la estructura.
//              Valor por defecto JHSV_MaterialPrincipal.
//      MaterialTablero         : Material de la superficie impresa del tablero.
//              Valor por defecto JHSV_MaterialTablero.                     
//      MaterialMetalico        : Material para la plaquita metalica de rebote del dado.
//              Valor por defecto JHSV_MaterialMetalico.
//      MaterialDado            : Material para la superficie del dado (sin los puntos).
//              Valor por defecto JHSV_MaterialDado.
//      MaterialSalida          : Material para la casilla de salida de Home-Parchis.
//              Valor por defecto JHSV_MaterialSalida.
//      MaterialMeta            : Material para la casilla de meta de Home-Parchis.
//              Valor por defecto JHSV_MaterialMeta.       
// 
// Incluye una macro para dibujar un dado.           
// #macro JHSV_Dado( LadoDado, MaterialDado )
//      LadoDado                : Lado del cubo para dibujar el dado.
//              Valor por defecto 7*L.
//      MaterialDado            : Material para la superficie del dado (sin los puntos).
//              Valor por defecto JHSV_MaterialDado.
//  
// Fichas independientes del tablero, pueden dibujarse con la siguiente macro.  
// Tomar en cuenta que las fichas se dibujan con un radio GruesoPlastico menor al
// tamano ajustado.
// #macro JHSV_Ficha( DiametroPosicion, AlturaPosicion,
//                        GruesoPlastico, IndiceColor )
//      DiametroPosicion        : Diametro de los cilindros para ubicar fichas.
//              Valor por defecto 11.0*L.
//      AlturaPosicion          : Altura de los cilindros para ubicar fichas.
//              Valor por defecto 4.5*L.
//      GruesoPlastico          : Grueso del plastico utilizado. 
//              Valor por defecto 1.7*L.                                        
//      IndiceColor             : Indice entero de color. Puede tomar los valores
//              JHSV_FichaAzul, JHSV_FichaVerde, JHSV_FichaRoja y JHSV_FichaAmarilla.
// 
// El tablero de juego se encuentra contenido (FraccionPresionar nulo)
// entre las siguientes coordenadas:
//      Minimo  = <-LadoFundamental/2,0,+LadoFundamental/2>
//      Maximo  = <+LadoFundamental/2,+0.110*LadoFundamental+5*GruesoPlastico,
//                 +LadoFundamental/2> 
// O con valores por defecto:
//      Minimo  = <-93.5,   0.0,-93.5 >*L
//      Maximo  = < 93.5,  29.1, 93.5 >*L 
// El tablero descansa centrado sobre el plano XZ, gravedad Y-.    
//
// Estas son variables predefinidas o generadas en el
// interna de las macros que no deberian modificarse directamente.
// Si quiere cambiarlas, recurra a los modificadores.  
// #declare JHSV_MacroTranslateDado   
//      Ubicacion del dado segun resultado dado por la macro a partir del origen.
//      Incluye el efecto del parametro "TranslateDado", y se calcula aunque
//      no se dibuje el dado del todo.
// #declare JHSV_TotalFichas                = 16;
//      Tamano de arreglo de fichas JHSV_ArregloFichas.
// #declare JHSV_FichaAzul                  = 0; 
//      Clave indice para ficha color azul.
// #declare JHSV_FichaVerde                 = 1;
//      Clave indice para ficha color verde.
// #declare JHSV_FichaRoja                  = 2;
//      Clave indice para ficha color rojo.
// #declare JHSV_FichaAmarilla              = 3;
//      Clave indice para ficha color amarillo.
// #declare JHSV_CasillaSalida              = 0;
//      Clave indice para casilla de salida de juego.
// #declare JHSV_CasillaActiva              = 1;
//      Clave indice para casilla de salida normal.
// #declare JHSV_CasillaMeta                = 2;
//      Clave indice para casilla de final de juego.
// #declare JHSV_DefaultMaterialPrincipal        
//      Material principal a aplicar a la estructura.
// #declare JHSV_DefaultMaterialTablero         
//      Material de la superficie impresa del tablero.
// #declare JHSV_DefaultMaterialMetalico        
//      Material para la plaquita metalica de rebote del dado.
// #declare JHSV_DefaultMaterialDado            
//      Material para la superficie del dado (sin los puntos).
// #declare JHSV_DefaultMaterialSalida          
//      Material para la casilla de salida de Home-Parchis.
// #declare JHSV_DefaultMaterialMeta            
//      Material para la casilla de meta de Home-Parchis.
//
// Los siguientes son los modificadores
// aplicables y redefinibles por el usuario (si se indica es default)   
// #declare JHSV_ArregloFichas             = array [ JHSV_TotalFichas ]
//      de vectores 4D de fichas dados como
//      < Visibilidad, Color, Estado, Posicion >   
//          Visibilidad     : "on" la ficha es visible, "off" invisible.
//          Color           : Indice de color (no modificar). Los colores
//              aparecen en el siguiente orden: cuatro JHSV_FichaAmarilla,
//              cuatro JHSV_FichaAzul, cuatro JHSV_FichaVerde y 
//              las ultimas cuatro JHSV_FichaRoja.
//          Estado          : Tres estados posibles como numero entero.
//              JHSV_CasillaSalida, JHSV_CasillaActiva y JHSV_CasillaMeta,
//              para indicar en cual de las tres regiones se ubica la ficha.
//          Posicion        : Las casillas de Salida y de Meta se numeran
//              desde 0 al 3. Siguiendo el sentido horario visto desde arriba,
//              y de afuera hacia adentro.
//              Para las casillas normales, la posicion 0 de cualquier color
//              es la casilla blanca de su propia esquina,
//              la numeracion sigue el sentido horario desde 0 hasta 27.
// #declare JHSV_ColorPuntoDado             = Black;
//      Color por diferencia del punto del dado.
// #declare JHSV_DadoVisible                = on;
//      "on" activa el dibujo del dado. "off" no dibuja el dado.
// #declare JHSV_PigmentAmarillo            = pigment { Yellow  }   
//      Pigmento a aplicar a las fichas con indice JHSV_FichaAmarilla                                           
// #declare JHSV_PigmentAzul                = pigment { Blue    }
//      Pigmento a aplicar a las fichas con indice JHSV_FichaAzul                                               
// #declare JHSV_PigmentVerde               = pigment { Green   }
//      Pigmento a aplicar a las fichas con indice JHSV_FichaVerde                                              
// #declare JHSV_PigmentRojo                = pigment { Red     } 
//      Pigmento a aplicar a las fichas con indice JHSV_FichaRoja                                             
// #declare JHSV_MaterialPrincipal          = material { ... }     
//      Material principal a aplicar a la estructura.
// #declare JHSV_MaterialTablero            = material { ... }    
//      Material de la superficie impresa del tablero.
// #declare JHSV_MaterialMetalico           = material { ... }   
//      Material para la plaquita metalica de rebote del dado.
// #declare JHSV_MaterialDado               = material { ... }    
//      Material para la superficie del dado (sin los puntos).
// #declare JHSV_MaterialSalida             = material { ... }     
//      Material para la casilla de salida de Home-Parchis.
// #declare JHSV_MaterialMeta               = material { ... }     
//      Material para la casilla de meta de Home-Parchis.
//        
// Para colocarlo simplemente
// #include "JHSV.inc" 
// object { JHSV_Home(1000, 5, 0.0) }
//                                  
// Ejemplo con la macro de texturas, mismo objeto default
// #include "JHSV.inc"
// object                       {   
//   JHSV_JuegoHomeSigloVeintiuno( 5, 0.0,    
//            0.187, 0.230, 0.0017, 0.0003, 0.011, 0.0045, 0.007, 0, 0,
//            JHSV_MaterialPrincipal, JHSV_MaterialTablero,
//            JHSV_MaterialMetalico, JHSV_MaterialDado, 
//            JHSV_MaterialSalida, JHSV_MaterialMeta )
// }
//        
// Con modificadores. Colocamos la Ficha 0 en la meta y la Ficha 1 la desaparecemos,
// note que no cambiamos el indice de color que es la componente y.
// #include "JHSV.inc"
// #include "textures.inc"
// #declare JHSV_MaterialPrincipal   = material { texture { PinkAlabaster } } 
// #declare JHSV_ArregloFichas[0]    = <on,JHSV_ArregloFichas[0].y,JHSV_CasillaMeta,2>;
// #declare JHSV_ArregloFichas[1]    = off*x+JHSV_ArregloFichas[1]*(y+z+t);
// object        { JHSV_Home(1000,5,0.0) }
// 

#include "colors.inc" 
#include "glass.inc"

#declare JHSV_L                         = 0.001;

#declare JHSV_ColorPuntoDado            = Black;
#declare JHSV_DadoVisible               = on;

#declare JHSV_TotalFichas               = 16;
#declare JHSV_FichaAzul                 = 0;
#declare JHSV_FichaVerde                = 1;
#declare JHSV_FichaRoja                 = 2;
#declare JHSV_FichaAmarilla             = 3;
#declare JHSV_CasillaSalida             = 0;
#declare JHSV_CasillaActiva             = 1;
#declare JHSV_CasillaMeta               = 2;
#declare JHSV_ArregloFichas             = array [ JHSV_TotalFichas ]
// < Visibilidad, Color, Estado, Posicion >
        {
           < on, JHSV_FichaAmarilla, JHSV_CasillaSalida, 0 >,
           < on, JHSV_FichaAmarilla, JHSV_CasillaSalida, 1 >,
           < on, JHSV_FichaAmarilla, JHSV_CasillaSalida, 2 >,
           < on, JHSV_FichaAmarilla, JHSV_CasillaSalida, 3 >,
           < on, JHSV_FichaAzul    , JHSV_CasillaSalida, 0 >,
           < on, JHSV_FichaAzul    , JHSV_CasillaSalida, 1 >,
           < on, JHSV_FichaAzul    , JHSV_CasillaSalida, 2 >,
           < on, JHSV_FichaAzul    , JHSV_CasillaSalida, 3 >,
           < on, JHSV_FichaVerde   , JHSV_CasillaSalida, 0 >,
           < on, JHSV_FichaVerde   , JHSV_CasillaSalida, 1 >,
           < on, JHSV_FichaVerde   , JHSV_CasillaSalida, 2 >,
           < on, JHSV_FichaVerde   , JHSV_CasillaSalida, 3 >,
           < on, JHSV_FichaRoja    , JHSV_CasillaSalida, 0 >,
           < on, JHSV_FichaRoja    , JHSV_CasillaSalida, 1 >,
           < on, JHSV_FichaRoja    , JHSV_CasillaSalida, 2 >,
           < on, JHSV_FichaRoja    , JHSV_CasillaSalida, 3 >
        }
                                              
#declare JHSV_PigmentAmarillo           = pigment { Yellow  }                                              
#declare JHSV_PigmentAzul               = pigment { Blue    }                                              
#declare JHSV_PigmentVerde              = pigment { Green   }                                              
#declare JHSV_PigmentRojo               = pigment { Red     }                                              
                                              
#declare JHSV_DefaultMaterialPrincipal  = 
  material                              { 
    texture                             {
      pigment   { color White*0.9+Green*0.1 transmit 0.85 }
      normal            {
        bumps
      }
      finish    {
        phong   0.3
        phong_size 20
      }
    }
  } 

#ifndef ( JHSV_MaterialPrincipal )
  #declare JHSV_MaterialPrincipal       = JHSV_DefaultMaterialPrincipal
#end  // #ifndef ( JHSV_MaterialPrincipal )

#declare JHSV_DefaultMaterialTablero    =
  material                              {
    texture                             {
      pigment           {
        radial
        pigment_map     {
          [ 0.0  color Blue ]
          [ 0.25 color Blue ]
          [ 0.25 color Green ]
          [ 0.50 color Green ]
          [ 0.50 color Red ]
          [ 0.75 color Red ]
          [ 0.75 color Yellow ]
          [ 1.00 color Yellow ]
        }
      }
      normal            {
        wrinkles
        translate       10               
        turbulence      0.4
        scale           0.03
      }
    }
  }

#ifndef ( JHSV_MaterialTablero )
  #declare JHSV_MaterialTablero         = JHSV_DefaultMaterialTablero
#end  // #ifndef ( JHSV_MaterialTablero )

#declare JHSV_DefaultMaterialMetalico   =
  material                              {
    texture                             {
      pigment   { color Silver }
      normal            {
        bumps
      }
      finish    {
        phong   0.3
        phong_size 20
      }
    }
  }

#ifndef ( JHSV_MaterialMetalico )
  #declare JHSV_MaterialMetalico        = JHSV_DefaultMaterialMetalico
#end  // #ifndef ( JHSV_MaterialMetalico )

#declare JHSV_DefaultMaterialDado       =
  material                              {
    texture                             {
      pigment   { color White }
      normal            {
        bumps
      }
      finish    {
        phong   0.3
        phong_size 20
      }
    }
  }

#ifndef ( JHSV_MaterialDado )
  #declare JHSV_MaterialDado            = JHSV_DefaultMaterialDado
#end  // #ifndef ( JHSV_MaterialDado )

#declare JHSV_DefaultMaterialSalida     =
  material                              {
    texture                             {
      pigment   { color White transmit 0.5 }
      normal            {
        bumps
      }
      finish    {
        phong   0.3
        phong_size 20
      }
    }
  }

#ifndef ( JHSV_MaterialSalida )
  #declare JHSV_MaterialSalida          = JHSV_DefaultMaterialSalida
#end  // #ifndef ( JHSV_MaterialSalida )

#declare JHSV_DefaultMaterialMeta       =
  material                              {
    texture                             {
      pigment                           {
        checker
        pigment   { color White transmit 0.5 }
        pigment   { color Black transmit 0.5 }
      }
      normal            {
        bumps
      }
      finish    {
        phong   0.3
        phong_size 20
      }
    }
  }

#ifndef ( JHSV_MaterialMeta )
  #declare JHSV_MaterialMeta            = JHSV_DefaultMaterialMeta
#end  // #ifndef ( JHSV_MaterialMeta )

#macro JHSV_Ficha( DiametroPosicion, AlturaPosicion,
                        GruesoPlastico, IndiceColor )
  difference                            {
    union                               {
      cylinder                          {
        0, 3.5*AlturaPosicion*y, DiametroPosicion/2-2*GruesoPlastico
      }
      cylinder                          {
        AlturaPosicion*y, 3.5*AlturaPosicion*y,
        DiametroPosicion/2-GruesoPlastico
      }
    }
    cylinder                            {
      -AlturaPosicion*y, ( 3.5*AlturaPosicion-GruesoPlastico )*y,
      DiametroPosicion/2-3*GruesoPlastico
    }
    #if ( IndiceColor >= 0 )
    texture                             {
      #switch ( IndiceColor )
        #case ( JHSV_FichaAmarilla )
          pigment { JHSV_PigmentAmarillo }
        #break
        #case ( JHSV_FichaRoja )
          pigment { JHSV_PigmentRojo }
        #break
        #case ( JHSV_FichaAzul )
          pigment { JHSV_PigmentAzul }
        #break
        #case ( JHSV_FichaVerde )
          pigment { JHSV_PigmentVerde }
        #break
      #end

      normal                              {
        bumps
      }
    }
    #end  // #if ( IndiceColor >= 0 )
  }
#end  // #macro JHSV_Ficha

#macro JHSV_CilindroVisible( DiametroPosicion, AlturaPosicion,
              GruesoPlastico, IndiceColor, IndiceEstado, Posicion )
  #local Cuenta                         = 0;
  #local TotalCuenta                    = dimension_size( JHSV_ArregloFichas, 1 );
  #while ( Cuenta < TotalCuenta )
    #local BanderaVisible               = off;
    #if ( JHSV_ArregloFichas[ Cuenta ].x )
      #if ( IndiceEstado = JHSV_ArregloFichas[ Cuenta ].z )
        #switch ( IndiceEstado )
          #case ( JHSV_CasillaMeta )
          #case ( JHSV_CasillaSalida )
            #if ( IndiceColor = JHSV_ArregloFichas[ Cuenta ].y )
              #local BanderaVisible     =
                      ( Posicion = JHSV_ArregloFichas[ Cuenta ].t );
            #end  // #if ( IndiceColor = JHSV_ArregloCuenta[ Cuenta ].y )
          #break
          #case ( JHSV_CasillaActiva )
            #local Visible                        =
              mod( ( 3-JHSV_ArregloFichas[Cuenta].y )*7+Posicion, 28 );
            #local BanderaVisible = ( Visible  = JHSV_ArregloFichas[ Cuenta ].t );
          #break
        #end  // #switch ( IndiceEstado )
        #if ( BanderaVisible )
          JHSV_Ficha( DiametroPosicion, AlturaPosicion,
                      GruesoPlastico, JHSV_ArregloFichas[ Cuenta ].y )
        #end  // #if ( BanderaVisible )
      #end  // #if ( IndiceEstado = ... )
    #end  // #if ( JHSV_ArregloCuenta[ Cuenta ].x )
    #local Cuenta                       = Cuenta+1;
    #if ( BanderaVisible )
      #local Cuenta                     = TotalCuenta;
    #end
  #end  // #while ( Cuenta < TotalCuenta )
#end  // #macro JHSV_CilindroVisible

#macro JHSV_Dado( LadoDado, MaterialDado )
  difference                            {
    #local RadioPunto                   = 0.1;
    superellipsoid                      {
      <0.2,0.2>
      scale                             0.5
    }
    union                               {
      // 1
      sphere                              {
        0, RadioPunto
        translate                 y/2
      }
      // END 1
      // 6
      sphere                              {
        0, RadioPunto
        translate                 -y/2+1*x/3+z/4
      }
      sphere                              {
        0, RadioPunto
        translate                 -y/2-1*x/3+z/4
      }
      sphere                              {
        0, RadioPunto
        translate                 -y/2+0*x/3+z/4
      }
      sphere                              {
        0, RadioPunto
        translate                 -y/2+1*x/3-z/4
      }
      sphere                              {
        0, RadioPunto
        translate                 -y/2+0*x/3-z/4
      }
      sphere                              {
        0, RadioPunto
        translate                 -y/2-1*x/3-z/4
      }
      // END 6
      // 4
      sphere                              {
        0, RadioPunto
        translate                 -z/2+x/4-y/4
      }
      sphere                              {
        0, RadioPunto
        translate                 -z/2-x/4-y/4
      }
      sphere                              {
        0, RadioPunto
        translate                 -z/2+x/4+y/4
      }
      sphere                              {
        0, RadioPunto
        translate                 -z/2-x/4+y/4
      }
      // END 4
      // 3
      sphere                              {
        0, RadioPunto
        translate                 +z/2-x/4+y/4
      }
      sphere                              {
        0, RadioPunto
        translate                 +z/2+x/4-y/4
      }
      sphere                              {
        0, RadioPunto
        translate                 +z/2
      }
      // END 3
      // 5
      sphere                              {
        0, RadioPunto
        translate                 -x/2+z/4-y/4
      }
      sphere                              {
        0, RadioPunto
        translate                 -x/2-z/4-y/4
      }
      sphere                              {
        0, RadioPunto
        translate                 -x/2+z/4+y/4
      }
      sphere                              {
        0, RadioPunto
        translate                 -x/2-z/4+y/4
      }
      sphere                            {
        0, RadioPunto
        translate               -x/2
      }
      // END 5
      // 2
      sphere                              {
        0, RadioPunto
        translate                 +x/2+z/4-y/4
      }
      sphere                              {
        0, RadioPunto
        translate                 +x/2-z/4+y/4
      }
      // END 2
      pigment           { JHSV_ColorPuntoDado }
    }
    material                            {
      MaterialDado
    }
    scale                       LadoDado
  }
#end  // #macro JHSV_Dado

#macro JHSV_JuegoHomeSigloVeintiuno( NumeroDado, FraccionPresionar,
                LadoFundamental, CurvaturaFundamental,
                GruesoPlastico, GruesoMetal,
                DiametroPosicion, AlturaPosicion, LadoDado,
                RotateDado, TranslateDado,
                MaterialPrincipal, MaterialTablero, MaterialMetalico,
                MaterialDado, MaterialSalida, MaterialMeta )
  merge                                 {
    #local RadioJuego                   = 0.43;
    // Tablero
    intersection                        {
      box                             {
        < -LadoFundamental/2+1.2*GruesoPlastico, 0,
          -LadoFundamental/2+1.2*GruesoPlastico >,
        < +LadoFundamental/2-1.2*GruesoPlastico, 0.8*GruesoPlastico,
          +LadoFundamental/2-1.2*GruesoPlastico >
      }
      cylinder                        {
        -GruesoPlastico*y, GruesoPlastico*y,
        CurvaturaFundamental/2-1.2*GruesoPlastico
      }
      material                          {
        MaterialTablero 
        scale               LadoFundamental/2
      }
    }
    // END Tablero
    difference                          {
      union                             {
        intersection                      {
          box                             {
            < -LadoFundamental/2, 0, -LadoFundamental/2 >,
            < +LadoFundamental/2, 2*GruesoPlastico, +LadoFundamental/2 >
          }
          cylinder                        {
            -GruesoPlastico*y, 3*GruesoPlastico*y,
            CurvaturaFundamental/2
          }
        }
        intersection                      {
          #local F2                       = 0.95;
          box                             {
            < -F2*LadoFundamental/2, 0, -F2*LadoFundamental/2 >,
            < +F2*LadoFundamental/2, 4*GruesoPlastico, +F2*LadoFundamental/2 >
          }
          cylinder                        {
            -GruesoPlastico*y, 11*GruesoPlastico*y,
            0.925*CurvaturaFundamental/2
          }
        }
      }
      prism                             {
        -GruesoPlastico, +5*GruesoPlastico,
        4,
        <0.4,0>,
        <0,0.4>,
        <-0.4,0>,
        <0,-0.4>
        scale           <LadoFundamental/2,1,LadoFundamental/2>
      }
      intersection                      {
        box                             {
          < -LadoFundamental/2+GruesoPlastico, -GruesoPlastico,
            -LadoFundamental/2+GruesoPlastico >,
          < +LadoFundamental/2-GruesoPlastico, GruesoPlastico,
            +LadoFundamental/2-GruesoPlastico >
        }
        cylinder                        {
          -GruesoPlastico*y, 3*GruesoPlastico*y,
          CurvaturaFundamental/2-GruesoPlastico
        }
      }
      intersection                      {
        box                             {
          < -F2*LadoFundamental/2+GruesoPlastico, 0,
            -F2*LadoFundamental/2+GruesoPlastico >,
          < +F2*LadoFundamental/2-GruesoPlastico, 3*GruesoPlastico,
            +F2*LadoFundamental/2-GruesoPlastico >
        }
        cylinder                        {
          -GruesoPlastico*y, 11*GruesoPlastico*y,
          0.925*CurvaturaFundamental/2-GruesoPlastico
        }
      }
      #local C                          = 0;
      #while ( C < 4 )
        union                           {
          prism                         {
            cubic_spline
            -GruesoPlastico, 10*GruesoPlastico,
            8,
            < 0.55,-0.35 >,
            < 0.20, 0 >,
            < 0.55, 0.35 >,
            < 0.70, 0.25 >,
            < 0.70,-0.25 >,
            < 0.55,-0.35 >,
            < 0.20, 0 >,
            < 0.55, 0.35 >
          }
          scale                         < LadoFundamental/2, 1,
                                          LadoFundamental/2 >
          rotate                          90*C*y
        }
        #local C                        = C+1;
      #end  // #while ( C < 4 )
    }
    // Boton Central
    // Boton Movible
    union                               {
      difference                        {
        union                           {
          sphere                        {
            0, 0.220*LadoFundamental/2
          }
          box                           {
            < -0.225*LadoFundamental/2,0,-0.225*LadoFundamental/2>,
            < +0.225*LadoFundamental/2,-GruesoPlastico,
              +0.225*LadoFundamental/2>
          }
        }
        sphere                          {
          0, 0.220*LadoFundamental/2-GruesoPlastico
        }
        plane                           {
          y, -GruesoPlastico
        }
        translate          ( 5-3*FraccionPresionar )*GruesoPlastico*y
      }
      #local CizallaMetal               = 1.10;
      difference                        {
        prism                           {
          conic_sweep
          1, 1+CizallaMetal
          4,
          <-0.5,-0.5>,
          <-0.5,+0.5>,
          <+0.5,+0.5>,
          <+0.5,-0.5>
          translate                     -y
          scale         <0.150*LadoFundamental,
             -( 4-3*FraccionPresionar )*GruesoPlastico/CizallaMetal,0.150*LadoFundamental>
          translate             (1+4-3*FraccionPresionar )*GruesoPlastico*y
          rotate                        45*y
        }
        prism                           {
          conic_sweep
          1, 1+CizallaMetal
          4,
          <-0.5,-0.5>,
          <-0.5,+0.5>,
          <+0.5,+0.5>,
          <+0.5,-0.5>
          translate                     -y
          scale         <0.150*LadoFundamental,
             -( 4-3*FraccionPresionar )*GruesoPlastico/CizallaMetal,0.150*LadoFundamental>
          translate             (1+4-3*FraccionPresionar )*GruesoPlastico*y
          rotate                        45*y
          translate                     -GruesoMetal*y
        }
        box                             {
          <-1,-1,0>,
          <1,1,1>
          scale                 LadoFundamental
          translate             +0.125*LadoFundamental*z/2
          rotate                        45*y
        }
        box                             {
          <-1,-1,0>,
          <1,1,1>
          scale                 -LadoFundamental
          translate             -0.125*LadoFundamental*z/2
          rotate                        45*y
        }      
        material                        {
          MaterialMetalico
        }
      }
    }
    // END BotonMovible
    // DADO
    #declare JHSV_MacroTranslateDado   =
          ( 1+4-3*FraccionPresionar )*GruesoPlastico*y+TranslateDado
          +LadoDado/2*y;
    #if ( JHSV_DadoVisible )
      object                              {
        JHSV_Dado( LadoDado, MaterialDado )
        #switch ( NumeroDado )
          #case ( 5 )
            rotate                         90*y
          #case ( 3 )
            rotate                         90*y
          #case ( 2 )
            rotate                         90*y
          #case ( 4 )
            rotate                         90*x
          #break
          #case ( 6 )
            rotate                        180*x
          #break
        #end
        rotate            RotateDado
        translate                 JHSV_MacroTranslateDado
      }
    #end  // #if ( JHSV_DadoVisible )
    // END DADO
    #local FC                           = 0.075;
    #local CizallaBoton                 = 0.10;
    // Boton Fijo
    difference                          {
      union                             {
        prism                             {
          +GruesoPlastico, +5*GruesoPlastico,
          8,
          <0.4-FC,-FC>,
          <0.4-FC,+FC>,
          <+FC,0.4-FC>,
          <-FC,0.4-FC>,
          <-0.4+FC,+FC>,
          <-0.4+FC,-FC>,
          <-FC,-0.4+FC>,
          <+FC,-0.4+FC>
          scale           <LadoFundamental/2,1,LadoFundamental/2>
        }
        prism                             {
          conic_sweep
          1-CizallaBoton, 1.0,
          8,
          <0.4-FC,-FC>,
          <0.4-FC,+FC>,
          <+FC,0.4-FC>,
          <-FC,0.4-FC>,
          <-0.4+FC,+FC>,
          <-0.4+FC,-FC>,
          <-FC,-0.4+FC>,
          <+FC,-0.4+FC>
          translate               -y
          scale           <LadoFundamental/2,-2*GruesoPlastico/CizallaBoton,LadoFundamental/2>
          translate               5*GruesoPlastico*y
        }
      }
      // Resta Boton esferico
      cylinder                                  {
        -GruesoPlastico*y, +8*GruesoPlastico*y, 0.225*LadoFundamental/2
      }
      //
      union                             {
        prism                             {
          +GruesoPlastico, +5*GruesoPlastico,
          8,
          <0.4-FC,-FC>,
          <0.4-FC,+FC>,
          <+FC,0.4-FC>,
          <-FC,0.4-FC>,
          <-0.4+FC,+FC>,
          <-0.4+FC,-FC>,
          <-FC,-0.4+FC>,
          <+FC,-0.4+FC>
          scale           <LadoFundamental/2,1,LadoFundamental/2>
        }
        prism                             {
          conic_sweep
          1-CizallaBoton, 1.0,
          8,
          <0.4-FC,-FC>,
          <0.4-FC,+FC>,
          <+FC,0.4-FC>,
          <-FC,0.4-FC>,
          <-0.4+FC,+FC>,
          <-0.4+FC,-FC>,
          <-FC,-0.4+FC>,
          <+FC,-0.4+FC>
          translate               -0.999*y
          scale           <LadoFundamental/2-GruesoPlastico,-GruesoPlastico/CizallaBoton,LadoFundamental/2-GruesoPlastico>
          translate               5*GruesoPlastico*y
        }
      }
    }
    difference                          {
      // Fondo de prism decorativo
      union                               {
        #local C                          = 0;
        #while ( C < 4 )
          difference                      {
            prism                         {
              cubic_spline
              GruesoPlastico, 4*GruesoPlastico,
              8,
              < 0.55,-0.35 >,
              < 0.20, 0 >,
              < 0.55, 0.35 >,
              < 0.70, 0.25 >,
              < 0.70,-0.25 >,
              < 0.55,-0.35 >,
              < 0.20, 0 >,
              < 0.55, 0.35 >
            }
            prism                         {
              cubic_spline
              2*GruesoPlastico, 5*GruesoPlastico,
              8,
              < 0.55,-0.35 >,
              < 0.20, 0 >,
              < 0.55, 0.35 >,
              < 0.70, 0.25 >,
              < 0.70,-0.25 >,
              < 0.55,-0.35 >,
              < 0.20, 0 >,
              < 0.55, 0.35 >
              translate           -<0.30,0,0>
              scale                         < 1-2*GruesoPlastico/LadoFundamental, 1,
                                              1-2*GruesoPlastico/LadoFundamental >
              translate           +<0.30,0,0>
            }
            scale                         < LadoFundamental/2, 1,
                                            LadoFundamental/2 >
            rotate                          90*C*y
          }
          #local C                        = C+1;
        #end  // #while ( C < 4 )
      }
      // Fondo de prism decorativo
      prism                             {
        -GruesoPlastico, +6*GruesoPlastico,
        8,
        <0.4-FC,-FC>,
        <0.4-FC,+FC>,
        <+FC,0.4-FC>,
        <-FC,0.4-FC>,
        <-0.4+FC,+FC>,
        <-0.4+FC,-FC>,
        <-FC,-0.4+FC>,
        <+FC,-0.4+FC>
        scale           <LadoFundamental/2,1,LadoFundamental/2>
      }
    }  // difference para Fondo de prism decorativo
    // Ciclo cilindros
    union                               {
      #local C                          = 0;
      #local AnguloDDeg                 = 35/3;
      #local Exponente                  = 2;
      #while ( C < 4 )
        union                           {
          #local D                      = -3;
          #while ( D < 4 )
            #local AnguloDRad           = D*radians( AnguloDDeg );
            #local Cos2                 = cos( AnguloDRad );
            #local Sin2                 = sqrt( 1-Cos2*Cos2 );
            #local PivCosD              = pow( Cos2, Exponente )+
                                          pow( Sin2, Exponente );
            #local Radio                =
                     RadioJuego*LadoFundamental/pow( PivCosD, 1/Exponente );
            union                       {
              difference                  {
                cylinder                  {
                  0, AlturaPosicion*y,
                  DiametroPosicion/2
                }
                cylinder                  {
                  -AlturaPosicion*y, 2*AlturaPosicion*y,
                  DiametroPosicion/2-GruesoPlastico
                }
                #if ( D = -3 )
                  material { MaterialSalida }
                #end  // #if ( D = -3 )
              }
              JHSV_CilindroVisible( DiametroPosicion, AlturaPosicion,
                GruesoPlastico, C, JHSV_CasillaActiva, C*7+D+3 )
              translate Radio*< cos( AnguloDRad ), 0, -sin( AnguloDRad ) >
            }
            #local D                    = D+1;
          #end  // #while ( D < 4 )
          // Cilindros del fondo
          #local D                      = 0;
          #while ( D < 4 )
            union                       {
              difference                  {
                cylinder                  {
                  0, AlturaPosicion*y,
                  DiametroPosicion/2
                }
                cylinder                  {
                  -AlturaPosicion*y, 2*AlturaPosicion*y,
                  DiametroPosicion/2-GruesoPlastico
                }
              }
              JHSV_CilindroVisible( DiametroPosicion, AlturaPosicion,
                GruesoPlastico, C, JHSV_CasillaSalida, D )
              translate         0.41*CurvaturaFundamental*x
              rotate                    45*y
              rotate            ( D-1.5 )*10*y
            }
            #local D                    = D+1;
          #end  // #while ( D < 5 )
          // END Cilindros del Fondo
          // Cilindros Finales
          #local D                      = 0;
          #while ( D < 4 )
            union                       {
              difference                  {
                cylinder                  {
                  0, AlturaPosicion*y,
                  DiametroPosicion/2
                }
                cylinder                  {
                  -AlturaPosicion*y, 2*AlturaPosicion*y,
                  DiametroPosicion/2-GruesoPlastico
                }
                material                        {
                  MaterialMeta
                  scale         GruesoPlastico
                }
              }
              JHSV_CilindroVisible( DiametroPosicion, AlturaPosicion,
                GruesoPlastico, C, JHSV_CasillaMeta, D )
              translate         ( 0.320-0.16*D/3 )*CurvaturaFundamental*x
              rotate                    45*y
            }
            #local D                    = D+1;
          #end  // #while ( D < 5 )
          // END Cilindros Finales
          rotate                        C*90*y
        }
        #local C                        = C+1;
      #end  // #while ( C < 4 )
      translate         4*GruesoPlastico*y
    }
    material                            {
      MaterialPrincipal
    }
  }
#end  // #macro JHSV_JuegoHomeSigloVeintiuno

#macro JHSV_Home( Semilla, NumeroDado, FraccionPresionar )
  object                                {
    #local L                            = JHSV_L;
    #local LadoFundamental              = 187.0*L;
    #local CurvaturaFundamental         = 230.0*L;
    #local GruesoPlastico               =   1.7*L;
    #local GruesoMetal                  =   0.3*L;
    #local DiametroPosicion             =  11.0*L;
    #local AlturaPosicion               =   4.5*L;
    #local LadoDado                     =   7.0*L;
    #local S                            = seed( Semilla );
    #declare JHSV_MacroRotateDado       = < 0, 360*rand( S ), 0 >;
    #local SP                           = rand( S );
    #local SA                           = 360*rand( S );
    #declare JHSV_MacroTranslateDado    =
        < SP*cos( SA ),0, SP*sin(SA) >*0.070*LadoFundamental;
    JHSV_JuegoHomeSigloVeintiuno( NumeroDado, FraccionPresionar,
                LadoFundamental, CurvaturaFundamental,
                GruesoPlastico, GruesoMetal,
                DiametroPosicion, AlturaPosicion, LadoDado,
                JHSV_MacroRotateDado, JHSV_MacroTranslateDado,
                JHSV_MaterialPrincipal, JHSV_MaterialTablero,
                JHSV_MaterialMetalico, JHSV_MaterialDado,
                JHSV_MaterialSalida, JHSV_MaterialMeta )
  }
#end  // #macro JHSV_Home()
