// This file is licensed under the terms of the CC-LGPL.
//
// Autor  : Rafael Angel Campos Vargas
// Correo : RofoelCompos@hotmail.com  
// Apdo. 964-1250 Escazu, San Jose, Costa Rica
// dedicado a Jesus y Maria...
//
// En este momento estoy creando un programa
// de animacion LibreN3D para FreeDOS con el compilador FreePascal
// falta mucho... 22 febrero 2017
//                
// Mis dibujos estan en unidades L (Lisa)
// 1 L equivale a 0.055063 pulgadas 
// definido como 0.001 para POVRAY
//     
// MIHD MotocicletaImitacionHarleyDavidson
// El archivo dibuja una motocicleta Harley Davidson de juguete,
// segn modelo 2012 XL 1200V seventy-two.             
// Casi no permite modificaciones, aunque se pueden variar las texturas.
// Es compatible con POV-Ray 3.1.
//
// La macro tiene cuatro parametros.  
// #macro MIHD_Motocicleta( GiroFrontal, RodarFrontal, 
//                            RodarPosterior, BalancePosterior ) 
//      GiroFrontal             : Angulo de giro de la manivela frontal.
//      RodarFrontal            : Angulo de rotacion de la llanta delantera.
//      RodarPosterior          : Angulo de rotacion de la llanta posterior.
//      BalancePosterior        : Use angulos muy pequenos.
//              Permite levantar la llanta
//              posterior debido al peso y movimiento del aparato.
//              En este movimiento considere que el asiento permanece
//              estatico, y la llanta dejara de tocar el suelo.
//
// La motocicleta con parametros nulos se encuentra contenida completamente 
// entre las siguientes coordenadas:
//      Minimo          = < -54.4,   0.0, -20.4 >*L
//      Maximo          = < +30.3, +46.2, +20.4 >*L
//
// Estas son variables predefinidas o generadas en el
// interior de las macros que no deberian modificarse directamente.
// Si quiere cambiarlas, recurra a los modificadores.  
// #declare MIHD_DefaultTexturaReencauche        
//      Textura de las llantas.
// #declare MIHD_DefaultTexturaMetalica          
//      Textura principal de las partes metalicas.
// #declare MIHD_DefaultTexturaPresion         
//      Textura para presion delantera.
// #declare MIHD_DefaultTexturaPrincipal        
//      Textura principal de la motocicleta. 
// #declare MIHD_DefaultTexturaCuero
//      Textura de elementos de cuero.
// #declare MIHD_DefaultTexturaTubular
//      Textura de tubos de conduccion externos.
// #declare MIHD_DefaultTexturaFondoFocoPrincipal
//      Fondo del foco principal delantero.
// #declare MIHD_DefaultTexturaVidrioFocoPrincipal
//      Textura del vidrio del foco principal delantero.
// #declare MIHD_DefaultTexturaFondoFocoLateral
//      Fondo del foco lateral posterior.
// #declare MIHD_DefaultTexturaVidrioFocoLateral
//      Textura del vidrio de focos laterales traseros.
// #declare MIHD_DefaultTexturaDecorativaLateral
//      Textura decoracion laterales posteriores.
//
// Los siguientes son los modificadores
// aplicables y redefinibles por el usuario (si se indica es default)   
// #declare MIHD_TexturaReencauche                = texture { ... }
//      Textura de las llantas.
// #declare MIHD_TexturaMetalica                  = texture { ... }         
//      Textura principal de las partes metalicas.
// #declare MIHD_TexturaPresion                   = texture { ... }     
//      Textura para presion delantera.
// #declare MIHD_TexturaPrincipal                 = texture { ... }    
//      Textura principal de la motocicleta. 
// #declare MIHD_TexturaCuero                     = texture { ... }
//      Textura de elementos de cuero.
// #declare MIHD_TexturaTubular                   = texture { ... }
//      Textura de tubos de conduccion externos.
// #declare MIHD_TexturaFondoFocoPrincipal        = texture { ... }
//      Fondo del foco principal delantero.
// #declare MIHD_TexturaVidrioFocoPrincipal       = texture { ... }
//      Textura del vidrio del foco principal delantero.
// #declare MIHD_TexturaFondoFocoLateral          = texture { ... }
//      Fondo del foco lateral posterior.
// #declare MIHD_TexturaVidrioFocoLateral         = texture { ... }
//      Textura del vidrio de focos laterales traseros.
// #declare MIHD_TexturaDecorativaLateral         = texture { ... }
//      Textura decoracion laterales posteriores.
//        
// Para colocarlo simplemente
// #include "MIHD.inc" 
// object { MIHD_Motocicleta( 0, 0, 0, 0 ) }
//                                 
// Ejemplo con la macro detallada, mismo objeto default
// Con modificadores
// #include "MIHD.inc"
// #include "textures.inc"
// #declare MIHD_TexturaPrincipal     = { PinkAlabaster }
// object { MIHD_Motocicleta( 0, 0, 0, 0 ) }
//                                 
#include "colors.inc"      
#include "textures.inc"  
#include "stones.inc"    
#include "functions.inc"
#include "transforms.inc"

#declare MIHD_L         = 0.001;    
#local L                = MIHD_L;

#declare MIHD_DefaultTexturaReencauche      =
  texture                                 {  
    pigment                               {
      pavement        
      number_of_sides 4
      number_of_tiles 6
      pattern       34  
      color_map                           {
        [ 0 Gray05 ]
        [ 1 Gray10 ]
      }
    }
    normal                                {
      pavement        
      number_of_sides 4
      number_of_tiles 6
      pattern       34  
    }
    scale     MIHD_L
  }      
  
#ifndef ( MIHD_TexturaReencauche )
  #declare MIHD_TexturaReencauche       = MIHD_DefaultTexturaReencauche
#end  // #ifndef ( MIHD_TexturaReencauche )    

#declare MIHD_DefaultTexturaMetalica    =
  texture                               {
    Chrome_Texture 
    normal                              {
      granite
    }
  }               
  
#ifndef ( MIHD_TexturaMetalica )
  #declare MIHD_TexturaMetalica         = MIHD_DefaultTexturaMetalica
#end  // #ifndef ( MIHD_TexturaMetalica )  

#declare MIHD_DefaultTexturaPresion     =
  texture                               {
    pigment { Coral } 
  }               
  
#ifndef ( MIHD_TexturaPresion )
  #declare MIHD_TexturaPresion          = MIHD_DefaultTexturaPresion
#end  // #ifndef ( MIHD_TexturaPresion )  
 
#declare MIHD_DefaultTexturaPrincipal   =
  texture                               {
    pigment { Jade scale 10*MIHD_L }  
    normal { wrinkles } 
    finish                              {
      phong 0.9*3
      phong_size 60
      metallic
    }
  }               
  texture                               {
    pigment { color DarkGreen transmit 0.5 }  
    normal { wrinkles } 
    finish                              {
      phong 0.9*3  
      phong_size 60
      metallic
    }
  }               
  
#ifndef ( MIHD_TexturaPrincipal )
  #declare MIHD_TexturaPrincipal        = MIHD_DefaultTexturaPrincipal
#end  // #ifndef ( MIHD_TexturaPrincipal )  

#declare MIHD_DefaultTexturaCuero       =
  texture                               {
    Cork
  }               
  texture                               {
    pigment { color Black transmit 0.2 }
  }
  
#ifndef ( MIHD_TexturaCuero )
  #declare MIHD_TexturaCuero            = MIHD_DefaultTexturaCuero
#end  // #ifndef ( MIHD_TexturaCuero ) 

#declare MIHD_DefaultTexturaTubular     =
  texture                               {                           
    T_Grnt15
  }
  
#ifndef ( MIHD_TexturaTubular )        
  #declare MIHD_TexturaTubular          = MIHD_DefaultTexturaTubular
#end  // #ifndef ( MIHD_TexturaTubular )   
 
#declare MIHD_DefaultTexturaFondoFocoLateral      =
  texture                               {                           
    pigment { color Red*0.5+White*0.5 } 
  }
  
#ifndef ( MIHD_TexturaFondoFocoLateral )        
  #declare MIHD_TexturaFondoFocoLateral           = 
                  MIHD_DefaultTexturaFondoFocoLateral
#end  // #ifndef ( MIHD_TexturaFondoFocoLateral )   
        
#declare MIHD_DefaultTexturaVidrioFocoLateral     =
  texture                               {                           
    pigment { Red transmit 0.8 } 
  }
  
#ifndef ( MIHD_TexturaVidrioFocoLateral )        
  #declare MIHD_TexturaVidrioFocoLateral          = 
                  MIHD_DefaultTexturaVidrioFocoLateral
#end  // #ifndef ( MIHD_TexturaVidrioFocoLateral )   

#declare MIHD_DefaultTexturaFondoFocoPrincipal         =
  texture                               {            
    pigment { White }
  }      
  
#ifndef ( MIHD_TexturaFondoFocoPrincipal )        
  #declare MIHD_TexturaFondoFocoPrincipal              = 
                  MIHD_DefaultTexturaFondoFocoPrincipal
#end  // #ifndef ( MIHD_TexturaFondoFocoPrincipal )   
 
#declare MIHD_DefaultTexturaVidrioFocoPrincipal         =
  texture                               {            
    pigment { color White transmit 0.5 }
    normal                      {
      checker
      scale     0.0077/10
    }                               
  }      
  
#ifndef ( MIHD_TexturaVidrioFocoPrincipal )        
  #declare MIHD_TexturaVidrioFocoPrincipal              = 
                  MIHD_DefaultTexturaVidrioFocoPrincipal
#end  // #ifndef ( MIHD_TexturaVidrioFocoPrincipal )   

#declare MIHD_DefaultTexturaDecorativaLateral     =
  texture                       { 
    pigment { color Red*0.75+Coral*0.25 }
  } 
  
#ifndef ( MIHD_TexturaDecorativaLateral )        
  #declare MIHD_TexturaDecorativaLateral          =
                MIHD_DefaultTexturaDecorativaLateral
#end  // #ifndef ( MIHD_TexturaDecorativaLateral )            
             
#macro MIHD_IntersectionObjetos
          ( MinObjeto, MaxObjeto, Objeto1, Objeto2, Objeto3 )  
  #local Min1           = min_extent( Objeto1 );     
  #local Max1           = max_extent( Objeto1 );
  #local Min2           = min_extent( Objeto2 );     
  #local Max2           = max_extent( Objeto2 );
  #local Min3           = min_extent( Objeto3 );     
  #local Max3           = max_extent( Objeto3 );
  intersection                    {
    object                        {
      Objeto1                          
      translate                   -( Min1+Max1 )/2
      scale                       2/( Max1-Min1 )
    }
    object                        {
      Objeto2                          
      translate                   -( Min2+Max2 )/2
      scale                       2/( Max2-Min2 )
    }
    object                        {
      Objeto3                          
      translate                   -( Min3+Max3 )/2
      scale                       2/( Max3-Min3 )
    }
    scale                       ( MaxObjeto-MinObjeto )/2
    translate                   ( MaxObjeto+MinObjeto )/2
  }    
#end  // #macro MIHD_IntersectionObjetos 

#macro MIHD_RuedaDentada( TotalParesDientes )
  difference                                {                         
    #local AnguloFD                         = 90/TotalParesDientes;
    #local radio                            = tan( radians( AnguloFD ) );
    #local posX                             = 1/cos( radians( AnguloFD ) );
    union                                   {
      cylinder                                {
        -z, z, 1
      }
      #for ( C, 1, TotalParesDientes )
        cylinder                            {
          -z, z, radio
          translate                         posX*x
          rotate                            4*C*AnguloFD*z
        }
      #end  // #for ( C, 1, TotalParesDientes )
    }                                          
    #for ( C, 1, TotalParesDientes )
      cylinder                              {
        -1.01*z, 1.01*z, radio
        translate                           posX*x
        rotate                              2*( 2*C+1 )*AnguloFD*z
      }
    #end  // #for ( C, 1, TotalParesDientes )
  }
#end  // #macro MIHD_RuedaDentada
                   
// DATOS                                             
// LLANTA DELANTERA                                             
#declare MIHD_RadioLlantaDelantera            =  13.1*L;
#declare MIHD_RadioInteriorLlantaDelantera    =   8.8*L; 
#declare MIHD_DiametroEjeLlantaDelantera      =   5.0*L;      
#declare MIHD_DiametroVarilla                 =   0.5*L;
#declare MIHD_GruesoLlanta                    =   3.8*L;                                             
                              
                                             
#declare MIHD_LlantaDelantera       =
  union                             {
    #local L                        = MIHD_L;    
    // Reencauche    
    union                           {   
      torus                         {
        MIHD_RadioLlantaDelantera-MIHD_GruesoLlanta/2,
        MIHD_GruesoLlanta/2               
        rotate                      90*x
        texture                     {
          uv_mapping
          MIHD_TexturaReencauche 
          rotate    90*x
        }
      }
      torus                         {
        MIHD_RadioInteriorLlantaDelantera+MIHD_GruesoLlanta/2,
        MIHD_GruesoLlanta/2               
        rotate                      90*x
        texture                     {
          uv_mapping
          MIHD_TexturaReencauche 
          rotate    90*x
        }
      }
      torus                         {
        ( MIHD_RadioLlantaDelantera+MIHD_RadioInteriorLlantaDelantera )/2,
        ( MIHD_RadioLlantaDelantera-MIHD_RadioInteriorLlantaDelantera
                -MIHD_GruesoLlanta )/2 
        rotate                      90*x
        scale                     <1,1,0.05>
        translate                 -MIHD_GruesoLlanta/2*z
      } 
      torus                         {
        ( MIHD_RadioLlantaDelantera+MIHD_RadioInteriorLlantaDelantera )/2,
        ( MIHD_RadioLlantaDelantera-MIHD_RadioInteriorLlantaDelantera
                -MIHD_GruesoLlanta )/2 
        rotate                      90*x
        scale                     <1,1,0.05>
        translate                 +MIHD_GruesoLlanta/2*z
      } 
      difference                    {
        cylinder                    {
          -MIHD_GruesoLlanta/2*z, +MIHD_GruesoLlanta/2*z,
          MIHD_RadioLlantaDelantera-MIHD_GruesoLlanta/2
        }
        cylinder                    {                   
          -MIHD_GruesoLlanta*z, +MIHD_GruesoLlanta*z,
          MIHD_RadioInteriorLlantaDelantera+MIHD_GruesoLlanta/2
        }
      }
      // Aro Metalico Interior    
      union                         {
        cylinder                    {
          -MIHD_GruesoLlanta/4*z, +MIHD_GruesoLlanta/4*z,
          MIHD_DiametroEjeLlantaDelantera/2
        }   
        difference                  {
          torus                         {
            MIHD_RadioInteriorLlantaDelantera+0.95*MIHD_GruesoLlanta/2,
            MIHD_GruesoLlanta/2               
            rotate                      90*x
          }            
          cylinder                      {
            -MIHD_GruesoLlanta*z, +MIHD_GruesoLlanta*z,
            MIHD_RadioInteriorLlantaDelantera+0.95*MIHD_GruesoLlanta/2
            inverse
          }
        }  
        // Varillas    
        #local C                    = 0;
        #while ( C < 10 )                 
          #local D                  = 0;
          #while ( D < 2 )                
            cylinder                {   
              < 0, -MIHD_DiametroEjeLlantaDelantera/2+MIHD_DiametroVarilla/2, 
                    -MIHD_GruesoLlanta/4+MIHD_DiametroVarilla/2 >,
              < MIHD_RadioInteriorLlantaDelantera,
                -MIHD_DiametroEjeLlantaDelantera/2+MIHD_DiametroVarilla/2, 0 >,
              MIHD_DiametroVarilla/2
              rotate      7*z  
              #if ( D = 1 )        
                scale               <1,-1,-1>
              #end  // #if ( D = 1 )
              rotate                36*C*z
            }
            #local D                = D+1;
          #end  // #while ( D < 2 )
          #local C                  = C+1;
        #end  // #while ( C < 10 )
        // END Varillas
        texture                     {
          MIHD_TexturaMetalica
        }
      }
      // END Aro metalico Interior
      texture                       {
        MIHD_TexturaReencauche
        rotate          90*x
        scale     1
      }
    }
    // END Reencauche                                                            
  }
  
// CuerpoFrontal                             
#declare MIHD_AnguloInclinacionCuerpoFrontal  =    30;
#declare MIHD_SeparacionBarrasCuerpoFrontal   =   7.4*L;
#declare MIHD_DiametroBarrasCuerpoFrontal     =   2.0*L;                              
#declare MIHD_LargoBarrasCuerpoFrontal        =  30.4*L; 
#declare MIHD_AireFocoVertical                =   7.7*L;    
#declare MIHD_EjeGiro                         =  -3.0*L;                        
#declare MIHD_AlturaManivela                  =   7.1*L;
#declare MIHD_SeparacionManivela              =  17.0*L;
#declare MIHD_LargoManivela                   =  10.2*L;
#declare MIHD_AlturaContactoCuerpoFrontal     =  
              MIHD_RadioLlantaDelantera+( MIHD_LargoBarrasCuerpoFrontal
                            -MIHD_DiametroBarrasCuerpoFrontal/2 )   
                *cos( radians( MIHD_AnguloInclinacionCuerpoFrontal ) )
            +MIHD_EjeGiro*sin( radians( MIHD_AnguloInclinacionCuerpoFrontal ) );
     
     
#macro MIHD_CuerpoFrontal( GiroFrontal, RodarDelantera )
  union                             {
    object                          {
      MIHD_LlantaDelantera              
      rotate                        -RodarDelantera*z
    }  
    // Soporte Llanta
    union                           {   
      cylinder                      {
        -( MIHD_SeparacionBarrasCuerpoFrontal/2+
           MIHD_DiametroBarrasCuerpoFrontal/3 )*z, 
         ( MIHD_SeparacionBarrasCuerpoFrontal/2+
           MIHD_DiametroBarrasCuerpoFrontal/3 )*z,
         MIHD_DiametroBarrasCuerpoFrontal/3  
      }   
      object                        {
        MIHD_RuedaDentada( 20 )               
        scale             < MIHD_DiametroEjeLlantaDelantera,
                            MIHD_DiametroEjeLlantaDelantera,
                            MIHD_DiametroVarilla/2 > 
        translate     MIHD_GruesoLlanta/2*z                     
        texture  { MIHD_TexturaMetalica normal { granite } scale 0.1 }
      }   
      union                         {
        object                        {
          MIHD_RuedaDentada( 15 )      
        }
        cylinder                      {
          -1.25*z, 1.25*z, 0.25
        }  
        texture         {
          MIHD_TexturaMetalica
        }
        scale             < MIHD_DiametroEjeLlantaDelantera/2,
                            MIHD_DiametroEjeLlantaDelantera/2,
                            MIHD_DiametroVarilla/2 > 
        translate         ( 2*MIHD_DiametroEjeLlantaDelantera/3)*x                    
        rotate            +130*z
        translate     ( MIHD_GruesoLlanta/2+MIHD_DiametroVarilla )*z                     
      }
    }
    // Barras Paralelas   
    union                           {   
      difference                    {
        #local RadioProtectorLlanta   = 1.06*MIHD_RadioLlantaDelantera;
        sphere                      {
          0, RadioProtectorLlanta
        }
        difference                  {
          sphere                      {
            0, 0.95*RadioProtectorLlanta
          }          
          plane                       {
            z, -MIHD_SeparacionBarrasCuerpoFrontal/2+0.05*RadioProtectorLlanta
          }
          plane                       {
            -z, -MIHD_SeparacionBarrasCuerpoFrontal/2-0.05*RadioProtectorLlanta
          }            
        }  
        difference                  {
          sphere                      {
            0, 0.90*RadioProtectorLlanta
          }    
          box                       {
            <-MIHD_DiametroBarrasCuerpoFrontal,0,
            -MIHD_SeparacionBarrasCuerpoFrontal>,
            <+MIHD_DiametroBarrasCuerpoFrontal,MIHD_LargoBarrasCuerpoFrontal,
            +MIHD_SeparacionBarrasCuerpoFrontal>
          }      
        }
        plane                       {
          z, -MIHD_SeparacionBarrasCuerpoFrontal/2
        }
        plane                       {
          -z, -MIHD_SeparacionBarrasCuerpoFrontal/2
        }  
        plane                       {
          x, 0
          rotate          40*z
        }
        plane                       {
          -x, 0
          rotate          -40*z
        }
        texture                     {
          MIHD_TexturaPrincipal
        }
      }
      #local D                      = 0;
      #while ( D < 2 )        
        union                         {      
          cylinder                    {                                      
            MIHD_RadioInteriorLlantaDelantera*y,
            MIHD_LargoBarrasCuerpoFrontal*y,
            MIHD_DiametroBarrasCuerpoFrontal/2
            translate                 -MIHD_SeparacionBarrasCuerpoFrontal/2*z
          }
          cylinder                    {                                      
            0, MIHD_RadioInteriorLlantaDelantera*y,
            MIHD_DiametroBarrasCuerpoFrontal/3
            texture { MIHD_TexturaPresion }
            translate                 -MIHD_SeparacionBarrasCuerpoFrontal/2*z
          } 
          difference                    {
            cylinder                      {
              -MIHD_DiametroBarrasCuerpoFrontal/2*z, 
              +MIHD_DiametroBarrasCuerpoFrontal/2*z,
               MIHD_DiametroBarrasCuerpoFrontal/2  
            }   
            cylinder                      {
              -MIHD_DiametroBarrasCuerpoFrontal*z, 
              +MIHD_DiametroBarrasCuerpoFrontal*z,
               MIHD_DiametroBarrasCuerpoFrontal/3  
            }     
            translate                 -MIHD_SeparacionBarrasCuerpoFrontal/2*z
          }
          torus                       {
            MIHD_DiametroBarrasCuerpoFrontal/2,
            MIHD_DiametroBarrasCuerpoFrontal/4
            scale             <1,2,1>                     
            translate                 MIHD_LargoBarrasCuerpoFrontal*y
            translate                 -MIHD_SeparacionBarrasCuerpoFrontal/2*z            
          }
          torus                       {
            MIHD_DiametroBarrasCuerpoFrontal/2,
            MIHD_DiametroBarrasCuerpoFrontal/4
            scale             <1,2,1>                     
            translate                 ( MIHD_LargoBarrasCuerpoFrontal-
                                        MIHD_AireFocoVertical )*y
            translate                 -MIHD_SeparacionBarrasCuerpoFrontal/2*z            
          }
          torus                       {
            MIHD_DiametroBarrasCuerpoFrontal/2,
            MIHD_DiametroBarrasCuerpoFrontal/4
            scale             <1,2,1>                     
            translate                 ( MIHD_RadioLlantaDelantera+
                                        MIHD_DiametroBarrasCuerpoFrontal )*y
            translate                 -MIHD_SeparacionBarrasCuerpoFrontal/2*z            
          }
          torus                       {
            MIHD_DiametroBarrasCuerpoFrontal/2,
            MIHD_DiametroBarrasCuerpoFrontal/2
            scale             <1,0.2,1>                     
            translate                 ( MIHD_RadioInteriorLlantaDelantera )*y
            translate                 -MIHD_SeparacionBarrasCuerpoFrontal/2*z            
          }
          #if ( D = 1 )        
            scale                     <1,1,-1>
          #end  // #if ( D = 1 ) 
        } 
        #local D                    = D+1; 
      #end  // #while ( D < 2 )
    }
    // END Barras Paralelas     
    // Eje Giro    
    union                           {
      cylinder                      {   
        0, -MIHD_AireFocoVertical*y, MIHD_DiametroBarrasCuerpoFrontal/2
        translate                 MIHD_LargoBarrasCuerpoFrontal*y
        translate                   MIHD_EjeGiro*x
      }    
      difference                    {
        prism                         {
          cubic_spline
          -MIHD_AireFocoVertical, 0                       
          6,              
          <0,-MIHD_SeparacionBarrasCuerpoFrontal/2>,
          <MIHD_EjeGiro,0>,
          <0,+MIHD_SeparacionBarrasCuerpoFrontal/2>,
          <0,-MIHD_SeparacionBarrasCuerpoFrontal/2>,
          <MIHD_EjeGiro,0>,
          <0,+MIHD_SeparacionBarrasCuerpoFrontal/2>         
          translate       MIHD_LargoBarrasCuerpoFrontal*y
        }                              
        box                         {
          <-MIHD_LargoBarrasCuerpoFrontal,-MIHD_DiametroBarrasCuerpoFrontal/2,
           -MIHD_LargoBarrasCuerpoFrontal>,
          <+MIHD_LargoBarrasCuerpoFrontal,
           -MIHD_AireFocoVertical+MIHD_DiametroBarrasCuerpoFrontal/2,
           +MIHD_LargoBarrasCuerpoFrontal>
          translate       MIHD_LargoBarrasCuerpoFrontal*y   
        }
      }
    }
    // END Eje Giro 
    // Manivela     
    union                           {
      #local PuntoAlto              = < MIHD_EjeGiro, MIHD_AlturaManivela,
                                            -MIHD_SeparacionManivela/2 >;  
      #local PuntoExtremo           = PuntoAlto+
                <MIHD_EjeGiro,-0.02*MIHD_AlturaManivela,-MIHD_LargoManivela>;  
      #local PuntoMango             = 0.8*PuntoAlto+0.2*PuntoExtremo;                                              
      #local DM                     = 0.7*MIHD_DiametroBarrasCuerpoFrontal/2;                                      
      sphere_sweep                    {
        cubic_spline
        9,
        PuntoExtremo, DM,
        PuntoMango, DM,
        1.1*PuntoMango-0.1*PuntoExtremo, DM,
        PuntoAlto/2*<1,0.1,+1>, DM,
        <0,0,0>, DM,    
        PuntoAlto/2*<1,0.1,-1>, DM,
        ( 1.1*PuntoMango-0.1*PuntoExtremo )*<1,1,-1>, DM,
        PuntoMango*<1,1,-1>, DM,
        PuntoExtremo*<1,1,-1>, DM
      }       
      union                         {   
        box                         {
          <0,-DM,-2*DM>,
          <DM,+DM,+2*DM>
        }      
        box                         {
          <-2*DM,0,-2*DM>,
          <+2*DM,-2*DM,2*DM>
          rotate      45*y
        }
        difference                  {
          cylinder                  { 
            -DM*y,1.1*DM*y,2.5*DM
          }
          cylinder                  {
            1.05*DM*y, 1.2*DM*y, 2.0*DM
            texture                 {
              MIHD_TexturaCuero
            }
          }  
          translate     DM*x
        }
        rotate                      30*z
      } 
      #local DesFreno                 = <2.0,-1.0,-2.0>*DM;   
      #local D                      = 0;
      #while ( D < 2 )        
        union                       {     
          union                     { 
            sphere                  {
              PuntoExtremo+DesFreno, DM/2
            }
            cone                    {
              PuntoExtremo+DesFreno, DM/2,
              PuntoMango+DesFreno*0.5, DM/2
            }
            cone                    {
              PuntoMango+DesFreno*0.5, DM/2,
              1.05*PuntoMango+0.05*PuntoExtremo+DesFreno, DM
            }
            sphere                  {
              1.05*PuntoMango+0.05*PuntoExtremo+DesFreno, DM  
            }                                                
            sphere                    {
              PuntoExtremo+DesFreno+DM/2, DM
            } 
            cylinder                  {
              0, DesFreno*0.5, DM/2
              translate     1.05*PuntoMango+0.05*PuntoExtremo
            }       
            sphere                  {
              PuntoMango+DesFreno+DesFreno*<0,1,1>,1.3*DM
            }
            union                 { 
              cylinder                {
                PuntoMango+2*DesFreno,
                PuntoMango+DesFreno+DesFreno*<0,1,1>,
                1.5*DM
              }
              cylinder                {
                0, 0.0001*x, 1.4*DM   
                texture { MIHD_TexturaFondoFocoLateral }
                translate     PuntoMango+2*DesFreno
              } 
              sphere                  {
                0, 1.5*DM
                scale   <0.3,1,1>  
                texture { MIHD_TexturaVidrioFocoLateral }
                translate     PuntoMango+2*DesFreno
              }                
              translate   -PuntoMango-2*DesFreno      
              rotate          -MIHD_AnguloInclinacionCuerpoFrontal*z                       
              translate   +PuntoMango+2*DesFreno
            }
            texture                 {
              MIHD_TexturaMetalica
            }                          
          }
          cylinder                  {
            PuntoMango, PuntoExtremo, 1.05*DM
          }  
          cylinder                  {
            1.01*PuntoExtremo-0.01*PuntoMango,
            0.90*PuntoExtremo+0.10*PuntoMango,
            1.2*DM
          }                                
          cylinder                  {
            PuntoMango,
            0.10*PuntoExtremo+0.90*PuntoMango,
            1.2*DM
          }                                
          texture                   {
            MIHD_TexturaCuero
          }       
          #if ( D = 1 )        
            scale           <1,1,-1>
          #end  // #if ( D = 1 )
        }
        #local D                    = D+1;   
      #end  // #while ( D < 2 )
      translate                   ( MIHD_LargoBarrasCuerpoFrontal+
                                    2*DM )*y-DM*x                              
    }   
    // END Manivela 
    // Foco Principal    
    union                           {
      #local RadioFoco              = 0.75*min( MIHD_AireFocoVertical,
                                        MIHD_SeparacionBarrasCuerpoFrontal )/2;
      difference                    {
        sphere                      {
          0, RadioFoco
        }             
        plane                       {
          -x, 0        
          texture { MIHD_TexturaFondoFocoPrincipal }
        }                               
        translate         RadioFoco*x/2
        rotate        -30*z
        translate     ( MIHD_LargoBarrasCuerpoFrontal
                          -MIHD_AireFocoVertical/2 )*y
                          +RadioFoco*x/2                    
      }
      difference                    {
        sphere                      {
          0, RadioFoco
        }             
        plane                       {
          x, 0        
        } 
        texture     { MIHD_TexturaVidrioFocoPrincipal }                 
        scale           <0.4,1,1>
        translate         RadioFoco*x/2
        rotate        -30*z
        translate     ( MIHD_LargoBarrasCuerpoFrontal
                          -MIHD_AireFocoVertical/2 )*y
                          +0.999*RadioFoco*x/2                    
      }
    }
    // END Foco Principal
    texture                         {
      MIHD_TexturaMetalica
    }
    translate                 -MIHD_EjeGiro*x 
    translate           -( MIHD_LargoBarrasCuerpoFrontal
                            -MIHD_DiametroBarrasCuerpoFrontal/2 )*y                           
    // El origen ahora esta en el eje de giro plano superior de contacto
    // del CuerpoFrontal con el resto de la moto                        
    rotate                    GiroFrontal*y   
    rotate          +MIHD_AnguloInclinacionCuerpoFrontal*z         
    translate       MIHD_AlturaContactoCuerpoFrontal*y
  }  
#end  // #macro CuerpoFrontal()  

// DATOS                                             
// LLANTA POSTERIOR                                             
#declare MIHD_RadioLlantaPosterior            =  12.6*L;
#declare MIHD_RadioInteriorLlantaPosterior    =   7.4*L; 
#declare MIHD_DiametroEjeLlantaPosterior      =   4.0*L;      
                                                                           
#declare MIHD_LlantaPosterior       =
  union                             {
    #local L                        = MIHD_L;    
    // Reencauche    
    union                           {   
      torus                         {
        MIHD_RadioLlantaPosterior-MIHD_GruesoLlanta/2,
        MIHD_GruesoLlanta/2               
        rotate                      90*x
        texture                     {
          uv_mapping
          MIHD_TexturaReencauche 
          rotate    90*x
        }
      }
      torus                         {
        MIHD_RadioInteriorLlantaPosterior+MIHD_GruesoLlanta/2,
        MIHD_GruesoLlanta/2               
        rotate                      90*x
        texture                     {
          uv_mapping
          MIHD_TexturaReencauche 
          rotate    90*x
        }
      }
      torus                         {
        ( MIHD_RadioLlantaPosterior+MIHD_RadioInteriorLlantaPosterior )/2,
        ( MIHD_RadioLlantaPosterior-MIHD_RadioInteriorLlantaPosterior
                -MIHD_GruesoLlanta )/2 
        rotate                      90*x
        scale                     <1,1,0.05>
        translate                 -MIHD_GruesoLlanta/2*z
      } 
      torus                         {
        ( MIHD_RadioLlantaPosterior+MIHD_RadioInteriorLlantaPosterior )/2,
        ( MIHD_RadioLlantaPosterior-MIHD_RadioInteriorLlantaPosterior
                -MIHD_GruesoLlanta )/2 
        rotate                      90*x
        scale                     <1,1,0.05>
        translate                 +MIHD_GruesoLlanta/2*z
      } 
      difference                    {
        cylinder                    {
          -MIHD_GruesoLlanta/2*z, +MIHD_GruesoLlanta/2*z,
          MIHD_RadioLlantaPosterior-MIHD_GruesoLlanta/2
        }
        cylinder                    {                   
          -MIHD_GruesoLlanta*z, +MIHD_GruesoLlanta*z,
          MIHD_RadioInteriorLlantaPosterior+MIHD_GruesoLlanta/2
        }
      }
      // Aro Metalico Interior    
      union                         {
        cylinder                    {
          -MIHD_GruesoLlanta/4*z, +MIHD_GruesoLlanta/4*z,
          MIHD_DiametroEjeLlantaPosterior/2
        }   
        difference                  {
          torus                         {
            MIHD_RadioInteriorLlantaPosterior+0.95*MIHD_GruesoLlanta/2,
            MIHD_GruesoLlanta/2               
            rotate                      90*x
          }            
          cylinder                      {
            -MIHD_GruesoLlanta*z, +MIHD_GruesoLlanta*z,
            MIHD_RadioInteriorLlantaPosterior+0.95*MIHD_GruesoLlanta/2
            inverse
          }
        }  
        // Varillas    
        #local C                    = 0;
        #while ( C < 10 )                 
          #local D                  = 0;
          #while ( D < 2 )                
            cylinder                {   
              < 0, -MIHD_DiametroEjeLlantaPosterior/2+MIHD_DiametroVarilla/2, 
                    -MIHD_GruesoLlanta/4+MIHD_DiametroVarilla/2 >,
              < MIHD_RadioInteriorLlantaPosterior,
                -MIHD_DiametroEjeLlantaPosterior/2+MIHD_DiametroVarilla/2, 0 >,
              MIHD_DiametroVarilla/2
              rotate      7*z  
              #if ( D = 1 )        
                scale               <1,-1,-1>
              #end  // #if ( D = 1 )
              rotate                36*C*z
            }
            #local D                = D+1;
          #end  // #while ( D < 2 )
          #local C                  = C+1;
        #end  // #while ( C < 10 )
        // END Varillas
        texture                     {
          MIHD_TexturaMetalica
        }
      }
      // END Aro metalico Interior
      texture                       {
        MIHD_TexturaReencauche
        rotate          90*x
        scale     1
      }
    }
    // END Reencauche     
  }         
  
#declare MIHD_PosicionXLlantaPosterior  = -41.8*L;
#declare MIHD_DeltaGiroBalancePosterior =   
            +17.6*L*x+1.2*L*y;   
      
#declare MIHD_EPSPLINESoporteHumanoDelanteroMin     =
  < -18.0*L, MIHD_AlturaContactoCuerpoFrontal-7.9*L, -5.4*L >;

#declare MIHD_EPSPLINESoporteHumanoDelanteroMax     =      
  < +5*L, MIHD_AlturaContactoCuerpoFrontal, +5.4*L >;
    
#declare MIHD_Superellipsoid = superellipsoid { < 0.4, 0.4 > }   
      
#declare MIHD_EPSPLINESuperiorSoporteHumanoDelantero =            
prism { cubic_spline linear_sweep -1.000000, 1.000000
  , 17
  , <205.000000, 211.000000>
  , <243.000000, 217.000000>
  , <261.000000, 227.000000>
  , <272.000000, 231.000000>
  , <277.000000, 239.000000>
  , <269.000000, 247.000000>
  , <259.000000, 250.000000>
  , <242.000000, 259.000000>
  , <210.000000, 266.000000>
  , <188.000000, 267.000000>
  , <172.000000, 251.000000>
  , <139.000000, 239.000000>
  , <172.000000, 229.000000>
  , <187.000000, 210.000000>
  , <224.000000, 212.000000>
  , <243.000000, 217.000000>
  , <252.000000, 220.000000> 
  rotate          180*y
}   

#declare MIHD_EPSPLINELateralSoporteHumanoDelantero =            
prism { cubic_spline linear_sweep -1.000000, 1.000000
  , 19
  , <326.000000, 191.000000>
  , <345.000000, 182.000000>
  , <365.000000, 175.000000>
  , <383.000000, 177.000000>
  , <392.000000, 174.000000>
  , <402.000000, 168.000000>
  , <405.000000, 171.000000>
  , <400.000000, 181.000000>
  , <388.000000, 186.000000>
  , <381.000000, 193.000000>
  , <378.000000, 206.000000>
  , <358.000000, 212.000000>
  , <341.000000, 215.000000>
  , <324.000000, 216.000000>
  , <312.000000, 217.000000>
  , <303.000000, 212.000000>
  , <314.000000, 198.000000>
  , <345.000000, 182.000000>
  , <354.000000, 179.000000>   
  rotate            90*x
}             
   
#declare MIHD_EPSPLINEAsientoMin                    =
  MIHD_EPSPLINESoporteHumanoDelanteroMin*<1,1,0>-1.0*L*x-5.7*L*z
        -13.7*L*x-4.0*L*y;
#declare MIHD_EPSPLINEAsientoMax                    =  
  MIHD_EPSPLINESoporteHumanoDelanteroMin*<1,1,0>-1.0*L*x+5.7*L*z+2.0*L*y;
   
#declare MIHD_EPSPLINESuperiorAsiento =
prism { cubic_spline linear_sweep -1.000000, 1.000000
  , 16
  , <107.000000, 202.000000>
  , <132.000000, 210.000000>
  , <145.000000, 217.000000>
  , <157.000000, 224.000000>
  , <157.000000, 231.000000>
  , <157.000000, 240.000000>
  , <143.000000, 246.000000>
  , <125.000000, 255.000000>
  , <109.000000, 262.000000>
  , <94.000000, 254.000000>
  , <88.000000, 233.000000>
  , <95.000000, 210.000000>
  , <107.000000, 201.000000>
  , <122.000000, 204.000000>
  , <132.000000, 210.000000>
  , <138.000000, 214.000000>
}                 

#declare MIHD_EPSPLINELateralAsiento =
prism { cubic_spline linear_sweep -1.000000, 1.000000
  , 16
  , <255.000000, 212.000000>
  , <265.000000, 213.000000>
  , <282.000000, 211.000000>
  , <291.000000, 209.000000>
  , <293.000000, 215.000000>
  , <286.000000, 218.000000>
  , <267.000000, 227.000000>
  , <253.000000, 231.000000>
  , <244.000000, 229.000000>
  , <234.000000, 223.000000>
  , <231.000000, 212.000000>
  , <236.000000, 207.000000>
  , <244.000000, 207.000000>
  , <254.000000, 211.000000>
  , <265.000000, 213.000000>
  , <273.000000, 212.000000>   
  rotate        90*x
}

#declare MIHD_EPSPLINESoporteTuberiaMin     =
  < -35.0*L, MIHD_RadioLlantaPosterior+2.0*L, -2.7*L >;

#declare MIHD_EPSPLINESoporteTuberiaMax     =      
  < +5*L, MIHD_AlturaContactoCuerpoFrontal-2.0*L, +2.7*L >;
    
#declare MIHD_EPSPLINEFrontalSoporteTuberia =
prism { linear_spline linear_sweep -1.000000, 1.000000
  , 40
  , <461.000000, 181.000000>
  , <457.000000, 188.000000>
  , <460.000000, 195.000000>
  , <455.000000, 204.000000>
  , <461.000000, 204.000000>
  , <454.000000, 214.000000>
  , <458.000000, 215.000000>
  , <455.000000, 227.000000>
  , <459.000000, 233.000000>
  , <454.000000, 241.000000>
  , <459.000000, 241.000000>
  , <453.000000, 251.000000>
  , <457.000000, 254.000000>
  , <450.000000, 259.000000>
  , <460.000000, 262.000000>
  , <453.000000, 271.000000>
  , <455.000000, 274.000000>
  , <453.000000, 284.000000>
  , <459.000000, 288.000000>
  , <455.000000, 293.000000>
  , <461.000000, 296.000000>
  , <465.000000, 296.000000>
  , <473.000000, 290.000000>
  , <469.000000, 285.000000>
  , <472.000000, 278.000000>
  , <467.000000, 275.000000>
  , <471.000000, 265.000000>
  , <466.000000, 262.000000>
  , <470.000000, 251.000000>
  , <467.000000, 243.000000>
  , <472.000000, 232.000000>
  , <468.000000, 222.000000>
  , <471.000000, 213.000000>
  , <466.000000, 207.000000>
  , <475.000000, 198.000000>
  , <466.000000, 198.000000>
  , <473.000000, 190.000000>
  , <470.000000, 184.000000>
  , <467.000000, 181.000000>
  , <461.000000, 181.000000>   
  rotate        90*y
}

#declare MIHD_EPSPLINESuperiorSoporteTuberia =
prism { linear_spline linear_sweep -1.000000, 1.000000
  , 57
  , <230.000000, 129.000000>
  , <224.000000, 133.000000>
  , <228.000000, 137.000000>
  , <234.000000, 144.000000>
  , <237.000000, 135.000000>
  , <239.000000, 144.000000>
  , <246.000000, 138.000000>
  , <253.000000, 146.000000>
  , <256.000000, 139.000000>
  , <260.000000, 141.000000>
  , <263.000000, 143.000000>
  , <271.000000, 139.000000>
  , <278.000000, 140.000000>
  , <281.000000, 138.000000>
  , <287.000000, 141.000000>
  , <292.000000, 140.000000>
  , <297.000000, 145.000000>
  , <297.000000, 139.000000>
  , <303.000000, 143.000000>
  , <310.000000, 139.000000>
  , <314.000000, 143.000000>
  , <322.000000, 137.000000>
  , <325.000000, 142.000000>
  , <333.000000, 137.000000>
  , <344.000000, 144.000000>
  , <350.000000, 132.000000>
  , <360.000000, 145.000000>
  , <364.000000, 140.000000>
  , <371.000000, 143.000000>
  , <378.000000, 137.000000>
  , <384.000000, 145.000000>
  , <387.000000, 139.000000>
  , <397.000000, 145.000000>
  , <403.000000, 144.000000>
  , <407.000000, 137.000000>
  , <403.000000, 129.000000>
  , <396.000000, 123.000000>
  , <387.000000, 128.000000>
  , <378.000000, 124.000000>
  , <370.000000, 126.000000>
  , <356.000000, 123.000000>
  , <344.000000, 126.000000>
  , <336.000000, 122.000000>
  , <328.000000, 128.000000>
  , <320.000000, 121.000000>
  , <312.000000, 128.000000>
  , <309.000000, 124.000000>
  , <301.000000, 129.000000>
  , <293.000000, 122.000000>
  , <288.000000, 126.000000>
  , <280.000000, 122.000000>
  , <272.000000, 126.000000>
  , <265.000000, 120.000000>
  , <253.000000, 130.000000>
  , <248.000000, 121.000000>
  , <237.000000, 127.000000>
  , <230.000000, 129.000000>  
  rotate        -180*y
}

#declare MIHD_EPSPLINELateralSoporteTuberia =
prism { cubic_spline linear_sweep -1.000000, 1.000000
  , 33
  , <342.000000, 207.000000>
  , <366.000000, 199.000000>
  , <387.000000, 190.000000>
  , <404.000000, 182.000000>
  , <404.000000, 190.000000>
  , <392.000000, 200.000000>
  , <376.000000, 201.000000>
  , <365.000000, 211.000000>
  , <345.000000, 213.000000>
  , <332.000000, 222.000000>
  , <311.000000, 223.000000>
  , <290.000000, 239.000000>
  , <279.000000, 243.000000>
  , <278.000000, 255.000000>
  , <274.000000, 268.000000>
  , <276.000000, 279.000000>
  , <276.000000, 290.000000>
  , <266.000000, 296.000000>
  , <266.000000, 282.000000>
  , <267.000000, 275.000000>
  , <267.000000, 262.000000>
  , <253.000000, 252.000000>
  , <241.000000, 243.000000>
  , <227.000000, 245.000000>
  , <227.000000, 232.000000>
  , <236.000000, 235.000000>
  , <263.000000, 233.000000>
  , <276.000000, 226.000000>
  , <294.000000, 221.000000>
  , <312.000000, 216.000000>
  , <335.000000, 209.000000>
  , <366.000000, 199.000000>
  , <377.000000, 193.000000>  
  rotate          90*x
}  

#declare MIHD_EPSPLINEBajoAsientoMax                    =
  MIHD_EPSPLINESoporteHumanoDelanteroMin*<1,1,0>-1.5*L*x-5.7*L*z-1.0*L*y;
#declare MIHD_EPSPLINEBajoAsientoMin                    =  
  MIHD_EPSPLINEBajoAsientoMax*<1,1,0>+5.7*L*z-8.2*L*y-14.6*L*x;
   
#declare MIHD_EPSPLINESuperiorBajoAsiento =
prism { cubic_spline linear_sweep -1.000000, 1.000000
  , 14
  , <91.000000, 240.000000>
  , <91.000000, 222.000000>
  , <110.000000, 211.000000>
  , <124.000000, 213.000000>
  , <140.000000, 214.000000>
  , <158.000000, 225.000000>
  , <155.000000, 241.000000>
  , <158.000000, 254.000000>
  , <142.000000, 265.000000>
  , <127.000000, 267.000000>
  , <110.000000, 270.000000>
  , <93.000000, 259.000000>
  , <91.000000, 222.000000>
  , <102.000000, 216.000000>
}    

#declare MIHD_EPSPLINELateralBajoAsiento =
prism { cubic_spline linear_sweep -1.000000, 1.000000
  , 13
  , <251.000000, 235.000000>
  , <264.000000, 232.000000>
  , <287.000000, 227.000000>
  , <289.000000, 240.000000>
  , <287.000000, 252.000000>
  , <278.000000, 260.000000>
  , <270.000000, 257.000000>
  , <257.000000, 252.000000>
  , <243.000000, 248.000000>
  , <241.000000, 241.000000>
  , <254.000000, 233.000000>
  , <264.000000, 232.000000>
  , <276.000000, 228.000000> 
  rotate            90*x
} 
  
#declare MIHD_EPSPLINEBajosMin                    =
  < -24.4*L, 10.0*L, -5.5*L >;
#declare MIHD_EPSPLINEBajosMax                    =  
  <  -2.7*L, 18.5*L, +7.9*L >;
     
#declare MIHD_EPSPLINESuperiorBajos =
prism { cubic_spline linear_sweep -1.000000, 1.000000
  , 29
  , <408.000000, 151.000000>
  , <426.000000, 153.000000>
  , <447.000000, 149.000000>
  , <457.000000, 154.000000>
  , <478.000000, 153.000000>
  , <492.000000, 144.000000>
  , <491.000000, 128.000000>
  , <486.000000, 112.000000>
  , <480.000000, 100.000000>
  , <465.000000, 109.000000>
  , <455.000000, 92.000000>
  , <453.000000, 101.000000>
  , <443.000000, 94.000000>
  , <436.000000, 97.000000>
  , <431.000000, 92.000000>
  , <427.000000, 102.000000>
  , <421.000000, 91.000000>
  , <417.000000, 97.000000>
  , <408.000000, 94.000000>
  , <403.000000, 100.000000>
  , <395.000000, 94.000000>
  , <385.000000, 102.000000>
  , <386.000000, 119.000000>
  , <378.000000, 134.000000>
  , <380.000000, 143.000000>
  , <390.000000, 152.000000>
  , <413.000000, 153.000000>
  , <426.000000, 153.000000>
  , <441.000000, 144.000000>  
  rotate            90*x
}                       

#declare MIHD_EPSPLINEFrontalBajos =
prism { cubic_spline linear_sweep -1.000000, 1.000000
  , 20
  , <535.000000, 315.000000>
  , <511.000000, 315.000000>
  , <506.000000, 304.000000>
  , <505.000000, 290.000000>
  , <506.000000, 278.000000>
  , <517.000000, 260.000000>
  , <528.000000, 262.000000>
  , <529.000000, 270.000000>
  , <529.000000, 277.000000>
  , <536.000000, 281.000000>
  , <541.000000, 271.000000>
  , <563.000000, 282.000000>
  , <557.000000, 286.000000>
  , <557.000000, 291.000000>
  , <561.000000, 294.000000>
  , <545.000000, 296.000000>
  , <540.000000, 306.000000>
  , <530.000000, 317.000000>
  , <511.000000, 315.000000>
  , <510.000000, 308.000000>
  rotate       90*y
}

#declare MIHD_EPSPLINELateralBajos =
prism { cubic_spline linear_sweep -1.000000, 1.000000
  , 21
  , <336.000000, 275.000000>
  , <348.000000, 277.000000>
  , <358.000000, 275.000000>
  , <374.000000, 269.000000>
  , <387.000000, 272.000000>
  , <386.000000, 297.000000>
  , <377.000000, 305.000000>
  , <385.000000, 311.000000>
  , <378.000000, 315.000000>
  , <349.000000, 316.000000>
  , <316.000000, 315.000000>
  , <298.000000, 313.000000>
  , <292.000000, 304.000000>
  , <280.000000, 296.000000>
  , <289.000000, 290.000000>
  , <299.000000, 277.000000>
  , <312.000000, 267.000000>
  , <317.000000, 279.000000>
  , <327.000000, 274.000000>
  , <348.000000, 277.000000>
  , <360.000000, 277.000000>
  rotate          180*y
}
  
#macro MIHD_CuerpoPosterior( RodarPosterior, BalancePosterior )
  union                           {
    // Balance Posterior                       
    union                         {                             
      object                      {
        MIHD_LlantaPosterior
        rotate            -RodarPosterior*z
      }     
      cylinder                    {                              
        -MIHD_GruesoLlanta*z, MIHD_GruesoLlanta*z, 
        2*MIHD_DiametroVarilla
        texture                   {
          MIHD_TexturaMetalica
        }
      }   
      cylinder                    {                              
        -MIHD_GruesoLlanta*z, MIHD_GruesoLlanta*z, 
        2*MIHD_DiametroVarilla
        texture                   {
          MIHD_TexturaTubular
        }
        translate                 +MIHD_DeltaGiroBalancePosterior
      }   
      // Simetrico Soporte Llanta Posterior
      #local C                      = 0;
      #while ( C < 2 )
        union                           {    
          cylinder                      {
            0, MIHD_DeltaGiroBalancePosterior, MIHD_DiametroVarilla
            translate       -MIHD_GruesoLlanta*z
            texture                     {
              MIHD_TexturaTubular
            }
          }                       
          cylinder                      {
            0, MIHD_DeltaGiroBalancePosterior, MIHD_DiametroVarilla    
            translate         -MIHD_DeltaGiroBalancePosterior
            rotate          -10*z
            translate         +MIHD_DeltaGiroBalancePosterior
            translate       -MIHD_GruesoLlanta*z
            texture                     {
              MIHD_TexturaTubular
            }
          }
          object                          {  
            MIHD_RuedaDentada( 20 )       
            scale     <6,6,1>*MIHD_DiametroVarilla
            translate       -MIHD_GruesoLlanta*z
            texture                     {
              MIHD_TexturaTubular
            }
          }                       
          #if ( C = 1 )        
            scale       <1,1,-1>
          #end  // #if ( C = 1 )
        }  
        #local C                      = C+1;
      #end  // #while ( C < 2 ) 
      // Llanta Posterior Vista Derrecha   
      union                         {     
        union                       {
          cylinder                      {
            0, MIHD_DeltaGiroBalancePosterior, MIHD_DiametroVarilla    
            translate         -MIHD_DeltaGiroBalancePosterior
            translate         +6*MIHD_DiametroVarilla*y
            rotate          -10*z
            translate         +MIHD_DeltaGiroBalancePosterior
            translate       -MIHD_GruesoLlanta*z
            texture                     {
              MIHD_TexturaTubular
            }
          }                          
        }  
        torus                       {
          12*MIHD_DiametroVarilla, 0.95*MIHD_DiametroVarilla
          rotate              90*x
          translate       -MIHD_GruesoLlanta*z   
          texture                 { MIHD_TexturaMetalica }
        }
      }
      // END Llanta Posterior Vista Derecha   
      // Llanta Posterior Vista Izquierda    
      union                       {
        cylinder                      {
          -( MIHD_SeparacionBarrasCuerpoFrontal/2+
             MIHD_DiametroBarrasCuerpoFrontal/3 )*z, 
           ( MIHD_SeparacionBarrasCuerpoFrontal/2+
             MIHD_DiametroBarrasCuerpoFrontal/3 )*z,
           MIHD_DiametroBarrasCuerpoFrontal/3  
        }   
        object                        {
          MIHD_RuedaDentada( 20 )               
          scale             < MIHD_DiametroEjeLlantaDelantera,
                              MIHD_DiametroEjeLlantaDelantera,
                              MIHD_DiametroVarilla/2 > 
          translate     MIHD_GruesoLlanta/2*z                     
          rotate            -RodarPosterior*z
          texture  { MIHD_TexturaMetalica normal { granite } scale 0.1 }
        }   
        union                         {
          object                        {
            MIHD_RuedaDentada( 15 )      
          }
          cylinder                      {
            -1.25*z, 1.25*z, 0.25
          }  
          texture         {
            MIHD_TexturaMetalica
          }
          scale             < MIHD_DiametroEjeLlantaDelantera/2,
                              MIHD_DiametroEjeLlantaDelantera/2,
                              MIHD_DiametroVarilla/2 > 
          translate         ( 2*MIHD_DiametroEjeLlantaDelantera/3)*x                    
          rotate            +50*z
          translate     ( MIHD_GruesoLlanta+MIHD_DiametroVarilla )*z                     
        }
      }
      // END Llanta Posterior Vista Izquierda   
      #local TransformRuedaPosterior  = 
        transform {
          translate                     -MIHD_DeltaGiroBalancePosterior
          rotate                        -BalancePosterior*z
          translate                     +MIHD_DeltaGiroBalancePosterior
          translate                     +MIHD_PosicionXLlantaPosterior*x
          translate                     +MIHD_RadioLlantaPosterior*y  
        }
      #local CentroRuedaPosterior     = < 0, 0, 0 >;  
      #local CentroRuedaPosterior     =
        vtransform( CentroRuedaPosterior, TransformRuedaPosterior );
      transform         { TransformRuedaPosterior }
    }               
    // Balance Rueda Posterior    
    union                             {         
      #local ExtremoBalance           =
          < 0.85*MIHD_PosicionXLlantaPosterior, 2.0*MIHD_RadioLlantaPosterior, 0 >;
      #local C                        = 0;
      #while ( C < 2 )        
        union                         {
          #local D                    = 0;
          #local TotalResorte         = 10;
          #local SepResorte           = 
            vlength( ExtremoBalance-CentroRuedaPosterior )/TotalResorte;
          #while ( D < TotalResorte )        
            torus                     {
              SepResorte/2, SepResorte/2
              rotate          -45*z
              translate         CentroRuedaPosterior+
                    D*( ExtremoBalance-CentroRuedaPosterior )/( TotalResorte-1 )
              texture { MIHD_TexturaMetalica }                                   
              translate         -0.75*MIHD_GruesoLlanta*z
            }
            #local D                  = D+1;
          #end  // #while ( D < TotalResorte )
          #if ( C = 1 )        
            scale                     <1,1,-1>
          #end  // #if ( C = 1 )
        }                    
        #local C                      = C+1;
      #end  // #while ( C < 2 )
    }
    // END Balance Rueda Posterior
    // Cuerpo Soporte Humano
    union                                     {
      difference                    {
        #local RadioProtectorLlanta   = 1.30*MIHD_RadioLlantaDelantera;
        sphere                      {
          0, RadioProtectorLlanta
        }
        difference                  {
          sphere                      {
            0, 0.95*RadioProtectorLlanta
          }          
          plane                       {
            z, -MIHD_SeparacionBarrasCuerpoFrontal/2+0.05*RadioProtectorLlanta
          }
          plane                       {
            -z, -MIHD_SeparacionBarrasCuerpoFrontal/2-0.05*RadioProtectorLlanta
          }            
        }  
        sphere                      {
          0, 0.85*RadioProtectorLlanta
        }    
        plane                       {
          z, -MIHD_SeparacionBarrasCuerpoFrontal/2
        }
        plane                       {
          -z, -MIHD_SeparacionBarrasCuerpoFrontal/2
        }  
        plane                       {
          x, 0
          rotate          40*z
        }
        plane                       {
          -x, 0
          rotate          -40*z
        }                                             
        translate                     +0.95*MIHD_PosicionXLlantaPosterior*x
        translate                     +MIHD_RadioLlantaPosterior*y
      }                 
      // Focos Posteriores    
      #local C                      = 0;
      #while ( C < 2 )        
        union                       {
          #local RadioFocoPosterior = 0.10*MIHD_RadioLlantaPosterior;                    
          box                       {
            < -0.5*MIHD_RadioLlantaPosterior, 0, -MIHD_GruesoLlanta/4 >,
            < +0.7*MIHD_RadioLlantaPosterior, 0.10*MIHD_RadioLlantaPosterior, 0 >
            matrix  <1,0,0,1,1,0,0,0,1,0,0,0>
          }
          box                       {
            < -0.0*MIHD_RadioLlantaPosterior, 0, -MIHD_GruesoLlanta/4 >,
            < +0.5*MIHD_RadioLlantaPosterior, -0.10*MIHD_RadioLlantaPosterior, 0 >
            texture { MIHD_TexturaDecorativaLateral }
            matrix  <1,0,0,1,1,0,0,0,1,0,0,0>
          }
          box                       {
            < -0.3*MIHD_RadioLlantaPosterior, 0, -4*RadioFocoPosterior >,
            < -0.3*MIHD_RadioLlantaPosterior-MIHD_GruesoLlanta/4, 0.10*MIHD_RadioLlantaPosterior, 0 >
          }
          box                       {
            < +0.3*MIHD_RadioLlantaPosterior, 0, -1.25*MIHD_GruesoLlanta >,
            < +0.3*MIHD_RadioLlantaPosterior-MIHD_GruesoLlanta/4, 0.10*MIHD_RadioLlantaPosterior, 0 >
          }
          union                     {
            cylinder                {
              0, 0.0001*x, 0.95*RadioFocoPosterior 
              texture { MIHD_TexturaFondoFocoLateral }
              translate         -2*RadioFocoPosterior*x
            } 
            sphere                  {
              0, RadioFocoPosterior
              scale   <0.3,1,1>
              texture { MIHD_TexturaVidrioFocoLateral }
              translate     -2*RadioFocoPosterior*x
            }
            cylinder                {
              0, -RadioFocoPosterior*x, RadioFocoPosterior
            }                                             
            sphere                    {
              -RadioFocoPosterior*x, RadioFocoPosterior
            }
            translate     -0.3*MIHD_RadioLlantaPosterior*x                              
            translate     RadioFocoPosterior*y
            translate     -3*RadioFocoPosterior*z
          }  
          translate               +0.95*MIHD_PosicionXLlantaPosterior*x
                                  +2*MIHD_RadioLlantaPosterior*y
                                  -MIHD_SeparacionBarrasCuerpoFrontal/2*z
          texture                   { MIHD_TexturaMetalica }
          #if ( C = 1 )        
            scale                   <1,1,-1>
          #end  // #if ( C = 1 )
        }
        #local C                    = C+1;
      #end  // #while ( C < 2 ) 
      // END Focos Posteriores
      // Soporte Humano Delantero      
      union                         {       
        object                      {
          MIHD_IntersectionObjetos
             ( MIHD_EPSPLINESoporteHumanoDelanteroMin,
               MIHD_EPSPLINESoporteHumanoDelanteroMax,
               MIHD_Superellipsoid,
               MIHD_EPSPLINESuperiorSoporteHumanoDelantero,
               MIHD_EPSPLINELateralSoporteHumanoDelantero ) 
        }      
        superellipsoid              {
          <0.3, 0.3>
          scale     < abs( MIHD_PosicionXLlantaPosterior )/4, 
                      MIHD_DiametroBarrasCuerpoFrontal/2,
                      MIHD_DiametroBarrasCuerpoFrontal >
          translate   MIHD_PosicionXLlantaPosterior/3*x
          translate   0.85*MIHD_AlturaContactoCuerpoFrontal*y            
          rotate          12*z             
        }
        object                      {
          MIHD_IntersectionObjetos
             ( MIHD_EPSPLINEAsientoMin,
               MIHD_EPSPLINEAsientoMax,
               MIHD_Superellipsoid,
               MIHD_EPSPLINESuperiorAsiento,
               MIHD_EPSPLINELateralAsiento ) 
          texture     { MIHD_TexturaCuero }     
        }      
      }                          
      // END Soporte Humano Delantero 
      // Soporte Tubular   
      union                                 {  
        object                        {
          MIHD_IntersectionObjetos
             ( MIHD_EPSPLINESoporteTuberiaMin,
               MIHD_EPSPLINESoporteTuberiaMax,
               MIHD_EPSPLINEFrontalSoporteTuberia,
               MIHD_EPSPLINESuperiorSoporteTuberia,
               MIHD_EPSPLINELateralSoporteTuberia ) 
        }
        object                              {
          MIHD_IntersectionObjetos
             ( MIHD_EPSPLINEBajoAsientoMin,
               MIHD_EPSPLINEBajoAsientoMax,
               MIHD_Superellipsoid,
               MIHD_EPSPLINESuperiorBajoAsiento,
               MIHD_EPSPLINELateralBajoAsiento ) 
        } 
        prism                               {
          cubic_spline
          MIHD_EPSPLINESoporteTuberiaMin.z/2,
          MIHD_EPSPLINESoporteTuberiaMax.z/2,
          6,
          <MIHD_EPSPLINEBajoAsientoMax.x,0.9*MIHD_EPSPLINEBajoAsientoMax.y>,
          <1.0*MIHD_EPSPLINEBajoAsientoMin.x,0.9*MIHD_EPSPLINEBajoAsientoMax.y>,
          <0.6*MIHD_EPSPLINEBajoAsientoMin.x,0.6*MIHD_EPSPLINEBajoAsientoMax.y>,
          <MIHD_EPSPLINEBajoAsientoMax.x,0.9*MIHD_EPSPLINEBajoAsientoMax.y>,
          <1.0*MIHD_EPSPLINEBajoAsientoMin.x,0.9*MIHD_EPSPLINEBajoAsientoMax.y>,
          <0.6*MIHD_EPSPLINEBajoAsientoMin.x,0.6*MIHD_EPSPLINEBajoAsientoMax.y>
          
          rotate                        -90*x
        }      
        texture     { MIHD_TexturaTubular }
      }
      // END Soporte Tubular   
      // Cilindrada y Bajos    
      union                               {  
        #local PC1                        =
           < abs( MIHD_PosicionXLlantaPosterior ), 
            MIHD_AlturaContactoCuerpoFrontal, 1.25*MIHD_GruesoLlanta >;
        #local PC                         = PC1*<1,1,0>;   
        #local DobleBloque                = <-0.2,0.50,0>*PC;
        union                             {
          #local C                        = 0;
          #while ( C < 2 )        
            union                         {  
              #local D                    = 0;
              #local TotalDobleBloque     = 10;
              #local CFMin                = 0.75;
              #while ( D < TotalDobleBloque )        
                superellipsoid            {
                  <0.4,0.4>
                  scale     <0.125,0,1>*PC1*( CFMin+D*( 1-CFMin )/TotalDobleBloque )+
                            0.5*y*PC1/TotalDobleBloque/1.7
                  #if ( D = 0 ) 
                    scale       1.05       
                    texture     { MIHD_TexturaTubular }
                  #end  // #if ( D = 0 )                                     
                  #if ( D = ( TotalDobleBloque-1 ) )        
                    scale       1.125
                    texture     { MIHD_TexturaTubular }
                  #end  // #if ( D = 0 )                                     
                  translate       0.5*y*PC1*D/TotalDobleBloque/2 
                }
                #local D                  = D+1;
              #end  // #while ( D < TotalDobleBloque )
              rotate                20*z
              translate            -0.080*x*PC1 
              #if ( C = 1 )        
                scale                   <-1,1,1>
              #end  // #if ( C = 1 )
            }   
            #local C                      = C+1;
          #end  // #while ( C < 2 )  
          translate                     DobleBloque
        }                               
        object                            {
          MIHD_IntersectionObjetos
             ( MIHD_EPSPLINEBajosMin,
               MIHD_EPSPLINEBajosMax,
               MIHD_EPSPLINEFrontalBajos,
               MIHD_EPSPLINESuperiorBajos,
               MIHD_EPSPLINELateralBajos ) 
        } 
        #local D                          = 0;
        #local TotalPlanoBajos            = 4;
        #while ( D < TotalPlanoBajos )
          superellipsoid                {
            <0.4,0.4>
            scale      ( MIHD_EPSPLINEBajosMax-MIHD_EPSPLINEBajosMin )*<0.3,0.01,0.01>
            translate     ( MIHD_EPSPLINEBajosMax+MIHD_EPSPLINEBajosMin )/2*x+
                          MIHD_EPSPLINEBajosMin*y+0.90*MIHD_EPSPLINEBajosMax*z
                          +D*( MIHD_EPSPLINEBajosMax-MIHD_EPSPLINEBajosMin )*y/( TotalPlanoBajos )
          }
          #local D                        = D+1;
        #end 
        julia_fractal{ 
          <-0.083,0.0,-0.83,-0.025>
          sqr              // Types: sqr  cube
          max_iteration 8
          precision 20     // 10...500
          texture{
            MIHD_TexturaTubular 
          } // end of texture          
          scale   <1,2,1>
          scale         PC1/10 
          translate     DobleBloque+0.250*y*PC1+0.175*x*PC1
        }
        julia_fractal{ 
          <-0.083,0.0,-0.83,-0.025>
          sqr              // Types: sqr  cube
          max_iteration 8
          precision 20     // 10...500
          texture{
            MIHD_TexturaTubular 
          } // end of texture 
          rotate    -45*z         
          scale   <1,1.5,1>
          scale         PC1/10 
          translate     <-0.575,0.5,0>*PC
        } 
        isosurface { //-------------------------------------------
        
          function{
           f_rounded_box(x,y,z,0.3,1,1,1)
           -(f_leopard(x*11,y*11,z*11)*0.25)
           }
          threshold 0
          contained_by {box {<-1,-1,-1>*1.35,<1,1,1>*1.35}}
          max_gradient 3.0
          accuracy 0.0001
         
          texture { MIHD_TexturaMetalica
                    finish { specular 0.3 
                             phong 0.3 reflection 0.07}
        	  }
          scale 0.75
          rotate <0,-50,0>
          rotate <-40,0,0>       
                  scale    <0.25,2.5,10>
                  rotate        -25*z
                  scale         PC1/10 
                  translate     <0.000,0.5,0>*PC
          
        } // end of isosurface -----------------------------------
        torus       {
          0.05*PC1.x, 0.05*PC1.x   
          rotate        90*x
          translate     DobleBloque-z*PC1+0.1*y*PC1
        }
        superellipsoid        {
          <0.4,0.4>
          scale         0.075*PC1.x*<0.7,1,0.7>
          translate     DobleBloque+z*PC1/2+0.1*y*PC1
        }   
        cylinder              {
          -2*y, 1.25*y, 0.5
          scale         0.075*PC1.x*<1,1,1>
          translate     DobleBloque+z*PC1/2+0.1*y*PC1
        }
        texture     { MIHD_TexturaMetalica }
      }
      // END Cilindrada y Bajos
      // Tubular    
      union                               {     
        #local C                          = 0;
        #while ( C < 2 ) 
          #local T0                       = < +0.100, 0.800, 1 >*PC1;                       
          #local T1                       = < -0.150, 0.275, 1 >*PC1;                       
          #local T2                       = < -0.550, 0.275, 1 >*PC1;                       
          #local T3                       = +MIHD_DeltaGiroBalancePosterior*x
                 +MIHD_PosicionXLlantaPosterior*x+MIHD_RadioLlantaPosterior*y
                      +PC1.z*z;             
          #local RTT                      = 0.015*PC1.x;            
          sphere_sweep                           {       
            cubic_spline,      
            9,
            T0*<1,1,-1>, RTT,
            T0*<1,1,0>, RTT,
            T0, RTT,
            T1, RTT,
            0.5*T1+0.5*T2, RTT,
            T2, RTT,
            T3, RTT,
            T3*<1,1,0>, RTT,
            T3*<1,1,-1>, RTT
            #if ( C = 1 )        
              scale             <1,1,-1>
            #end  // #if ( C = 1 )
          }
          #local C                        = C+1;
        #end  // #while ( C < 2 ) 
        #local C                         = 0;
        #while ( C < 2 )
          // Por dos muflas del mismo lado   
          #if ( C = 0 )
            #local MA0                       = < -0.050, 0.575, -1 >*PC1;                       
            #local MA1                       = < -0.000, 0.425, -1.75 >*PC1;                       
            #local MA2                       = < -0.050, 0.250, -1.75 >*PC1;                       
            #local MA3                       = < -0.650, 0.250, -1.75 >*PC1;  
          #else  // #else ( C = 0 )
            #local MA0                       = < -0.350, 0.575, -1 >*PC1;                       
            #local MA1                       = < -0.400, 0.475, -1.55 >*PC1;                       
            #local MA2                       = < -0.400, 0.350, -1.55 >*PC1;                       
            #local MA3                       = < -1.150, 0.350, -1.55 >*PC1;  
          #end   // #else ( C = 0 )
          union {
            sphere { MA0, 2.0*RTT }  
            sphere { MA2, 2.5*RTT }  
            sphere { MA1, 2.0*RTT }  
            texture { MIHD_TexturaMetalica }
          }
          difference                            {                    
            sphere_sweep                           {       
              cubic_spline,      
              8,
              MA0*<1,1,-1>, 1.5*RTT,
              MA0*<1,1,0>, 1.5*RTT,
              MA0, 1.5*RTT,
              MA1, 1.5*RTT,
              MA2, 2*RTT,
              0.5*MA2+0.5*MA3, 3*RTT,
              MA3, 4*RTT,
              -0.5*MA2+1.5*MA3, 4.5*RTT   
            }
            cone                          {
              0.5*MA2+0.5*MA3, 1*RTT,
              -0.5*MA2+1.5*MA3, 4.5*RTT
              texture { MIHD_TexturaTubular }
            }
            texture   { MIHD_TexturaMetalica }
          }      
          #local C                        = C+1;
        #end  // #while ( C < 2 )
        texture     { MIHD_TexturaTubular }
      }
      // END Tubular
      texture                       {
        MIHD_TexturaPrincipal
      }    
    }
    // END Cuerpo Soporte Humano        
    // END Balance Posterior
  }
#end  // #macro MIHD_CuerpoPosterior()      
  
#macro MIHD_Motocicleta( GiroFrontal, RodarFrontal, 
                            RodarPosterior, BalancePosterior )       
  union                                       {
    object                                    {
      MIHD_CuerpoFrontal( GiroFrontal, RodarFrontal )
    }
    object                                    {
      MIHD_CuerpoPosterior( RodarPosterior, BalancePosterior )
    }
    // El punto de referencia principal sera
    // MIHD_AlturaContacoCuerpoFrontal*y del que depende el resto
  }
#end  // #macro MIHD_Motocicleta()  
