// This file is licensed under the terms of the CC-LGPL.
//
// Autor  : Rafael Angel Campos Vargas
// Correo : RofoelCompos@hotmail.com  
// Apdo. 964-1250 Escazu, San Jose, Costa Rica
// dedicado a Jesus y Maria...
//
// En este momento estoy creando un programa
// de animacion LibreN3D para FreeDOS con el compilador FreePascal
// falta mucho... 28 noviembre 2016
//                
// Mis dibujos estan en unidades L (Lisa)
// 1 L equivale a 0.055063 pulgadas 
// definido como 0.001 para POVRAY
//     
// PCII PinzaCocinaImportadaInoxidable 
// El archivo dibuja una pinza para cocina.
// Es muy poco flexible. Es compatible con POVRay 3.1.
//
// La macro principal tiene tres parametros. Tomar en cuenta que la pinza no se dibuja
//      bien si utiliza todos los parametros nulos. Para cerrar completamente podria
//      utilizar los valores (0.0,1.0,0.3). 
// #macro PCII_Pinza( FraccionAbierto, FraccionSeguro, FraccionPaleta ) 
//      FraccionAbierto         : Fraccion para abrir la pinza, 0.0 para cerrado y
//              1.0 para completamente abierto.
//      FraccionSeguro          : La pinza original se asegura al desplazar
//              un poco la estructura principal hacia abajo y cerrar la boca.
//              Este parametro simula este desplazamiento, desde 0.0 hasta 1.0.
//      FraccionPaleta          : Dobla la pinza hacia adentro. Utilice 0.0 para
//              no doblar, y 1.0 para el maximo.
//
// La siguiente macro permite mayor versatilidad.
// #macro PCII_PinzaCocinaImportadaInoxidable(
//                FraccionAbierto, FraccionSeguro, FraccionPaleta,
//                AnguloAbierto, AnguloPaleta, LargoFundamental,
//                GruesoXFundamental, GruesoZFundamental,
//                GruesoMetal, EjeGiro, RadioOjo, 
//                LargoColgar, LargoAgarre, AnchoAgarre,
//                MaterialMetalico, MaterialAgarre )
//      FraccionAbierto         : Fraccion para abrir la pinza, 0.0 para cerrado y
//              1.0 para completamente abierto.
//      FraccionSeguro          : La pinza original se asegura desplazando
//              un poco la estructura principal hacia abajo y cerrando la boca.
//              Este parametro simula este desplazamiento, desde 0.0 hasta 1.0.
//      FraccionPaleta          : Dobla la pinza hacia adentro. Utilice 0.0 para
//              no doblar, y 1.0 para el maximo.
//      AnguloAbierto           : Angulo maximo de apertura de la pinza.
//              Valor por defecto 30.
//      AnguloPaleta            : Angulo maximo de cierre de las paletas.
//              Valor por defecto 17.5.
//      LargoFundamental        : Referencia para el largo Y de las pinzas.
//              Valor por defecto 145*L.
//      GruesoXFundamental      : Referencia para el grueso X de las pinzas.
//              Valor por defecto 6.35*L.
//      GruesoZFundamental      : Referencia para el grueso Z de las pinzas.
//              Valor por defecto 11.8*L.
//      GruesoMetal             : Grueso principal del metal utilizado.
//              Valor por defecto 1*L.
//      EjeGiro                 : Referencia para ubicar en Y el eje de giro.
//              Valor por defecto 9*L.
//      RadioOjo                : Radio del anillo para colgar la pinza.
//              Valor por defecto 6.25*L.
//      LargoColgar             : Aplica la base fija no movible con la pinza.
//              Valor por defecto 21.5*L.
//      LargoAgarre             : Largo de la paleta de agarre.
//              Valor por defecto 56.3*L.
//      AnchoAgarre             : Ancho en Z de la paleta de agarre.
//              Valor por defecto 40*L.                      
//      MaterialMetalico        : Material principal a aplicar al aparato.
//              Valor por defecto PCII_MaterialMetalico.
//      MaterialAgarre          : Material para la paleta de agarre.
//              Valor por defecto PCII_MaterialAgarre.
//
// La pinza completamente abierta con parametros (1,0,1) se ubica completamente
// en las siguientes coordenadas:      
//      Minimo  = < -43.5,-210.0, -25.5 >*L
//      Maximo  = <  43.5,   8.0,  25.5 >*L
// El anillo para colgar la pinza se encuentra centrado en el plano YZ. Cuelga
// con gravedad Y-, con los dos brazos a ambos lados del eje Y.     
//
// Estas son variables predefinidas o generadas en el
// interna de las macros que no deberian modificarse directamente.
// Si quiere cambiarlas, recurra a los modificadores.  
// #declare PCII_DefaultMaterialMetalico      
//      Material principal a aplicar al aparato.
// #declare PCII_DefaultMaterialAgarre           
//      Material para la paleta de agarre.
//
// Los siguientes son los modificadores
// aplicables y redefinibles por el usuario (si se indica es default)   
// #declare PCII_MaterialMetalico           = material { ... }
//      Material principal a aplicar al aparato.
// #declare PCII_MaterialAgarre             = material { ... }
//      Material para la paleta de agarre.
//        
// Para colocarlo simplemente
// #include "PCII.inc" 
// object { PCII_Pinza(1.0,0.0,1.0) }
//                                  
// Ejemplo con la macro de texturas, mismo objeto default
// #include "PCII.inc"   
// object                       {
//   PCII_PinzaCocinaImportadaInoxidable( 1.0, 0.0, 1.0,
//         30, 17.5, 0.145, 0.00635, 0.0118, 0.001, 0.009,
//         0.00625, 0.0215, 0.0563, 0.040,           
//         PCII_MaterialMetalico, PCII_MaterialAgarre )
// }
//        
// Con modificadores
// #include "PCII.inc"
// #include "textures.inc"
// #declare PCII_MaterialMetalico   = material { textire { PinkAlabaster } }
// object        { PCII_Pinza(1.0,0.0,1.0) }
// 

#include "colors.inc"

#declare OjoAbsoluto            = 256;
#declare VecesCentroAbsoluto    = 10;     
#declare L                      = 0.001;   
camera {
        location  <   0,  0, -( VecesCentroAbsoluto+1 )*OjoAbsoluto >*L
        direction z
        sky       y
        angle     38.74
//        look_at  <   0,  0, -VecesCentroAbsoluto*OjoAbsoluto >*L
}         
light_source    {
         < 500  , 500   , -3000 >*L
         White
}
background      {
         Gray50
}    

#include "PCII.inc"

#local FactorAcercamiento       = 8.0;

object                          {
  PCII_Pinza( 1.0, 0.0, 1.0 )      
  rotate                  90*z
  translate             -0.100*x    
  translate       -( VecesCentroAbsoluto+1 )*( FactorAcercamiento-1 )
                        /( FactorAcercamiento )*OjoAbsoluto*L*z    
}
