// This file is licensed under the terms of the CC-LGPL.
//
// Autor  : Rafael Angel Campos Vargas
// Correo : RofoelCompos@hotmail.com  
// Apdo. 964-1250 Escazu, San Jose, Costa Rica
// dedicado a Jesus y Maria...
//
// En este momento estoy creando un programa
// de animacion LibreN3D para FreeDOS con el compilador FreePascal
// falta mucho... 4 octubre 2016
//                
// Mis dibujos estan en unidades L (Lisa)
// 1 L equivale a 0.055063 pulgadas 
// definido como 0.001 para POVRAY
//      
// PCMC PlanchaCuidadoMaterialCaliente
// El archivo dibuja una plancha algo pasada de moda para uso domstico, 
// es muy poco flexible pero incluye una macro para modificar
// texturas y unas pocas caracteristicas. Es compatible con POV-Ray 3.1.
//
// La macro principal tiene dos parametros.   
// #macro PCMC_Plancha( AnguloNivel, BanderaLuz )   
//      AnguloNivel             : Gira la barrita para graduar la
//              temperatura un angulo en grados, positivo o negativo.
//      BanderaLuz              : "on" simula el encendido de una
//              luz indicadora. "off" la luz se encuentra apagada.               
//
// La siguiente macro permite mayor control de las caracteristicas.
// #macro PCMC_PlanchaCuidadoMaterialCaliente( AnguloNivel, BanderaLuz,
//                LargoBase, GruesoBaseCaliente, AltoMetalico,
//                AnguloAgarradera, AnchoBase, ExcesoLargoBase,
//                MaterialPrincipal, MaterialMetalico,
//                MaterialMetalicoCaliente, MaterialBaseAgarradera )
//      AnguloNivel             : Gira la barrita para graduar la
//              temperatura un angulo en grados, positivo o negativo.
//      BanderaLuz              : "on" simula el encendido de una
//              luz indicadora. "off" la luz se encuentra apagada. 
//      LargoBase               : Referencia para el largo en X de la plancha.
//              Valor por defecto 140*L.
//      GruesoBaseCaliente      : Grueso de la placa metalica en la base caliente.
//              Valor por defecto 7*L.
//      AltoMetalico            : Referencia para el alto de la base metalica principal
//              de la plancha. Valor por defecto 20*L.
//      AnguloAgarradera        : Inclinacion para la estructura superior de la plancha.
//              Valor por defecto 2.86.
//      AnchoBase               : Referencia para el ancho Z de la plancha.
//              Valor por defecto 84*L.
//      ExcesoLargoBase         : Referencia para el largo en X adicional total de la plancha
//              respecto a su base caliente. Valor por defecto 25*L.
//      MaterialPrincipal       : Material a aplicar a la estructura de la agarradera.
//              Valor por defecto PCMC_MaterialPrincipal.
//      MaterialMetalico        : Material principal para los elementos metalicos.
//              Valor por defecto PCMC_MaterialMetalico.
//      MaterialMetalicoCaliente        : Material para la base caliente inferior.
//              Valor por defecto PCMC_MaterialMetalicoCaliente.
//      MaterialBaseAgarradera          : Material a aplicar a la base de la agarradera.
//              Valor por defecto PCMC_MaterialBaseAgarradera.             
//      
// La plancha con sus valores por defecto se encuentra completamente contenida en:   
//      Punto Minimo    = <-95,  0,-45>*L
//      Punto Maximo    = < 73, 76, 45>*L 
// La plancha mira con el frente hacia X+ gravedad Y-, descansa parcialmente 
// centrada sobre el origen del plano XZ.
//
// Estas son variables predefinidas o generadas en el
// interior de las macros que no deberian modificarse directamente.
// Si quiere cambiarlas, recurra a los modificadores.  
// #declare PCMC_DefaultMaterialPrincipal       
//      Material a aplicar a la estructura de la agarradera.
// #declare PCMC_DefaultMaterialMetalico        
//      Material principal para los elementos metalicos.
// #declare PCMC_DefaultMaterialMetalicoCaliente        
//      Material para la base caliente inferior.
// #declare PCMC_DefaultMaterialBaseAgarradera          
//      Material a aplicar a la base de la agarradera.
//
// Los siguientes son los modificadores
// aplicables y redefinibles por el usuario (si se indica es default)   
// #declare PCMC_MaterialPrincipal                  = material { ... }      
//      Material a aplicar a la estructura de la agarradera.
// #declare PCMC_MaterialMetalico                   = material { ... } 
//      Material principal para los elementos metalicos.
// #declare PCMC_MaterialMetalicoCaliente           = material { ... } 
//      Material para la base caliente inferior.
// #declare PCMC_MaterialBaseAgarradera             = material { ... } 
//      Material a aplicar a la base de la agarradera.
// #declare PCMC_ColorNivel                         = Orange;
//      Color a aplicar a la barrita selectora de temperatura.
//        
// Para colocarlo simplemente
// #include "PCMC.inc" 
// object { PCMC_Plancha( 0, off ) }
//                                 
// Ejemplo con la macro detallada, mismo objeto default
// #include "PCMC.inc"   
// object                       {
//   PCMC_PlanchaCuidadoMaterialCaliente( 0, off, 0.140, 0.007, 0.020, 2.86, 0.084, 0.025,
//        PCMC_MaterialPrincipal, PCMC_MaterialMetalico,
//        PCMC_MaterialMetalicoCaliente, PCMC_MaterialBaseAgarradera )  
// }
//        
// Con modificadores
// #include "PCMC.inc"
// #include "textures.inc"
// #declare PCMC_MaterialPrincipal   = material { textures { PinkAlabaster } }
// object        { PCMC_Plancha( 0, off ) }
//
#declare PCMC_L                 = 0.001;
#include "metals.inc"  
#include "stones.inc"

#declare PCMC_ColorNivel        = Orange;

#declare PCMC_DefaultMaterialPrincipal          =
  material                      {
    texture                     { 
      T_Stone31               
      scale 5
      normal                {
        bumps
        turbulence          0.05
        scale               0.1
      }
      finish                {
        phong               0.1
        phong_size          20
        metallic
      }
    }
  }

#ifndef ( PCMC_MaterialPrincipal )
  #declare PCMC_MaterialPrincipal       = PCMC_DefaultMaterialPrincipal
#end  // #ifndef ( PCMC_MaterialPrincipal )
 
#declare PCMC_DefaultMaterialMetalico           =
  material                      {
    texture                     {
      T_Silver_2A
      normal                {
        bumps
        scale               0.025
      }
      finish                {
        phong               0.5
        phong_size          20
        roughness           0.1
        metallic
      }
    }
  }

#ifndef ( PCMC_MaterialMetalico )
  #declare PCMC_MaterialMetalico        = PCMC_DefaultMaterialMetalico
#end  // #ifndef ( PCMC_MaterialMetalico )

#declare PCMC_DefaultMaterialMetalicoCaliente   =
  material                      {
    texture                     {
      T_Chrome_1C
      normal                {
        gradient y
        scale               0.001
      }
      finish                {   
        phong               0.5
        phong_size          20
        roughness           0.01
        metallic
      }
    }
  }

#ifndef ( PCMC_MaterialMetalicoCaliente )
  #declare PCMC_MaterialMetalicoCaliente =
                PCMC_DefaultMaterialMetalicoCaliente
#end  // #ifndef ( PCMC_MaterialMetalicoCaliente )

#declare PCMC_DefaultMaterialBaseAgarradera   =
  material                      {
    texture                     {
      checker
      texture                   {
        T_Silver_5E
        normal                  {
          gradient z
          triangle_wave
          scale  0.01
        }
      }
      texture                   {
        pigment { Gray30 }
        normal { granite }
      }
      
      translate -0.5
      scale      <0.35,1,1>
    }
  }

#ifndef ( PCMC_MaterialBaseAgarradera )
  #declare PCMC_MaterialBaseAgarradera =
                PCMC_DefaultMaterialBaseAgarradera
#end  // #ifndef ( PCMC_MaterialBaseAgarradera )

#macro PCMC_PlanchaCuidadoMaterialCaliente( AnguloNivel, BanderaLuz,
                LargoBase, GruesoBaseCaliente, AltoMetalico,
                AnguloAgarradera, AnchoBase, ExcesoLargoBase,
                MaterialPrincipal, MaterialMetalico,
                MaterialMetalicoCaliente, MaterialBaseAgarradera )
  union                                         {
    #local HH                                 = 0.1;
    #local LargoInclinado                     =
      0.895*LargoBase/cos( radians( AnguloAgarradera ) );
    #local AnchoInclinado                     = 0.7*AnchoBase;
    #local BlobMango                            =
      blob {
         threshold 0.6
         component 1.000000, 0.595751, <0.000000, 0.000000, 0.000000>
         component 1.000000, 0.595751, <0.000000, 0.400000, 0.000000>
         component 1.000000, 0.595751, <0.000000, 0.800000, 0.000000>
         component 1.000000, 0.595751, <0.000000, 1.200000, 0.000000>
         component 2.000000, 0.595751, <-0.200000, 1.600000, 0.000000>
         component 2.000000, 0.595751, <-0.200000, 1.600000, 0.000000>
         component 2.000000, 0.521259, <-0.400000, 1.600000, 0.000000>
         component 2.000000, 0.521259, <-0.721553, 1.594329, 0.000000>
         component 2.000000, 0.521259, <-0.868456, 1.568405, 0.000000>
         component 2.000000, 0.521259, <-1.162262, 1.516557, 0.000000>
         component 2.000000, 0.521259, <-1.024000, 1.533840, 0.000000>
         component 2.000000, 0.521259, <-2.345927, 1.348523, 0.000000>
         component 2.000000, 0.521259, <-2.475747, 1.343730, 0.000000>
         component 2.000000, 0.521259, <-2.786836, 1.309165, 0.000000>
         component 2.000000, 0.521259, <-2.605367, 1.317806, 0.000000>
         component 2.000000, 0.521259, <-2.916456, 1.317806, 0.000000>
         component 2.000000, 0.521259, <-3.620726, 1.391258, 0.000000>
         component 1.000000, 0.595751, <-4.475060, 0.000000, 0.000000>
         component 1.000000, 0.595751, <-4.628640, 0.100000, 0.300000>
         component 1.000000, 0.595751, <-4.628640, 0.100000,-0.300000>
         component 1.000000, 0.595751, <-4.528640, 0.300000, 0.400000>
         component 1.000000, 0.595751, <-4.528640, 0.300000,-0.400000>
         component 1.000000, 0.595751, <-4.475060, 0.400000, 0.000000>
         component 1.000000, 0.595751, <-4.475060, 0.800000, 0.000000>
         component 1.000000, 0.595751, <-4.475060, 1.200000, 0.000000>
         component 1.000000, 0.521259, <-4.828640, 1.600000, 0.000000>
         component 2.000000, 0.521259, <-3.352844, 1.330768, 0.000000>
         component 2.000000, 0.521259, <-1.335089, 1.481992, 0.000000>
         component 2.000000, 0.521259, <-1.481992, 1.473350, 0.000000>
         component 2.000000, 0.521259, <-0.556897, 1.635443, 0.000000>
         component 2.000000, 0.521259, <-3.508389, 1.365333, 0.000000>
         component 1.000000, 0.521259, <-4.227783, 1.238819, 0.000000>
         component 2.000000, 0.595751, <0.253230, 0.217721, 0.000000>
         component 2.000000, 0.595751, <0.286985, 0.622785, 0.000000>
         component 2.000000, 0.521259, <-1.602971, 1.456068, 0.000000>
         component 2.000000, 0.521259, <-1.749874, 1.430143, 0.000000>
         component 2.000000, 0.521259, <-1.896776, 1.404220, 0.000000>
         component 2.000000, 0.521259, <-2.035038, 1.386937, 0.000000>
         component 2.000000, 0.521259, <-2.156017, 1.369654, 0.000000>
         component 2.000000, 0.521259, <-3.072000, 1.317806, 0.000000>
         component 2.000000, 0.521259, <-3.218903, 1.326447, 0.000000>
         component 2.000000, 0.521259, <-3.767629, 1.391258, 0.000000>
         component 2.000000, 0.521259, <-3.931815, 1.365333, 0.000000>
         component 1.000000, 0.521259, <-4.026870, 1.339409, 0.000000>
         scale <1.0,1,1.65>*<LargoBase,AltoMetalico,AnchoBase>/
                <0.140,0.020,0.084>
      }

    // Base
    union                                       {
      #local II                                 = 0.10;
      prism                                     {
        conic_sweep
        bezier_spline
        1, 1+II, 12
        <-LargoInclinado/2,-AnchoInclinado/2>,
        <-1.025*LargoInclinado/2,-AnchoInclinado/4>,
        <-1.025*LargoInclinado/2,+AnchoInclinado/4>,
        <-LargoInclinado/2,+AnchoInclinado/2>,
        <-LargoInclinado/2,+AnchoInclinado/2>,
        <+LargoInclinado/20,+1.1*AnchoInclinado/2>,
        <+LargoInclinado/4,+0.7*AnchoInclinado/2>,
        <+LargoInclinado/2,0>,
        <+LargoInclinado/2,0>,
        <+LargoInclinado/4,-0.7*AnchoInclinado/2>,
        <+LargoInclinado/20,-1.1*AnchoInclinado/2>,
        <-LargoInclinado/2,-AnchoInclinado/2>
        translate                               -y
        scale                           <1.00,-0.8*AltoMetalico/II,1.00>
        translate                                 -( 1.0-0.2 )*ExcesoLargoBase*x
        rotate                                    AnguloAgarradera*z
        translate                       ( AltoMetalico+GruesoBaseCaliente )*y
        translate 0.5*LargoBase*tan( radians( AnguloAgarradera ) )*y
        material                                {
          MaterialMetalico
        }
      }
      intersection                                {
        #local GG                                 =0.1;
        prism                                     {
          conic_sweep
          bezier_spline
          1, 1-GG, 24
          <-LargoBase/2-0.975*ExcesoLargoBase,+0>,
          <-LargoBase/2-0.975*ExcesoLargoBase,0.1*AnchoBase/2>,
          <-LargoBase/2-1.1*ExcesoLargoBase,0.85*AnchoBase/2>,
          <-LargoBase/2-ExcesoLargoBase,+0.9*AnchoBase/2>,
          <-LargoBase/2-ExcesoLargoBase,+0.9*AnchoBase/2>,
          <-LargoBase/2-0.9*ExcesoLargoBase,0.95*AnchoBase/2>,
          <-LargoBase/2-0.9*ExcesoLargoBase,0.95*AnchoBase/2>,
          <-LargoBase/2,+AnchoBase/2>,
          <-LargoBase/2,+AnchoBase/2>,
          <+LargoBase/20,+1.1*AnchoBase/2>,
          <+LargoBase/4,+0.7*AnchoBase/2>,
          <+LargoBase/2,0>,
          <+LargoBase/2,0>,
          <+LargoBase/4,-0.7*AnchoBase/2>,
          <+LargoBase/20,-1.1*AnchoBase/2>,
          <-LargoBase/2,-AnchoBase/2>,
          <-LargoBase/2,-AnchoBase/2>,
          <-LargoBase/2-0.9*ExcesoLargoBase,-0.95*AnchoBase/2>,
          <-LargoBase/2-0.9*ExcesoLargoBase,-0.95*AnchoBase/2>,
          <-LargoBase/2-ExcesoLargoBase,-0.9*AnchoBase/2>,
          <-LargoBase/2-ExcesoLargoBase,-0.9*AnchoBase/2>,
          <-LargoBase/2-1.1*ExcesoLargoBase,-0.85*AnchoBase/2>,
          <-LargoBase/2-0.975*ExcesoLargoBase,-0.1*AnchoBase/2>,
          <-LargoBase/2-0.975*ExcesoLargoBase,+0>
          translate                       -y
          scale           <1,-2*AltoMetalico/GG,1>
        }
        prism                                     {
          -AnchoBase, AnchoBase,5,
          < +LargoBase/2,0>,
          < +LargoBase/2-4*ExcesoLargoBase/5,
                  AltoMetalico+LargoBase*tan( radians( AnguloAgarradera ) ) >,
          < -LargoBase/2-4*ExcesoLargoBase/5, AltoMetalico>,
          < -LargoBase/2-ExcesoLargoBase, 0 >,
          < +LargoBase/2,0>
          rotate                                  -90*x
        }
  
        union                                     {
          superellipsoid                            {
            <0.8,0.8>
            scale   < LargoBase/2+ExcesoLargoBase/2,
                    AltoMetalico+1.2*LargoBase*tan( radians( AnguloAgarradera ) ),
                    AnchoBase/2 >
            translate       -ExcesoLargoBase/2*x
          }
          superellipsoid                            {
            <0.5,0.5>
            scale   < LargoBase/2,
                    AltoMetalico+0.2*LargoBase*tan( radians( AnguloAgarradera ) ),
                    AnchoBase/2 >
            translate       -ExcesoLargoBase*x
          }
        }
        material                                  {
          MaterialMetalico
        }
        translate                         GruesoBaseCaliente*y
      }
      // Base Caliente
      difference                                  {
        #local FF                                 = 0.04;
        prism                                     {
          bezier_spline
          conic_sweep
          1, 1+FF, 12
          <-LargoBase/2,-AnchoBase/2>,
          <-1.025*LargoBase/2,-AnchoBase/4>,
          <-1.025*LargoBase/2,+AnchoBase/4>,
          <-LargoBase/2,+AnchoBase/2>,
          <-LargoBase/2,+AnchoBase/2>,
          <+LargoBase/20,+1.1*AnchoBase/2>,
          <+LargoBase/4,+0.7*AnchoBase/2>,
          <+LargoBase/2,0>,
          <+LargoBase/2,0>,
          <+LargoBase/4,-0.7*AnchoBase/2>,
          <+LargoBase/20,-1.1*AnchoBase/2>,
          <-LargoBase/2,-AnchoBase/2>
          translate               -y
          scale                   <1,-GruesoBaseCaliente/FF,1>
          translate               1.001*GruesoBaseCaliente*y
        }
        difference                                {
          union                                   {
            cylinder                                {
              -z, z, 0.05
              rotate              65*y
            }
            cylinder                                {
              -z, z, 0.05
              rotate              -65*y
            }
          }
          box                                     {
            -1,1
            scale <1,1,0.01>
          }
          plane                                   {
            x, 0
          }
          scale           < 2*LargoBase, 4*GruesoBaseCaliente, 2*AnchoBase >
          translate               0.5*GruesoBaseCaliente*y
        }
        material                                  {
          MaterialMetalicoCaliente
        }
      }
      // END Base Caliente
    }
    // END Base
    // Agarradera
    union                                       {
      // Luz Agarradera
      sphere                                    {
        0, 1
        pigment { color Red transmit 0.95 }
        scale                   <1,1,0.25>*0.025*LargoBase
        rotate                  20*x
        translate               -0.35*LargoBase*x
        translate               1.0*AltoMetalico*y
        translate               -0.15*AnchoBase*z
      }
      #if ( BanderaLuz )
        sphere                                  {
          0, 0.95
          pigment { Red }
          finish { ambient 1.0 diffuse 0.0 }
          scale                   <1,1,0.25>*0.025*LargoBase
          rotate                  20*x
          translate               -0.35*LargoBase*x
          translate               1.0*AltoMetalico*y
          translate               -0.15*AnchoBase*z
        }
      #else 
        sphere                                  {
          0, 0.95
          pigment { color Red*0.1 transmit 0.5 }
          finish { ambient 1.0 diffuse 0.0 }
          scale                   <1,1,0.25>*0.025*LargoBase
          rotate                  20*x
          translate               -0.35*LargoBase*x
          translate               1.0*AltoMetalico*y
          translate               -0.15*AnchoBase*z
        }
      #end
      // END Luz Agarradera
      prism                                     {
        conic_sweep
        bezier_spline
        1, 1-HH, 12
        <-LargoInclinado/2,-AnchoInclinado/2>,
        <-1.025*LargoInclinado/2,-AnchoInclinado/4>,
        <-1.025*LargoInclinado/2,+AnchoInclinado/4>,
        <-LargoInclinado/2,+AnchoInclinado/2>,
        <-LargoInclinado/2,+AnchoInclinado/2>,
        <+LargoInclinado/20,+1.1*AnchoInclinado/2>,
        <+LargoInclinado/4,+0.7*AnchoInclinado/2>,
        <+LargoInclinado/2,0>,
        <+LargoInclinado/2,0>,
        <+LargoInclinado/4,-0.7*AnchoInclinado/2>,
        <+LargoInclinado/20,-1.1*AnchoInclinado/2>,
        <-LargoInclinado/2,-AnchoInclinado/2>
        translate                               -y
        scale                           <0.99,-0.5*AltoMetalico/HH,0.99>
        material                                {
          MaterialBaseAgarradera
          scale LargoBase
        }
      }
      object                                    {
        BlobMango
        scale  0.020
        translate               0.18*AltoMetalico*y
        translate               0.275*LargoBase*x
      }
      union                                     {
        difference                                {
          sphere                                    {
            0, 1
          }
          plane                                   {
            -y, 0
          }
        }
        box                                     {
          <0.00,0.00,-0.15>,
          <1.25,0.05,+0.15>
          pigment { PCMC_ColorNivel }
          rotate                ( AnguloNivel )*y
        }
        cylinder                                {
          0, 0.1*y 1
          material { MaterialMetalico scale 1/LargoBase }
        }
        scale                   <1,2,1>
        scale                   0.05*LargoBase
        translate               0.30*LargoBase*x
        translate               1.5*AltoMetalico*y
      }
      translate                                 -( 1.0-0.2 )*ExcesoLargoBase*x
      rotate                                    AnguloAgarradera*z
      translate                       ( AltoMetalico+GruesoBaseCaliente )*y
      translate 0.5*LargoBase*tan( radians( AnguloAgarradera ) )*y
    }
    // END Agarradera
    material                                    {
      MaterialPrincipal
    }
  }
#end  // #macro PCMC_PlanchaCuidadoMaterialCaliente()

#macro PCMC_Plancha( AnguloNivel, BanderaLuz )
  object                                        {
    #local L                                    = PCMC_L;
    #local LargoBase                            = 140*L;
    #local GruesoBaseCaliente                   =   7*L;
    #local AltoMetalico                         =  20*L;
    #local AnguloAgarraderaG                    = 2.86;
    #local AnchoBase                            =  84*L;
    #local ExcesoLargoBase                      =  25*L;
    PCMC_PlanchaCuidadoMaterialCaliente( AnguloNivel, BanderaLuz,
        LargoBase, GruesoBaseCaliente, AltoMetalico,
        AnguloAgarraderaG, AnchoBase, ExcesoLargoBase,
        PCMC_MaterialPrincipal, PCMC_MaterialMetalico,
        PCMC_MaterialMetalicoCaliente,
        PCMC_MaterialBaseAgarradera )
  }
#end  // #macro PCMC_Plancha( Nivel, BanderaLuz )
